// file: $isip/class/algo/Calculus/calc_04.cc
// version: $Id: calc_04.cc,v 1.8 2002/05/31 21:57:23 picone Exp $
//

// isip include files
//
#include "Calculus.h"

// method: sofSize
//
// arguments: none
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
long Calculus::sofSize() const {
  
  // total things common to all algorithms
  //

  // start with the space required for the algorithm name
  //
  long bytes = ALGO_MAP.elementSofSize();

  // add the space required for the implementation name
  //
  bytes += IMPL_MAP.elementSofSize();

  // add the space required for the computational mode name
  //
  bytes += CMODE_MAP.elementSofSize();

  // add the space required for common data elements
  //
  bytes += order_d.sofSize();

  // add the space required for the debug level
  //
  bytes += debug_level_d.sofSize();
  
  // check the algorithm: differentiation
  //
  if (algorithm_d == DIFFERENTIATION) {

    // check the implementation: regression, central difference,
    //  backward difference
    //
    if ((implementation_d == REGRESSION) ||
	(implementation_d == CENTRAL_DIFFERENCE) ||
	(implementation_d == BACKWARD_DIFFERENCE)) {

      // calculate space required for the window length
      //
      bytes += delta_win_d.sofSize();
    }

    // check the implementation: unknown
    //
    else {
      return Error::handle(name(), L"sofSize",
			   ERR_UNKIMP, __FILE__, __LINE__);
    }
  }

  // check the algorithm: differentiation
  //
  else if (algorithm_d == INTEGRATION) {
    return Error::handle(name(), L"sofSize",
			 ERR_UNSUPA, __FILE__, __LINE__);
  }      

  // algorithm: unknown
  //
  else {
    return Error::handle(name(), L"sofSize",
                         ERR_UNKALG, __FILE__, __LINE__);
  }  

  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a boolean value indicating status
//
// this method has the object write itself to an Sof file
//
boolean Calculus::write(Sof& sof_a, long tag_a, const String& name_a) const {

  // declare local variable
  //
  long obj_size;

  if (sof_a.isText()) {
    obj_size = Sof::ANY_SIZE;
  }
  else {
    obj_size = sofSize();
  }
  
  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }

  // write data and exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a boolean value indicating status
//
// this method branches on the algorithm name
//
boolean Calculus::writeData(Sof& sof_a, const String& pname_a) const {
  
  // declare local variables
  //
  boolean status = false;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm name
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (long)algorithm_d);
    
  // write the implementation
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (long)implementation_d);
    
  // write the computational mode
  //
  CMODE_MAP.writeElementData(sof_a, PARAM_CMODE, (long)cmode_d);
    
  // write the order
  //
  order_d.writeData(sof_a, PARAM_ORDER);

  // write debug level (from the base class)
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);

  // check known algorithms: invoke the proper lower-level method
  //
  if (algorithm_d == DIFFERENTIATION) {
    status = writeDataCommon(sof_a, pname_a);
  }

  // exit ungracefully
  //
  else {
    return Error::handle(name(), L"writeData",
			 ERR_UNKALG, __FILE__, __LINE__);
  }

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return status;
}

// method: writeDataCommon
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a boolean value indicating status
//
// this method writes data to a file for several algorithms that
// share the same parameters
//
boolean Calculus::writeDataCommon(Sof& sof_a,
				  const String& pname_a) const {

  // write delta_win
  //
  delta_win_d.writeData(sof_a, PARAM_DELTAWIN);

  // exit gracefully
  //
  return true;
}
