// file: $isip/class/algo/Energy/enrgy_04.cc
// version: $Id: enrgy_04.cc,v 1.12 2001/12/11 17:47:43 gao Exp $
//

// isip include files
//
#include "Energy.h"

// method: sofSize
//
// arguments: none 
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
long Energy::sofSize() const {

  // start with the space required for the algorithm name
  //
  long bytes = ALGO_MAP.elementSofSize();

  // add the space required for the implementation name
  //
  bytes += IMPL_MAP.elementSofSize();

  // add the space required for energy floor
  //
  bytes += floor_d.sofSize();

  // add the space required for the debug level
  //
  bytes += debug_level_d.sofSize();

  // add the space required for the ma_coef
  //
  bytes += ma_coef_d.sofSize();

  // add the space required for the ar.coef
  //  
  bytes += ar_coef_d.sofSize();
  
  // check the algorithm:
  //  at this point, there are no algorithm or implementation specific
  //  data to be accounted for
  //

  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a boolean value indicating status
//
// this method has the object write itself to an Sof file
//
boolean Energy::write(Sof& sof_a, long tag_a, const String& name_a) const {

  // declare local variable
  //
  long obj_size;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to be dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // set the size to be the size of the object written to the Sof file
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // write data and exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a boolean value indicating status
//
// this method branches on the algorithm name
//
boolean Energy::writeData(Sof& sof_a, const String& pname_a) const {
  
  // declare local variables
  //
  boolean status = false;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm name
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (long)algorithm_d);
    
  // write the implementation type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (long)implementation_d);
  
  // write the energy floor
  //
  floor_d.writeData(sof_a, PARAM_FLOOR);
  
  // write the debug level (from the base class)
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);

  // check known algorithms: SUM
  //
  if (algorithm_d == SUM) {
    status = writeDataSum(sof_a, pname_a);
  }

  // check known algorithms: FILTER
  //
  else if (algorithm_d == FILTER) {
    status = writeDataFilt(sof_a, pname_a);
  }

  // check unsupported algorithms
  //
  else {
    return Error::handle(name(), L"writeData",
			 ERR_UNKALG, __FILE__, __LINE__);
  }

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return status;
}

// method: writeDataSum
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a boolean value indicating status
//
// this method writes data to a file for the SUM algorithm. note there
// is no additional data that needs to be written.
//
boolean Energy::writeDataSum(Sof& sof_a, const String& pname_a) const {

  // exit gracefully
  //
  return true;
}

// method: writeDataFilt
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a boolean value indicating status
//
// this method writes data to a file for the FILTER algorithm
//
boolean Energy::writeDataFilt(Sof& sof_a, const String& pname_a) const {

  // write the moving average coefficients
  //
  ma_coef_d.writeData(sof_a, PARAM_MA_COEF);
  
  // write the autoregressive coefficients
  //
  ar_coef_d.writeData(sof_a, PARAM_AR_COEF);
  
  // exit gracefully
  //
  return true;
}
