// file: $isip/class/algo/FourierTransform/ft_04.cc
// version: $Id: ft_04.cc,v 1.10 2002/05/31 21:57:29 picone Exp $
//

// isip include files
//
#include "FourierTransform.h"

// method: sofSize
//
// arguments: none
//
// return: size of the object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
long FourierTransform::sofSize() const {
  
  // start with the space required for the algorithm name
  //
  long bytes = ALGO_MAP.elementSofSize();

  // add the space required for the implementation, direction, resolution names
  //
  bytes += IMPL_MAP.elementSofSize();
  bytes += DIRE_MAP.elementSofSize();
  bytes += OUTM_MAP.elementSofSize();
  bytes += RESO_MAP.elementSofSize();
  
  // add the space required for flag about user specified i/o length
  //
  bytes += ilen_d.sofSize();
  bytes += olen_d.sofSize();
  
  // add the space required for fourier transform order
  //
  bytes += order_d.sofSize();

  // add the space required for the normalization
  //
  bytes += NORM_MAP.elementSofSize();

  // check the algorithm:
  //  at this point, there are no algorithm or implementation specific
  //  data to be accounted for
  //

  // add the space required for the debug level
  //
  bytes += debug_level_d.sofSize();

  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a boolean value indicating status
//
// this method has the object write itself to an Sof file
//
boolean FourierTransform::write(Sof& sof_a, long tag_a,
				const String& name_a) const {

  // declare local variable
  //
  long obj_size;

  if (sof_a.isText()) {
    obj_size = Sof::ANY_SIZE;
  }
  else {
    obj_size = sofSize();
  }
  
  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }

  // write data and exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) textual parameter name
//
// return: logical error status
//
// this method branches on the algorithm name
//
boolean FourierTransform::writeData(Sof& sof_a, const String& pname_a) const {

  // declare local variables
  //
  boolean status = false;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm name
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (long)algorithm_d);
    
  // check known algorithms: invoke the proper lower-level method
  //
  if ((algorithm_d == DFT) || (algorithm_d == FFT) || (algorithm_d == DCT)) {
    status = writeDataCommon(sof_a, pname_a);
  }

  // check unsupported algorithms
  //
  else {
    return Error::handle(name(), L"writeData",
			 ERR_UNKALG, __FILE__, __LINE__);
  }

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);

  // exit gracefully
  //
  return status;

  return true;
}

// method: writeDataCommon
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a boolean value indicating status
//
// this method writes data to a file for several algorithms that
// share the same parameters
//
boolean FourierTransform::writeDataCommon(Sof& sof_a, const String& pname_a) const {

  // write the implementation, direction, resolution
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION, (long)implementation_d);
  DIRE_MAP.writeElementData(sof_a, PARAM_DIRECTION, (long)direction_d);
  OUTM_MAP.writeElementData(sof_a, PARAM_OTYPE, (long)otype_d);
  RESO_MAP.writeElementData(sof_a, PARAM_RESOLUTION, (long)resolution_d);
  
  // write the user specified i/o length
  //
  ilen_d.writeData(sof_a, PARAM_INPUT_LEN);
  olen_d.writeData(sof_a, PARAM_OUTPUT_LEN);
  
  // write the transform order
  //
  order_d.writeData(sof_a, PARAM_ORDER);
  
  // write the normalization
  //
  NORM_MAP.writeElementData(sof_a, PARAM_NORMALIZATION, (long)normalization_d);

  // write the debug level (from the base class)
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);

  // exit gracefully
  //
  return true;
}
