// file: $isip/class/math/matrix/MatrixUlong/MatrixUlong.h
// version: $Id: MatrixUlong.h,v 1.27 2001/04/23 22:57:27 peng Exp $ 
//

// make sure definitions are only made once
//
#ifndef ISIP_MATRIX_ULONG
#define ISIP_MATRIX_ULONG

// isip include files
//
#ifndef ISIP_MMATRIX
#include <MMatrix.h>
#endif

// MatrixUlong: a matrix of Ulong objects
//
class MatrixUlong : public MMatrix<Ulong, ulong> {

  //---------------------------------------------------------------------------
  //
  // public constants
  //
  //---------------------------------------------------------------------------
public:

  // define the class name
  //
  static const String CLASS_NAME;

  //----------------------------------------
  //
  // default values and arguments
  //
  //----------------------------------------  

  // default arguments to methods
  //
  
  // define the default value(s) of the class data
  //
  
  //----------------------------------------
  //
  // error codes
  //
  //----------------------------------------  

  static const long ERR = 24300;

  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:
  
  // static memory manager
  //
  static MemoryManager mgr_d;
  
  //---------------------------------------------------------------------------
  //
  // required public methods
  //
  //---------------------------------------------------------------------------
public:
 
  // method: name
  //
  static const String& name() {
    return CLASS_NAME;
  }

  // other static methods
  //
  static boolean diagnose(Integral::DEBUG debug_level);
  
  // setDebug methods
  //  these methods are inherited from the MMatrix template class
  //
  
  // method: debug
  //
  boolean debug(const unichar* msg) const {
    return MMatrix<Ulong, ulong>::debug(name(), &msg[0]);
  }

  // method: destructor
  //  the destructor does not do anything here since it inherits the
  //  destructor from the MMatrix template class
  //
  ~MatrixUlong() {}

  // method: default constructor
  //    
  MatrixUlong(long nrows = DEF_SIZE, long ncols = DEF_SIZE,
	      Integral::MTYPE type = Integral::DEF_MTYPE):
    MMatrix<Ulong, ulong>(nrows, ncols, type) {
  }

  // method: copy constructor
  //    
  MatrixUlong(const MatrixUlong& arg): MMatrix<Ulong, ulong>(arg) {
  }
  
  // assign methods:
  //  these methods are inherited from the MMatrix template class  
  //   

  // method: operator=
  //
  MatrixUlong& operator=(const MatrixUlong& matrix) {
    assign(matrix);
    return *this;
  }
    
  // i/o methods:
  //  the sofSize, readData and writeData methods are inherited from
  //  the MMatrix template class
  //

  // method: read
  //
  boolean read(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MMatrix<Ulong, ulong>::read(sof, tag, name);
  }

  // method: write
  //
  boolean write(Sof& sof, long tag, const String& name = CLASS_NAME) const {
    return MMatrix<Ulong, ulong>::write(sof, tag, name);
  }

  // equality methods:
  //  these methods are inherited from the MMatrix template class
  //

  // method: new
  //
  static void* operator new(size_t size) {
    return mgr_d.get();
  }

  // method: new[]
  //
  static void* operator new[](size_t size) {
    return mgr_d.getBlock(size);
  }

  // method: delete
  //
  static void operator delete(void* ptr) {
    mgr_d.release(ptr);
  }

  // method: delete[]
  //
  static void operator delete[](void* ptr) {
    mgr_d.releaseBlock(ptr);
  }

  // method: setGrowSize
  //
  static boolean setGrowSize(long grow_size) {
    return mgr_d.setGrow(grow_size);
  }

  // other memory management methods:
  //  the clear method is inherited from the MMatrix template class
  // 

  //---------------------------------------------------------------------------
  //
  // class-specific public methods:
  //  extensions to required methods  
  //
  //---------------------------------------------------------------------------

  // method: constructor
  //  this method constructs a matrix with a unichar* object
  //
  MatrixUlong(long nrows, long ncols, const unichar* str,
	      Integral::MTYPE type = Integral::DEF_MTYPE, const Char delim = DEF_DELIM) {
    assign(nrows, ncols, str, type, delim);
  }
  
  // method: operator=
  //  this method assigns a ulong value to the matrix
  //        
  MatrixUlong& operator=(ulong value) {
    assign(value);
    return *this;
  }
  
  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:

};

// end of include file
//
#endif
