// file: $isip/class/math/scalar/Byte/Byte.h
// version: $Id: Byte.h,v 1.37 2001/10/14 04:24:20 bahety Exp $
//

// make sure definitions are only made once
//
#ifndef ISIP_BYTE
#define ISIP_BYTE

// isip include files
//
#ifndef ISIP_SCALAR
#include <MScalar.h>
#endif

// Byte: a class that manages an 8-bit unsigned integer. this class mainly
// inherits the MScalar template functionality. we only need to add Sof file
// support and memory management
//
class Byte : public MScalar<byte, byte8> {

  //---------------------------------------------------------------------------
  //
  // public constants
  //
  //---------------------------------------------------------------------------
public:

  // define the class name
  //
  static const String CLASS_NAME;
  //----------------------------------------
  //
  // other important constants
  //
  //----------------------------------------
  
  //----------------------------------------
  //
  // default values and arguments
  //
  //----------------------------------------  

  // define the default value(s) of the class data
  //
  
  // default arguments to methods
  //
  
  //----------------------------------------
  //
  // error codes
  //
  //----------------------------------------  

  static const long ERR = 20200;
  
  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:

  // static memory manager
  //
  static MemoryManager mgr_d;
  
  //---------------------------------------------------------------------------
  //
  // required public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: name
  //
  static const String& name() {
    return CLASS_NAME;
  }

  // other static methods
  //
  static boolean diagnose(Integral::DEBUG debug_level);

  // debug methods:
  //  the setDebug method is inherited from the MScalar template class
  //
  boolean debug(const unichar* message) const;
  
  // method: destructor
  //
  ~Byte() {}

  // method: default constructor
  //
  Byte(byte arg = DEF_VALUE) {
    value_d = arg;
  }

  // method: copy constructor
  //
  Byte(const Byte& arg) {
    assign(arg.value_d);
  }

  // assign methods:
  //  these methods are inherited from the MScalar template class
  //
  
  // method: operator=
  //
  Byte& operator= (const Byte& arg) {
    assign(arg);
    return *this;
  }
  
  // i/o methods:
  //  the sofSize, readData and writeData are inherited from the MScalar
  //  template class
  //

  // method: read
  //
  boolean read(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MScalar<byte, byte8>::read(sof, tag, name);
  }

  // method: write
  //
  boolean write(Sof& sof, long tag, const String& name = CLASS_NAME) const {
    return MScalar<byte, byte8>::write(sof, tag, name);
  }

  // equality methods:
  //  these methods are inherited from the MScalar template class
  //
  
  // method: new
  //
  static void* operator new(size_t size) {
    return mgr_d.get();
  }

  // method: new[]
  //
  static void* operator new[](size_t size) {
    return mgr_d.getBlock(size);
  }

  // method: delete
  //
  static void operator delete(void* ptr) {
    mgr_d.release(ptr);
  }

  // method: delete[]
  //
  static void operator delete[](void* ptr) {
    mgr_d.releaseBlock(ptr);
  }

  // method: setGrowSize
  //
  static boolean setGrowSize(long grow_size) {
    return mgr_d.setGrow(grow_size);
  }

  // other memory allocation methods:
  //  the clear method is inherited from the MScalar template class
  //
  
  //---------------------------------------------------------------------------
  //
  // class-specific public methods
  //
  //---------------------------------------------------------------------------
public:

  // method: operator=
  //
  Byte& operator= (byte arg) {
    value_d = arg;
    return *this;
  }
  
  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:

  //declare trigonometric methods as private because they cannot be
  //defined for byte operations
  //
  byte acos(byte);
  byte acosh(byte);
  byte asin(byte);
  byte asinh(byte);
  byte atan(byte);
  byte atanh(byte);
  byte cos(byte);
  byte cosh(byte);
  byte sin(byte);
  byte sinh(byte);
  byte tan(byte);
  byte tanh(byte);
  
  //declare those methods as private which cannot be defined for
  //unsigned numbers
  //
  byte abs(byte);
  byte neg(byte);
  byte sign(byte);

  //declare those methods as private which cannot be defined for
  //integer numbers
  //  
  byte ceil(byte);
  byte rceil(byte);
  byte rfloor(byte);
  byte round(byte);
  byte floor(byte);

};

// end of include file
//
#endif
