// file: $isip/class/math/scalar/Llong/Llong.h
// version: $Id: Llong.h,v 1.30 2001/10/31 05:16:14 gao Exp $
//

// make sure definitions are only made once
//
#ifndef ISIP_LLONG
#define ISIP_LLONG

// isip include files
//
#ifndef ISIP_SCALAR
#include <MScalar.h>
#endif

// Llong: a class that manages a 64-bit signed integer. this class mainly
// inherits the MScalar template functionality. we only need to add Sof file
// support and memory management
//
class Llong : public MScalar<llong, int64> {

  //---------------------------------------------------------------------------
  //
  // public constants
  //
  //---------------------------------------------------------------------------
public:

  // define the class name
  //
  static const String CLASS_NAME;

  //----------------------------------------
  //
  // other important constants
  //
  //----------------------------------------
  
  //----------------------------------------
  //
  // default values and arguments
  //
  //----------------------------------------  

  // define the default value(s) of the class data
  //
  
  // default arguments to methods
  //

  //----------------------------------------
  //
  // error codes
  //
  //----------------------------------------  

  static const long ERR = 20900;

  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:

  // static memory manager
  //
  static MemoryManager mgr_d;
  
  //---------------------------------------------------------------------------
  //
  // required public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: name
  //
  static const String& name() {
    return CLASS_NAME;
  }

  // other static methods
  //
  static boolean diagnose(Integral::DEBUG debug_level);

  // debug methods:
  //  the serDebug method is inherited from the MScalar template class
  //
  boolean debug(const unichar* message) const;
  
  // method: destructor
  //
  ~Llong() {}

  // method: default constructor
  //
  Llong(llong arg = DEF_VALUE) {
    value_d = arg;
  }

  // method: copy constructor 
  //
  Llong(const Llong& arg) {
    assign(arg.value_d);
  }
  
  // assign methods:
  //  these methods are inherited from the MScalar template class
  //

  // method: operator=
  //
  Llong& operator= (const Llong& arg) {
    assign(arg);
    return *this;
  }
  
  // i/o methods:
  //  the sofSize, readData and writeData are inherited from the MScalar
  //  template class
  //

  // method: read
  //
  boolean read(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MScalar<llong, int64>::read(sof, tag, name);
  }

  // method: write
  //
  boolean write(Sof& sof, long tag, const String& name = CLASS_NAME) const {
    return MScalar<llong, int64>::write(sof, tag, name);
  }

  // equality methods:
  //  these methods are inherited from the MScalar template class
  //

  // method: new
  //
  static void* operator new(size_t size) {
    return mgr_d.get();
  }

  // method: new[]
  //  
  static void* operator new[](size_t size) {
    return mgr_d.getBlock(size);
  }

  // method: delete
  //  
  static void operator delete(void* ptr) {
    mgr_d.release(ptr);
  }

  // method: delete[]
  //  
  static void operator delete[](void* ptr) {
    mgr_d.releaseBlock(ptr);
  }

  // method: setGrowSize
  //  
  static boolean setGrowSize(long grow_size) {
    return mgr_d.setGrow(grow_size);
  }

  // other memory management methods:
  //  these methods are inherited from the MScalar template class
  //
  
  //---------------------------------------------------------------------------
  //
  // class-specific public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: operator=
  //
  Llong& operator= (llong arg) {
    value_d = arg;
    return *this;
  }

  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:
  //declare trigonometric methods as private because they cannot be
  //defined for Llong  operations
  //
  llong acos(llong);
  llong acosh(llong);
  llong asin(llong);
  llong asinh(llong);
  llong atan(llong);
  llong atanh(llong);
  llong cos(llong);
  llong cosh(llong);
  llong sin(llong);
  llong sinh(llong);
  llong tan(llong);
  llong tanh(llong);

  //declare those methods as private which cannot be defined for
  //integer numbers
  //  
  llong ceil(llong);
  llong rceil(llong);
  llong rfloor(llong);
  llong round(llong);
  llong floor(llong);
};

// end of include file
//
#endif
