// file: $isip/class/math/scalar/Ullong/Ullong.h
// version: $Id: Ullong.h,v 1.28 2001/10/14 04:26:04 bahety Exp $
//

// make sure definitions are only made once
//
#ifndef ISIP_ULLONG
#define ISIP_ULLONG

// isip include files
//
#ifndef ISIP_SCALAR
#include <MScalar.h>
#endif

// Ullong: a class manages the a 64-bit unsigned integer. this class mainly
// inherits the MScalar template functionality. we only need to add Sof file
// support and memory management
//
class Ullong : public MScalar<ullong, uint64> {

  //---------------------------------------------------------------------------
  //
  // public constants
  //
  //---------------------------------------------------------------------------
public:

  // define the class name
  //
  static const String CLASS_NAME;

  //----------------------------------------
  //
  // other important constants
  //
  //----------------------------------------
  
  //----------------------------------------
  //
  // default arguments and values
  //
  //----------------------------------------  

  // define the default value(s) of the class data
  //
  
  // default arguments to methods
  //
  
  //----------------------------------------
  //
  // error codes
  //
  //----------------------------------------  

  static const long ERR = 20600;

  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:

  // static memory manager
  //
  static MemoryManager mgr_d;
  
  //---------------------------------------------------------------------------
  //
  // required public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: name
  //
  static const String& name() {
    return CLASS_NAME;
  }

  // other static methods
  //
  static boolean diagnose(Integral::DEBUG debug_level);

  // debug methods:
  //  the setDebug method is inherited from the MScalar template class
  //
  boolean debug(const unichar* message) const;
  
  // method: destructor
  //
  ~Ullong() {}

  // method: default constructor
  //
  Ullong(ullong arg = DEF_VALUE) {
    value_d = arg;
  }

  // method: copy constructor
  //
  Ullong(const Ullong& arg) {
    assign(arg.value_d);
  }

  // assign methods:
  //  these methods are inherited from the MScalar template class
  //

  // method: operator=
  //
  Ullong& operator= (const Ullong& arg) {
    assign(arg);
    return *this;
  }
    
  // i/o methods:
  //  the sofSize, readData, and writeData are inherited from the MScalar
  //  template class
  //

  // method: read
  //
  boolean read(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MScalar<ullong, uint64>::read(sof, tag, name);
  }

  // method: write
  //
  boolean write(Sof& sof, long tag, const String& name = CLASS_NAME) const {
    return MScalar<ullong, uint64>::write(sof, tag, name);
  }  

  // equality methods:
  //  the equality methods are inherited from the MScalar template class
  //

  // method: new
  //
  static void* operator new(size_t size) {
    return mgr_d.get();
  }

  // method: new[]
  //
  static void* operator new[](size_t size) {
    return mgr_d.getBlock(size);
  }

  // method: delete
  //
  static void operator delete(void* ptr) {
    mgr_d.release(ptr);
  }

  // method: delete[]
  static void operator delete[](void* ptr) {
    mgr_d.releaseBlock(ptr);
  }

  // method: setGrowSize
  //
  static boolean setGrowSize(long grow_size) {
    return mgr_d.setGrow(grow_size);
  }

  // other memory management:
  //  these methods are inherited from the MScalar template class
  //
  
  //---------------------------------------------------------------------------
  //
  // class-specific public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: operator=
  //
  Ullong& operator= (ullong arg) {
    value_d = arg;
    return *this;
  }
  
  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:
  //declare trigonometric methods as private because they cannot be
  //defined for Ullong operations
  //
  ullong acos(ullong);
  ullong acosh(ullong);
  ullong asin(ullong);
  ullong asinh(ullong);
  ullong atan(ullong);
  ullong atanh(ullong);
  ullong cos(ullong);
  ullong cosh(ullong);
  ullong sin(ullong);
  ullong sinh(ullong);
  ullong tan(ullong);
  ullong tanh(ullong);
  
  //declare those methods as private which cannot be defined for
  //unsigned numbers
  //
  ullong abs(ullong);
  ullong neg(ullong);
  ullong sign(ullong);

  //declare those methods as private which cannot be defined for
  //integer numbers
  //  
  ullong ceil(ullong);
  ullong rceil(ullong);
  ullong rfloor(ullong);
  ullong round(ullong);
  ullong floor(ullong);

};

// end of include file
//
#endif
