// file: $isip/class/math/scalar/Ulong/Ulong.h
// version: $Id: Ulong.h,v 1.29 2001/10/31 05:18:51 gao Exp $
//

// make sure definitions are only made once
//
#ifndef ISIP_ULONG
#define ISIP_ULONG

// isip include files
//
#ifndef ISIP_SCALAR
#include <MScalar.h>
#endif

// Ulong: a class that manages a 32-bit unsigned integer. this class mainly
// inherits the MScalar template functionality. we only need to add Sof file
// support and memory management
//
class Ulong : public MScalar<ulong, uint32> {

  //---------------------------------------------------------------------------
  //
  // public constants
  //
  //---------------------------------------------------------------------------
public:

  // define the class name
  //
  static const String CLASS_NAME;

  //----------------------------------------
  //
  // other important constants
  //
  //----------------------------------------
    
  //----------------------------------------
  //
  // default values and arguments
  //
  //----------------------------------------
  
  // define the default value(s) of the class data
  //
  
  // default arguments to methods
  //
   
  //----------------------------------------
  //
  // error codes
  //
  //----------------------------------------  

  static const long ERR = 20500;

  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:

  // static memory manager
  //
  static MemoryManager mgr_d;
  
  //---------------------------------------------------------------------------
  //
  // required public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: name
  //
  static const String& name() {
    return CLASS_NAME;
  }

  // other static methods
  //
  static boolean diagnose(Integral::DEBUG debug_level);

  // debug methods:
  //  the setDebug method is inherited from the MScalar template class
  //
  boolean debug(const unichar* message) const;
  
  // method: destructor
  //
  ~Ulong() {}
  
  // method: default constructor
  //
  Ulong(ulong arg = DEF_VALUE) {
    value_d = arg;
  }

  // method: copy constructor
  //
  Ulong(const Ulong& arg) {
    assign(arg.value_d);
  }

  // assign methods:
  //  these methods are inherited from the MScalar template class
  //

  // method: operator=
  //
  Ulong& operator= (const Ulong& arg) {
    assign(arg);
    return *this;
  }

  // i/o methods:
  //  the sofSize, readData and writeData are inherited from the MScalar
  //  template class
  //

  // method: read
  //
  boolean read(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MScalar<ulong, uint32>::read(sof, tag, name);
  }

  // method: write
  //
  boolean write(Sof& sof, long tag, const String& name = CLASS_NAME) const {
    return MScalar<ulong, uint32>::write(sof, tag, name);
  }

  // equality methods:
  //  these methods are inherited from the MScalar template class
  //
  
  // method: new
  //
  static void* operator new(size_t size) {
    return mgr_d.get();
  }

  // method: new[]
  //  
  static void* operator new[](size_t size) {
    return mgr_d.getBlock(size);
  }

  // method: delete
  //  
  static void operator delete(void* ptr) {
    mgr_d.release(ptr);
  }

  // method: delete[]
  //  
  static void operator delete[](void* ptr) {
    mgr_d.releaseBlock(ptr);
  }

  // method: setGrowSize
  //  
  static boolean setGrowSize(long grow_size) {
    return mgr_d.setGrow(grow_size);
  }

  // other memory management:
  //  these methods are inherited from the MScalar template class
  //
  
  //---------------------------------------------------------------------------
  //
  // class-specific public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: operator=
  //
  Ulong& operator= (ulong arg) {
    value_d = arg;
    return *this;
  }

  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:
  //declare trigonometric methods as private because they cannot be
  //defined for Ulong operations
  //
  ulong acos(ulong);
  ulong acosh(ulong);
  ulong asin(ulong);
  ulong asinh(ulong);
  ulong atan(ulong);
  ulong atanh(ulong);
  ulong cos(ulong);
  ulong cosh(ulong);
  ulong sin(ulong);
  ulong sinh(ulong);
  ulong tan(ulong);
  ulong tanh(ulong);
  
  //declare those methods as private which cannot be defined for
  //unsigned numbers
  //
  ulong abs(ulong);
  ulong neg(ulong);
  ulong sign(ulong);

  //declare those methods as private which cannot be defined for
  //integer numbers
  //  
  ulong ceil(ulong);
  ulong rceil(ulong);
  ulong rfloor(ulong);
  ulong round(ulong);
  ulong floor(ulong);
 
};

// end of include file
//
#endif
