// file: $isip/class/math/scalar/Ushort/Ushort.h
// version: $Id: Ushort.h,v 1.26 2001/10/31 05:18:58 gao Exp $
//

// make sure definitions are only made once
//
#ifndef ISIP_USHORT
#define ISIP_USHORT

// isip include files
//
#ifndef ISIP_SCALAR
#include <MScalar.h>
#endif

// Ushort: a class that manages a 16-bit unsigned integer. this class mainly
// inherits the MScalar template functionality. we only need to add Sof file
// support and memory management
//
class Ushort : public MScalar<ushort, uint16> {

  //---------------------------------------------------------------------------
  //
  // public constants
  //
  //---------------------------------------------------------------------------
public:

  // define the class name
  //
  static const String CLASS_NAME;

  //----------------------------------------
  //
  // other important constants
  //
  //----------------------------------------
    
  //----------------------------------------
  //
  // default values and arguments
  //
  //----------------------------------------  

  // define the default value(s) of the class data
  //
  
  // default arguments to methods
  //
  
  //----------------------------------------
  //
  // error codes
  //
  //----------------------------------------  

  static const long ERR = 20400;

  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:

  // static memory manager
  //
  static MemoryManager mgr_d;
  
  //---------------------------------------------------------------------------
  //
  // required public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: name
  //
  static const String& name() {
    return CLASS_NAME;
  }

  // other static methods
  //
  static boolean diagnose(Integral::DEBUG debug_level);

  // debug methods:
  //  the setDebug method is inherited from the MScalar template class
  //
  boolean debug(const unichar* message) const;
  
  // method: destructor
  //
  ~Ushort() {}

  // method: default constructor
  //
  Ushort(ushort arg = DEF_VALUE) {
    value_d = arg;
  }

  // method: copy constructor
  //
  Ushort(const Ushort& arg) {
    assign(arg.value_d);
  }

  // assign methods:
  //  these methods are inherited from the MScalar template class
  //

  // method: operator=
  //
  Ushort& operator= (const Ushort& arg) {
    assign(arg);
    return *this;
  }
  
  // i/o methods:
  //  the sofSize, readData and writeData are inherited from the MScalar
  //  template class
  //

  // method: read
  //
  boolean read(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MScalar<ushort, uint16>::read(sof, tag, name);
  }

  // method: write
  //
  boolean write(Sof& sof, long tag, const String& name = CLASS_NAME) const {
    return MScalar<ushort, uint16>::write(sof, tag, name);
  }  

  // equality methods:
  //  these methods are inherited from the MScalar template class
  //

  // method: new
  //
  static void* operator new(size_t size) {
    return mgr_d.get();
  }

  // method: new[]
  //
  static void* operator new[](size_t size) {
    return mgr_d.getBlock(size);
  }

  // method: delete
  //
  static void operator delete(void* ptr) {
    mgr_d.release(ptr);
  }

  // method: delete[]
  //
  static void operator delete[](void* ptr) {
    mgr_d.releaseBlock(ptr);
  }

  // method: setGrowSize
  //
  static boolean setGrowSize(long grow_size) {
    return mgr_d.setGrow(grow_size);
  }

  // other memory management:
  //  these methods are inherited from the MScalar template class
  //
  
  //---------------------------------------------------------------------------
  //
  // class-specific public methods
  //
  //---------------------------------------------------------------------------

  // method: operator=
  //
  Ushort& operator= (ushort arg) {
    value_d = arg;
    return *this;
  }
  
  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:
  //declare trigonometric methods as private because they cannot be
  //defined for Ullong operations
  //

  ushort acos(ushort);
  ushort acosh(ushort);
  ushort asin(ushort);
  ushort asinh(ushort);
  ushort atan(ushort);
  ushort atanh(ushort);
  ushort cos(ushort);
  ushort cosh(ushort);
  ushort sin(ushort);
  ushort sinh(ushort);
  ushort tan(ushort);
  ushort tanh(ushort);
  
  //declare those methods as private which cannot be defined for
  //unsigned numbers
  //
  ushort abs(ushort);
  ushort neg(ushort);
  ushort sign(ushort);

  //declare those methods as private which cannot be defined for
  //integer numbers
  //  
  ushort ceil(ushort);
  ushort rceil(ushort);
  ushort rfloor(ushort);
  ushort round(ushort);
  ushort floor(ushort);

};

// end of include file
//
#endif
