// file: $isip/class/mmedia/Annotation/anno_03.cc
// version: $Id: anno_03.cc,v 1.4 2002/11/24 16:43:37 gao Exp $
//

// isip include files
//
#include "Annotation.h"

// method: read
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a boolean value indicating status
//
// this method has the object read itself from an Sof file
//
boolean Annotation::read(Sof& sof_a, long tag_a, const String& name_a) {

  // read the instance of the object from the Sof file
  //
  if (!sof_a.find(name_a, tag_a)) {
    return false;
  }
  
  // read the actual data from the sof file
  //
  if (!readData(sof_a)) {
    return false;
  }
  
  // exit gracefully
  //
  return true;  
}

// method: readData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//  long size: (input) size in bytes of object (or full_size)
//  boolean param: (input) is the parameter name in the file?
//  boolean nested: (input) are we nested?
//
// return: a boolean value indicating status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean Annotation::readData(Sof& sof_a, const String& pname_a,
			  long size_a, boolean param_a,
			  boolean nested_a) {

  // set i/o parameters
  //
  Integral::DEBUG debug_level = Integral::NONE;
  SofParser parser;
  parser.setDebug(debug_level);
    
  // ignore implicit parameter setting
  //

  // clear the annotation
  //
  this->clear();
  
  // are we nested?
  //
  if (nested_a) {
    parser.setNest();
  }
  
  // load the parse
  //
  if (!parser.load(sof_a, size_a)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }	    

  // get the id
  //
  if (!id_d.readData(sof_a, PARAM_ID,
			parser.getEntry(sof_a, PARAM_ID),
			false, false)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }
  
  // get the type
  //
  if (!type_d.readData(sof_a, PARAM_TYPE,
			parser.getEntry(sof_a, PARAM_TYPE),
			false, false)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }

  // get the channel index
  //
  if (parser.isPresent(sof_a, PARAM_CHANNEL_INDEX)) {
    if (!channel_index_d.readData(sof_a, PARAM_CHANNEL_INDEX,
                                  parser.getEntry(sof_a,
                                                     PARAM_CHANNEL_INDEX),
                                  false, false)) {
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::IO,
                           __FILE__, __LINE__, Error::WARNING);
    }
  }
  else {
    channel_index_d.assign(DEF_CHANNEL_INDEX);
  }

  
  // get the feature map
  //
  if (!feature_map_d.readData(sof_a, PARAM_FEATURES,
			parser.getEntry(sof_a, PARAM_FEATURES),
			true, true)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }        
  
  // exit gracefully
  //
  return true;
}

