// file: $isip/class/mmedia/AnnotationGraph/angr_04.cc
// version: $Id: angr_04.cc,v 1.3 2002/08/08 20:05:40 gao Exp $
//

// isip include files
//
#include "AnnotationGraph.h"

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: boolean value indicating status
//
// this method has the object write itself to an Sof file
//
boolean AnnotationGraph::write(Sof& sof_a, long tag_a,
		       const String& name_a) const {

  long obj_size = 0;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // write the primitive member data
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: boolean value indicating status
//
// this method has the object write itself to an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean AnnotationGraph::writeData(Sof& sof_a, const String& pname_a) const {

  // declare local variables
  //
  DoubleLinkedList<Pair<Long, Anchor> > ancrs;
  DoubleLinkedList<Triple<Long, Long, Annotation> > annos;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the graph id
  //
  id_d.writeData(sof_a, PARAM_ID);
  
  // write the graph type
  //
  type_d.writeData(sof_a, PARAM_TYPE);  

  // get the annotations and anchors associated with the graph
  //
  const_cast<AnnotationGraph*>(this)->get(ancrs, annos);

  // write the list of anchors
  //
  ancrs.writeData(sof_a, PARAM_ANCHORS);
  
  // write the list of annotations
  //
  annos.writeData(sof_a, PARAM_ANNOTATIONS);
  
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}



