// file: $isip/class/pr/NGramNode/nnode_04.cc
// version: $Id: nnode_04.cc,v 1.4 2002/09/29 17:56:46 huang Exp $
//

// isip include files
//
#include "NGramNode.h"

// method: sofSize
//
// arguments: none
//
// return: a long value indicating the size of object
//
// this method return  the size of the object
//
long NGramNode::sofSize() const {
  
  // calculate the space of data type and algorithm
  //
  long bytes = index_d.sofSize();
  bytes += lm_score_d.sofSize();
  bytes += backoff_d.sofSize();
  
  if (next_gram_d != NULL) {
    bytes += next_gram_d->sofSize();
  }
  else {
    HashTable<Long, NGramNode> next_gram;
    bytes +=  next_gram.sofSize();
  }
  
  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a boolean value indicating status
//
// this method has the object write itself, with specified name and
// tag, to an Sof file
//
boolean NGramNode::write(Sof& sof_a, long tag_a,
			 const String& name_a) const {

  // declare local variables
  //  
  long obj_size;

  // write the instance of the object into the Sof file
  //  
  if (sof_a.isText()) {
    obj_size = Sof::ANY_SIZE;
  }
  else {
    obj_size = sofSize();
  }
  
  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }

  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) textual parameter name
//
// return: logical error status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean NGramNode::writeData(Sof& sof_a, const String& pname_a) const {

  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);
    
  // write the configuration
  //  
  index_d.writeData(sof_a, PARAM_INDEX);
  lm_score_d.writeData(sof_a, PARAM_LM_SCORE);
  backoff_d.writeData(sof_a, PARAM_BACKOFF);

  if (next_gram_d != NULL) {

    next_gram_d->writeData(sof_a, PARAM_HASH);

  }
  else {
    HashTable<Long, NGramNode> next_gram;
    next_gram.writeData(sof_a, PARAM_HASH);    
  }
  
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
