// file: $isip/class/algo/SegmentConcat/segc_00.cc
// version: $Id: segc_00.cc,v 1.2 2003/05/05 22:48:21 parihar Exp $
//

// isip include files
//
#include "SegmentConcat.h"

//------------------------------------------------------------------------
//
// required public methods
//
//-----------------------------------------------------------------------

// method: assign
//
// arguments:
//  const SegmentConcat& arg: (input) object to be assigned
//
// return: a boolean value indicating status
//
// this method assigns the input object to the current object
//
boolean SegmentConcat::assign(const SegmentConcat& arg_a) {

  // assign data from the input SegmentConcat object
  //
  algorithm_d = arg_a.algorithm_d;
  implementation_d = arg_a.implementation_d;
  dim_d = arg_a.dim_d;

  // exit gracefully
  //
  return true;
}

// method: eq
//
// arguments:
//  const SegmentConcat& arg: (input) object to be compared
//
// return: a boolean value indicating status
//
// this method checks whether the current object is identical to the
// input object
//
boolean SegmentConcat::eq(const SegmentConcat& arg_a) const {

  // compare the data members
  //
  if ((algorithm_d != arg_a.algorithm_d) ||
      (implementation_d != arg_a.implementation_d) ||
      (dim_d != arg_a.dim_d)) {
    return false;
  }

  // exit gracefully
  //
  return true;
}

// method: clear
//
// arguments:
//  Integral::CMODE ctype: (input) clear mode
//
// return: a boolean value indicating status
//
// this method resets the data members to the default values
//
boolean SegmentConcat::clear(Integral::CMODE ctype_a) {
  
  // reset the data members unless the mode is RETAIN
  //
  if (ctype_a != Integral::RETAIN) {

    // clear the design parameters
    //
    algorithm_d = DEF_ALGORITHM;
    implementation_d = DEF_IMPLEMENTATION;
    dim_d = DEF_DIMENSION;
  }
  
  // exit gracefully
  //
  return true;
}

//-----------------------------------------------------------------------------
//
// we define non-integral constants in the default constructor
//      
//-----------------------------------------------------------------------------

// constants: class name
//
const String SegmentConcat::CLASS_NAME(L"SegmentConcat");

// constants: i/o related constants
//
const String SegmentConcat::DEF_PARAM(L"");
const String SegmentConcat::PARAM_ALGORITHM(L"algorithm");
const String SegmentConcat::PARAM_IMPLEMENTATION(L"implementation");

// constants: name map(s) for the enumerated values
//
const NameMap SegmentConcat::ALGO_MAP(L"MIN_MAX, SEG_CONCAT");
const NameMap SegmentConcat::IMPL_MAP(L"NONE, NORMALIZE");

// static instantiations: memory manager
//
Integral::DEBUG SegmentConcat::debug_level_d = Integral::NONE;
MemoryManager SegmentConcat::mgr_d(sizeof(SegmentConcat), SegmentConcat::name());
