// file: $isip/class/pr/PhoneticDecisionTree/pdt_03.cc
// version: $Id: pdt_03.cc,v 1.2 2002/09/11 20:04:06 parihar Exp $
//

// isip include files
//
#include "PhoneticDecisionTree.h"

//------------------------------------------------------------------
// these methods have to be in the same file so they can use the same
// static parser pointer
//------------------------------------------------------------------

// declare a static sof parser pointer
//
static SofParser* parser_d = (SofParser*)NULL;

// method: setParser
//
// arguments:
//  SofParser* parser: (input) sof file object
//
// return: a boolean value indicating status
//
// this method sets the parser from the parent object to be used in
// the next readData method.
//
boolean PhoneticDecisionTree::setParser(SofParser* parser_a) {
  
  // set the parser
  //
  parser_d = parser_a;

  // exit gracefully
  //
  return true;
}

// method: read
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a boolean value indicating status
//
// this method has the object read itself from an Sof file
//
boolean PhoneticDecisionTree::read(Sof& sof_a, long tag_a,
				   const String& name_a) {
  
  // get the instance of the object from the Sof file
  //
  if (!sof_a.find(name_a, tag_a)) {
    return false;
  }

  // read the actual data from the sof file
  //
  return readData(sof_a);
}

// method: readData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//  long size: (input) size of the object
//  boolean param: (input) is the parameter specified?
//  boolean nested: (input) is this nested?
//
// return: a boolean value indicating status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean PhoneticDecisionTree::readData(Sof& sof_a, const String& pname_a,
				       long size_a, boolean param_a,
				       boolean nested_a) {
  // declare local variables
  //
  boolean status = true;

  // we need a parser
  // 
  if (parser_d == (SofParser*)NULL) {
    
    parser_d = new SofParser;
    
    if (nested_a) {
      parser_d->setNest();
    }
    
    // load the parse
    //
    if (!parser_d->load(sof_a, size_a)) {

      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
                           __FILE__, __LINE__, Error::WARNING);
    }           
  }

  // set the debug level of the parser
  //
  parser_d->setDebug(debug_level_d.getIndex());
  
  // get the stopmode
  //
  if (parser_d->isPresent(sof_a, PARAM_STOPMODE)) {
    if (!STOPMODE_MAP.readElementData((long&)stopmode_d, sof_a,
				      PARAM_STOPMODE,
				      parser_d->getEntry(sof_a,
							 PARAM_STOPMODE))) {

      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readDataText", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    stopmode_d = DEF_STOPMODE;
  }

  // get the runmode
  //
  if (parser_d->isPresent(sof_a, PARAM_RUNMODE)) {
    if (!RUNMODE_MAP.readElementData((long&)implementation_d, sof_a,
				  PARAM_RUNMODE,
				  parser_d->getEntry(sof_a,
						     PARAM_RUNMODE))) {
      
      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    runmode_d = DEF_RUNMODE;
  }
  
  // get the algorithm name
  //
  if (parser_d->isPresent(sof_a, PARAM_ALGORITHM)) {
    if (!ALGO_MAP.readElementData((long&)algorithm_d, sof_a,
				  PARAM_ALGORITHM,
				  parser_d->getEntry(sof_a, PARAM_ALGORITHM))) {

      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    algorithm_d = DEF_ALGORITHM;
  }

  // get the implementation
  //
  if (parser_d->isPresent(sof_a, PARAM_IMPLEMENTATION)) {
    if (!IMPL_MAP.readElementData((long&)implementation_d, sof_a,
				  PARAM_IMPLEMENTATION,
				  parser_d->getEntry(sof_a,
						     PARAM_IMPLEMENTATION))) {
      
      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    implementation_d = DEF_IMPLEMENTATION;
  }

  // get the split threshold
  //
  if (parser_d->isPresent(sof_a, PARAM_SPLIT_THRESHOLD)) {
    if (!split_threshold_d.readData(sof_a, PARAM_SPLIT_THRESHOLD,
				    parser_d->getEntry(sof_a,
						       PARAM_SPLIT_THRESHOLD))) {
      
      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    split_threshold_d = DEF_SPLIT_THRESHOLD;
  }

  // get the merge threshold
  //
  if (parser_d->isPresent(sof_a, PARAM_MERGE_THRESHOLD)) {
    if (!merge_threshold_d.readData(sof_a, PARAM_MERGE_THRESHOLD,
				    parser_d->getEntry(sof_a,
						       PARAM_MERGE_THRESHOLD))) {
      
      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    merge_threshold_d = DEF_MERGE_THRESHOLD;
  }

  // get the num_occ threshold
  //
  if (parser_d->isPresent(sof_a, PARAM_NUM_OCC_THRESHOLD)) {
    if (!num_occ_threshold_d.readData(sof_a, PARAM_NUM_OCC_THRESHOLD,
				      parser_d->getEntry(sof_a,
							 PARAM_NUM_OCC_THRESHOLD))) {
      
      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    num_occ_threshold_d = DEF_NUM_OCC_THRESHOLD;
  }

  // read the BiGraph
  //
  if (!BiGraph<PhoneticDecisionTreeNode>::readData(sof_a, PARAM_BDT)) {
    return Error::handle(name(), L"readData", Error::ARG,
			 __FILE__, __LINE__);
  }

  // get the debug level
  //
  if (parser_d->isPresent(sof_a, PARAM_DBGL)) {
    if (!debug_level_d.readData(sof_a, PARAM_DBGL,
				parser_d->getEntry(sof_a, PARAM_DBGL))) {
      
      // delete the parser
      //
      delete parser_d;
      parser_d = (SofParser*)NULL;
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::IO,
			   __FILE__, __LINE__, Error::WARNING);
    }
  }
  else {
    debug_level_d = Integral::DEF_DEBUG;
  }

  // check that all parameters are accounted for
  //
  if (!parser_d->checkParams(sof_a)) {

    // delete the parser
    //
    delete parser_d;
    parser_d = (SofParser*)NULL;

    // return a warning message
    //
    return Error::handle(name(), L"readData", Error::IO,
			 __FILE__, __LINE__, Error::WARNING);
  }

  // delete the parser
  //
  delete parser_d;
  parser_d = (SofParser*)NULL;

  // exit gracefully
  //
  return status;
}
