// file: $isip/class/pr/PhoneticDecisionTree/pdt_04.cc
// version: $Id: pdt_04.cc,v 1.2 2002/09/11 20:04:06 parihar Exp $
//

// isip include files
//
#include "PhoneticDecisionTree.h"

// method: sofSize
//
// arguments: none 
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
long PhoneticDecisionTree::sofSize() const {

  // start with the space required for the algorithm name
  //
  long bytes = ALGO_MAP.elementSofSize();

  // add the space required for the implementation name
  //
  bytes += IMPL_MAP.elementSofSize();

  // add the space required for the stopmode
  //
  bytes += STOPMODE_MAP.elementSofSize();

  // add the space required for the runmode
  //
  bytes += RUNMODE_MAP.elementSofSize();

  // add the space required for the split_threshold
  //
  bytes += split_threshold_d.sofSize();

  // add the space required for the merge_threshold
  //
  bytes += merge_threshold_d.sofSize();

  // add the space required for the num_occ_threshold
  //
  bytes += num_occ_threshold_d.sofSize();

  // add the space required for the base BiGraph
  //
  bytes += BiGraph<PhoneticDecisionTreeNode>::sofSize();
  
  // add the space required for the debug level
  //
  bytes += debug_level_d.sofSize();

  // add the space for other parameters defined in the
  // DecisionTreeBase class
  //
  //  bytes += attributes_d.sofSize();

  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a boolean value indicating status
//
// this method has the object write itself to an Sof file
//
boolean PhoneticDecisionTree::write(Sof& sof_a, long tag_a,
				    const String& name_a) const {

  // declare local variable
  //
  long obj_size;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to be dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // set the size to be the size of the object written to the Sof file
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // write data and exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a boolean value indicating status
//
// this method branches on the algorithm name
//
boolean PhoneticDecisionTree::writeData(Sof& sof_a,
					const String& pname_a) const {
  
  // declare local variables
  //
  boolean status = false;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the stopmode (from the base class)
  //
  STOPMODE_MAP.writeElementData(sof_a, PARAM_STOPMODE, (long)stopmode_d);
    
  // write the runmode (from the base class)
  //
  RUNMODE_MAP.writeElementData(sof_a, PARAM_RUNMODE, (long)runmode_d);

  // write the algorithm name
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (long)algorithm_d);
    
  // write the implementation type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (long)implementation_d);
  
  // write the split threshold
  //
  split_threshold_d.writeData(sof_a, PARAM_SPLIT_THRESHOLD);
  
  // write the merge threshold
  //
  merge_threshold_d.writeData(sof_a, PARAM_MERGE_THRESHOLD);
  
  // write the num-occ threshold
  //
  num_occ_threshold_d.writeData(sof_a, PARAM_NUM_OCC_THRESHOLD);
  
  // write the BiGraph (base class)
  //
  if (!BiGraph<PhoneticDecisionTreeNode>::writeData(sof_a, PARAM_BDT)) {
    return Error::handle(name(), L"writeData", Error::ARG,
			 __FILE__, __LINE__);
  }
  
  // write the debug level (from the base class)
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);
 
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return status;
}
