// file: $isip/class/pr/PhoneticDecisionTreeNode/pdtnod_04.cc
// version: $Id: pdtnod_04.cc,v 1.2 2002/09/11 20:04:08 parihar Exp $
//

// isip include files
//
#include "PhoneticDecisionTreeNode.h"

// method: sofSize
//
// arguments: none 
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
long PhoneticDecisionTreeNode::sofSize() const {

  // start with the space required for the datapoints
  //
  long bytes = datapoints_d.sofSize();

  // add the space required for the best-attribute
  //
  bytes += best_attribute_d.sofSize();

  // add the space required for the typical_index
  //
  bytes += typical_index_d.sofSize();
  
  // add the space required for the actual_index
  //
  bytes += actual_index_d.sofSize();
  
  // add the space required for the typical statistical-model
  //
  bytes += typical_stat_model_d.sofSize();
  
  // add the space required for the flag that denotes if this node
  // exists
  //
  bytes += flag_exists_d.sofSize();
  
  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: boolean value indicating status
//
// this method has the object write itself to an Sof file
//
boolean PhoneticDecisionTreeNode::write(Sof& sof_a, long tag_a,
					const String& name_a) const {

  long obj_size = 0;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: boolean value indicating status
//
// this method has the object write itself to an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean PhoneticDecisionTreeNode::writeData(Sof& sof_a,
					    const String& pname_a) const {
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the datapoints
  //
  datapoints_d.writeData(sof_a, PARAM_DATAPOINTS);

  // write the best attribute
  //
  best_attribute_d.writeData(sof_a, PARAM_BEST_ATTRIBUTE);
  
  // write the typical statistical-model index
  //
  typical_index_d.writeData(sof_a, PARAM_TYPICAL_INDEX);
  
  // write the actual index
  //
  actual_index_d.writeData(sof_a, PARAM_ACTUAL_INDEX);
  
  // write the typical statistical-model
  //
  typical_stat_model_d.writeData(sof_a, PARAM_TYPICAL_STAT_MODEL);
  
  // write the falg that denotes whether a node exists or is merged
  // with some other node
  //
  flag_exists_d.writeData(sof_a, PARAM_FLAG_EXISTS);
  
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
