// file: $isip/class/search/Context/cont_04.cc
// version: $Id: cont_04.cc,v 1.2 2002/08/15 19:54:09 alphonso Exp $
//

// isip include files
//
#include "Context.h"

// method: sofSize
//
// arguments: none
//
// return: a long value indicating the size of object
//
// this method return  the size of the object
//
long Context::sofSize() const {
  
  // calculate the space of data type and algorithm
  //
  Long tmp;
  long bytes = 2 * tmp.sofSize();

  bytes += CircularDelayLine<Ulong>::sofSize();
  
  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a boolean value indicating status
//
// this method has the object write itself, with specified name and
// tag, to an Sof file
//
boolean Context::write(Sof& sof_a, long tag_a,
			  const String& name_a) const {

  // declare local variables
  //  
  long obj_size;

  // write the instance of the object into the Sof file
  //  
  if (sof_a.isText()) {
    obj_size = Sof::ANY_SIZE;
  }
  else {
    obj_size = sofSize();
  }
  
  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }

  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) textual parameter name
//
// return: logical error status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean Context::writeData(Sof& sof_a, const String& pname_a) const {

  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the configuration
  //  
  Long value;
  value.assign(central_d);
  value.writeData(sof_a, PARAM_CENTRAL);

  value.assign(ext_length_d);
  value.writeData(sof_a, PARAM_EXTENSION_LENGTH);

  // write the index
  //
  index_d.writeData(sof_a, PARAM_INDEX);
    
  // write the buffer
  //
  v_d.writeData(sof_a, PARAM_BUFFER);

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
