// file: $isip/class/stat/SearchLevel/slev_04.cc
// version: $Id: slev_04.cc,v 1.3 2002/09/15 03:46:48 parihar Exp $
//

// isip include files
//
#include "SearchLevel.h"

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: boolean value indicating status
//
// this method has the object write itself to an Sof file
//
boolean SearchLevel::write(Sof& sof_a, long tag_a,
			     const String& name_a) const {

  long obj_size = 0;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: boolean value indicating status
//
// this method has the object write itself to an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean SearchLevel::writeData(Sof& sof_a, const String& pname_a) const {
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the nsymbol parameters
  //
  use_nsymbol_d.writeData(sof_a, PARAM_USE_NSYMBOL);
  nsymbol_order_d.writeData(sof_a, PARAM_NSYMBOL_ORDER);

  // write the context parameters
  //  
  use_context_d.writeData(sof_a, PARAM_USE_CONTEXT);
  left_context_d.writeData(sof_a, PARAM_LEFT_CONTEXT_LENGTH);
  right_context_d.writeData(sof_a, PARAM_RIGHT_CONTEXT_LENGTH);

  // write the lexical tree parameters
  //  
  use_lexical_tree_d.writeData(sof_a, PARAM_USE_LEXICAL_TREE);

  // write the scale and symbol insertion parameters
  //
  lm_scale_d.writeData(sof_a, PARAM_LM_SCALE);
  tr_scale_d.writeData(sof_a, PARAM_TR_SCALE);  
  symbol_penalty_d.writeData(sof_a, PARAM_SYMBOL_PENALTY);
  
  // write the beam prunning parameters
  //  
  use_beam_prune_d.writeData(sof_a, PARAM_USE_BEAM);
  beam_threshold_d.writeData(sof_a, PARAM_BEAM_THRESHOLD);

  // write the instance prunning parameters
  //    
  use_instance_prune_d.writeData(sof_a, PARAM_USE_INSTANCE);
  instance_threshold_d.writeData(sof_a, PARAM_INSTANCE_THRESHOLD);
    
  // write the symbol occupancy parameters
  //
  write_symbol_occupancy_d.writeData(sof_a, PARAM_WRITE_SYMBOL_OCCUPANCY);
  symbol_occupancy_file_d.writeData(sof_a, PARAM_SYMBOL_OCCUPANCY_FILE);

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
