// file: $isip/class/search/StackSearch/ssrch_00.cc
// version: $Id: ssrch_00.cc,v 1.4 2002/09/06 21:46:31 jelinek Exp $
//

// isip include files
//
#include "StackSearch.h"

// method: destructor
//
// arguments: none
//
// return: none
//
// this is the default destructor for the StackSearch class.
//
StackSearch::~StackSearch() {

  // clear the current traces
  //
  clearTraceStorage();
}

// method: default constructor
//
// arguments: none
//
// return: none
//
// this is the default constructor for the StackSearch class.
//
StackSearch::StackSearch() {

  // initialize those data members that need to be initialized
  //
  num_frames_d = ALL_FRAMES;
  current_frame_d = START_FRAME;
  n_best_d = DEF_N_BEST;
  
  // set the default number of levels
  //
  setNumLevels(DEF_NUM_LEVELS);

  // set the allocation mode for the valid hypotheses list
  //
  valid_hyps_d.setAllocationMode(DstrBase::USER);

  // allocate variables for the stack decoder
  // note: stat_model_scores_d are not allocated yet, because we don't
  // know the number of statistical models. It will be done in
  // initSizes() method
  //
  // there will be one initial stack with index zero
  //
  stacks_d.setLength(MAX_NUM_FRAMES + 1);
  max_stack_scores_d.setLength(MAX_NUM_FRAMES + 1);
  max_frame_scores_d.setDimensions(MAX_NUM_FRAMES, DEF_NUM_LEVELS);
}

// method: copy constructor
//
// arguments:
//  const StackSearch& arg: (input) search to copy
//
// return: none
//
// this is the copy constructor for the StackSearch class
//
StackSearch::StackSearch(const StackSearch& arg_a) {

  // initialize the data members that need it
  //
  clear();

  // set the allocation mode for the valid hypotheses list
  //
  valid_hyps_d.setAllocationMode(DstrBase::USER);

  // assign the search
  //
  assign(arg_a);
}

//-----------------------------------------------------------------------------
//
// we define non-integral constants in the default constructor
//      
//-----------------------------------------------------------------------------

// constants: required constants such as class name
//
const String StackSearch::CLASS_NAME(L"StackSearch");

// constants: i/o related constants
//
const String StackSearch::DEF_PARAM(L"stack_search");

// static instantiations: memory manager and debug level
//
MemoryManager StackSearch::mgr_d(sizeof(StackSearch), name());
Integral::DEBUG StackSearch::debug_level_d = Integral::NONE;

//
// end of file
