// file: $isip/class/stat/Histogram/hist_02.cc
// version: $Id: hist_02.cc,v 1.1 2001/02/01 16:47:49 hamaker Exp $
//

// isip include files
//
#include "Histogram.h"
#include <Console.h>

// method: diagnose
//
// arguments:
//  Integral::DEBUG level: (input) debug level for diagnostics
//
// return: a boolean value indicating status
//
// this is the diagnose method
//
boolean Histogram::diagnose(Integral::DEBUG level_a) {

  //---------------------------------------------------------------------------
  //
  // 0. preliminaries
  //
  //---------------------------------------------------------------------------

  // output the class name
  //
  if (level_a > Integral::NONE) {
    SysString output(L"diagnosing class ");
    output.concat(CLASS_NAME);
    output.concat(L": ");
    Console::put(output);
    Console::increaseIndention();
  }
  
  //--------------------------------------------------------------------------
  //
  // 1. required public methods
  //
  //--------------------------------------------------------------------------

  // set indentation
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing required public methods...\n");
    Console::increaseIndention();
  }

  // create data to test with
  //
  VectorDouble test_bins(L"1, 3, 5, 7, 9");
  VectorDouble test_edges_bins(L"0, 2.5, 5, 7.5, 10");
  VectorDouble test_counts(L"3, 2, 2, 2, 2");
  VectorDouble test_ones(L"1, 1, 1, 1, 1");
  VectorDouble test_data(L"0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10");
  
  // do a quick test of constructors, destructors and memory management
  // methods
  //
  Histogram::setGrowSize(5);
  Histogram* hist0 = new Histogram(CENTERS);
  Histogram* hist1 = new Histogram(*hist0);
  Histogram* hist2 = new Histogram[10];

  // clear the pointers
  //
  delete hist0;
  delete hist1;
  delete [] hist2;

  // set the internal data of hist0
  //
  hist0 = new Histogram(CENTERS);
  hist0->setBins(test_bins);
  hist0->setCounts(test_counts);

  // test the assign and eq methods
  //
  hist1 = new Histogram;
  hist1->assign(*hist0);
  if (!hist1->eq(*hist0)) {
    hist0->debug(L"expected Histogram");
    hist1->debug(L"actual Histogram");
    return Error::handle(name(), L"assign/eq", Error::TEST,
			 __FILE__, __LINE__);
  }
  delete hist1;

  // test copy constructor
  //
  hist1 = new Histogram(*hist0);
  if (!hist1->eq(*hist0)) {
    hist0->debug(L"expected Histogram");
    hist1->debug(L"actual Histogram");
    return Error::handle(name(), L"copy constructor", Error::TEST,
			 __FILE__, __LINE__);
  }

  // test the clear method
  //
  hist0->clear();
  if (hist1->eq(*hist0)) {
    hist0->debug(L"hist0 not clear");
    return Error::handle(name(), L"clear", Error::TEST, __FILE__, __LINE__);
  }
  
  // test the operator= method
  //
  *hist0 = *hist1;
  if (!hist1->eq(*hist0)) {
    hist1->debug(L"expected Histogram");
    hist0->debug(L"actual Histogram");
    return Error::handle(name(), L"operator=", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  // clear the pointers
  //
  delete hist0;
  delete hist1;

  // test the i/o methods
  //
  Histogram hist3;
  Histogram hist4;
  Histogram hist5;
  Histogram hist6;
  hist3.setMode(CENTERS);
  hist4.setMode(EDGES);

  hist3.setBins(test_bins);
  hist3.setCounts(test_counts);

  hist4.setBins(test_bins);
  hist4.setCounts(test_counts);
  
  // we need binary and text sof files
  //
  String tmp_filename0;
  Integral::makeTemp(tmp_filename0);
  String tmp_filename1;
  Integral::makeTemp(tmp_filename1);

  // open files in write mode
  //
  Sof tmp_file0;
  tmp_file0.open(tmp_filename0, File::WRITE_ONLY, File::TEXT);
  Sof tmp_file1;
  tmp_file1.open(tmp_filename1, File::WRITE_ONLY, File::BINARY);

  hist3.write(tmp_file0, (long)0);
  hist4.write(tmp_file0, (long)1);
  hist3.write(tmp_file1, (long)0);
  hist4.write(tmp_file1, (long)1);

  // close the files
  //
  tmp_file0.close();
  tmp_file1.close();

  // open the files in read mode
  //
  tmp_file0.open(tmp_filename0);
  tmp_file1.open(tmp_filename1);

  // read the histograms back in
  //
  hist5.read(tmp_file0, (long)0);
  hist6.read(tmp_file0, (long)1);

  if (!hist5.eq(hist3)) {
    hist3.debug(L"expected Histogram");
    hist5.debug(L"hist5 actual");
    return Error::handle(name(), L"read", Error::TEST, __FILE__, __LINE__);
  }

  if (!hist6.eq(hist4)) {
    hist4.debug(L"expected Histogram");
    hist6.debug(L"hist6 actual");
    return Error::handle(name(), L"read", Error::TEST, __FILE__, __LINE__);
  }

  // read the model written in binary
  //
  hist5.clear();
  hist6.clear();
  hist5.read(tmp_file1, (long)0);
  hist6.read(tmp_file1, (long)1);

  if (!hist5.eq(hist3)) {
    hist3.debug(L"expected Histogram");
    hist5.debug(L"hist5 actual");
    return Error::handle(name(), L"read", Error::TEST, __FILE__, __LINE__);
  }

  if (!hist6.eq(hist4)) {
    hist4.debug(L"expected Histogram");
    hist6.debug(L"hist6 actual");
    return Error::handle(name(), L"read", Error::TEST, __FILE__, __LINE__);
  }

  // close and  delete the temporary files
  //
  tmp_file0.close();
  tmp_file1.close();
  
  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }

  //---------------------------------------------------------------------------
  //
  // 2. class-specific public methods:
  //     set methods
  //
  //---------------------------------------------------------------------------

  // set indentation
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing class-specific public methods: set methods...\n");
    Console::increaseIndention();
  }

  // test setMode
  //
  hist6.setMode(CENTERS);
  if (hist6.mode_d != CENTERS) {
    return Error::handle(name(), L"setMode", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  // test setBins, all types
  //
  VectorByte byte_bins;
  byte_bins.assign(test_bins);

  VectorDouble double_bins;
  double_bins.assign(test_bins);

  VectorFloat float_bins;
  float_bins.assign(test_bins);

  VectorLlong llong_bins;
  llong_bins.assign(test_bins);

  VectorLong long_bins;
  long_bins.assign(test_bins);

  VectorShort short_bins;
  short_bins.assign(test_bins);

  VectorUllong ullong_bins;
  ullong_bins.assign(test_bins);

  VectorUlong ulong_bins;
  ulong_bins.assign(test_bins);

  VectorUshort ushort_bins;
  ushort_bins.assign(test_bins);

  if (!hist6.setBins(byte_bins) || !hist6.bins_d.eq(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins(double_bins) || !hist6.bins_d.eq(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins(float_bins) || !hist6.bins_d.eq(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins(llong_bins) || !hist6.bins_d.eq(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins(long_bins) || !hist6.bins_d.eq(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins(short_bins) || !hist6.bins_d.eq(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins(ullong_bins) || !hist6.bins_d.eq(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins(ulong_bins) || !hist6.bins_d.eq(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins(ushort_bins) || !hist6.bins_d.eq(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }

  // test the scalar setBins, all types
  //
  double min = test_data.min();
  double max = test_data.max();
  long num_bins = test_bins.length();
  
  if (!hist6.setBins((byte)min, (byte)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((double)min, (double)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((float)min, (float)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((llong)min, (llong)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((long)min, (long)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((short)min, (short)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((ullong)min, (ullong)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((ulong)min, (ulong)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((ushort)min, (ushort)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_bins)) {
    test_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }

  // test setBins in EDGES mode
  //
  hist6.setMode(EDGES);
  if (!hist6.setBins((byte)min, (byte)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_edges_bins)) {
    test_edges_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((double)min, (double)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_edges_bins)) {
    test_edges_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((float)min, (float)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_edges_bins)) {
    test_edges_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((llong)min, (llong)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_edges_bins)) {
    test_edges_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((long)min, (long)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_edges_bins)) {
    test_edges_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((short)min, (short)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_edges_bins)) {
    test_edges_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((ullong)min, (ullong)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_edges_bins)) {
    test_edges_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((ulong)min, (ulong)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_edges_bins)) {
    test_edges_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setBins((ushort)min, (ushort)max, num_bins) ||
      !hist6.bins_d.almostEqual(test_edges_bins)) {
    test_edges_bins.debug(L"expected bins");
    hist6.bins_d.debug(L"actual bins");
    return Error::handle(name(), L"setBins", Error::TEST, __FILE__,
			 __LINE__);
  }

  // test setCounts, all types
  //
  VectorByte byte_counts;
  byte_counts.assign(test_counts);

  VectorDouble double_counts;
  double_counts.assign(test_counts);

  VectorFloat float_counts;
  float_counts.assign(test_counts);

  VectorLlong llong_counts;
  llong_counts.assign(test_counts);

  VectorLong long_counts;
  long_counts.assign(test_counts);

  VectorShort short_counts;
  short_counts.assign(test_counts);

  VectorUllong ullong_counts;
  ullong_counts.assign(test_counts);

  VectorUlong ulong_counts;
  ulong_counts.assign(test_counts);

  VectorUshort ushort_counts;
  ushort_counts.assign(test_counts);

  if (!hist6.setCounts(byte_counts) || !hist6.counts_d.eq(test_counts)) {
    test_counts.debug(L"expected counts");
    hist6.counts_d.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setCounts(double_counts) || !hist6.counts_d.eq(test_counts)) {
    test_counts.debug(L"expected counts");
    hist6.counts_d.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setCounts(float_counts) || !hist6.counts_d.eq(test_counts)) {
    test_counts.debug(L"expected counts");
    hist6.counts_d.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setCounts(llong_counts) || !hist6.counts_d.eq(test_counts)) {
    test_counts.debug(L"expected counts");
    hist6.counts_d.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setCounts(long_counts) || !hist6.counts_d.eq(test_counts)) {
    test_counts.debug(L"expected counts");
    hist6.counts_d.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setCounts(short_counts) || !hist6.counts_d.eq(test_counts)) {
    test_counts.debug(L"expected counts");
    hist6.counts_d.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setCounts(ullong_counts) || !hist6.counts_d.eq(test_counts)) {
    test_counts.debug(L"expected counts");
    hist6.counts_d.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setCounts(ulong_counts) || !hist6.counts_d.eq(test_counts)) {
    test_counts.debug(L"expected counts");
    hist6.counts_d.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!hist6.setCounts(ushort_counts) || !hist6.counts_d.eq(test_counts)) {
    test_counts.debug(L"expected counts");
    hist6.counts_d.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  //---------------------------------------------------------------------------
  //
  // 3. class-specific public methods:
  //     get methods
  //
  //---------------------------------------------------------------------------

  // set indentation
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing class-specific public methods: get methods...\n");
    Console::increaseIndention();
  }

  // test getMode
  //
  if (hist5.getMode() != CENTERS) {
    return Error::handle(name(), L"getMode", Error::TEST, __FILE__,
			 __LINE__);
  }

  // test getBins
  //
  VectorDouble tmp;
  if (!hist6.getBins(tmp) || !tmp.almostEqual(test_edges_bins)) {
    return Error::handle(name(), L"getBins", Error::TEST, __FILE__,
			 __LINE__);
  }

  // test getCounts
  //
  hist6.setCounts(test_counts);

  hist6.getCounts(byte_counts);
  hist6.getCounts(double_counts);
  hist6.getCounts(float_counts);
  hist6.getCounts(llong_counts);
  hist6.getCounts(long_counts);
  hist6.getCounts(short_counts);
  hist6.getCounts(ullong_counts);
  hist6.getCounts(ulong_counts);
  hist6.getCounts(ushort_counts);

  VectorDouble tmp_vec;
  
  if (!tmp_vec.assign(byte_counts) || !hist6.counts_d.eq(tmp_vec)) {
    test_counts.debug(L"expected counts");
    byte_counts.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!tmp_vec.assign(double_counts) || !hist6.counts_d.eq(tmp_vec)) {
    test_counts.debug(L"expected counts");
    double_counts.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!tmp_vec.assign(float_counts) || !hist6.counts_d.eq(tmp_vec)) {
    test_counts.debug(L"expected counts");
    float_counts.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!tmp_vec.assign(llong_counts) || !hist6.counts_d.eq(tmp_vec)) {
    test_counts.debug(L"expected counts");
    llong_counts.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!tmp_vec.assign(long_counts) || !hist6.counts_d.eq(tmp_vec)) {
    test_counts.debug(L"expected counts");
    long_counts.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!tmp_vec.assign(short_counts) || !hist6.counts_d.eq(tmp_vec)) {
    test_counts.debug(L"expected counts");
    short_counts.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!tmp_vec.assign(ullong_counts) || !hist6.counts_d.eq(tmp_vec)) {
    test_counts.debug(L"expected counts");
    ullong_counts.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!tmp_vec.assign(ulong_counts) || !hist6.counts_d.eq(tmp_vec)) {
    test_counts.debug(L"expected counts");
    ulong_counts.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }
  if (!tmp_vec.assign(ushort_counts) || !hist6.counts_d.eq(tmp_vec)) {
    test_counts.debug(L"expected counts");
    ushort_counts.debug(L"actual counts");
    return Error::handle(name(), L"setCounts", Error::TEST, __FILE__,
			 __LINE__);
  }

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }

  //---------------------------------------------------------------------------
  //
  // 4. class-specific public methods:
  //     compute methods
  //
  //---------------------------------------------------------------------------

  // set indentation
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing class-specific public methods: get methods...\n");
    Console::increaseIndention();
  }

  // create a vector of expected counts for comparison
  //
  VectorDouble expected_counts;
  VectorDouble actual_counts;

  // test compute methods with centered bins
  //
  
  // test a new histogram in CENTERS mode for each data type
  //
  expected_counts.assign(L"2, 1, 1, 1, 1, 1, 1, 1, 1, 1");
  hist6.clear();
  hist6.setMode(CENTERS);
  VectorByte byte_data;
  byte_data.assign(test_data);
  hist6.compute(byte_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  VectorDouble double_data;
  double_data.assign(test_data);
  hist6.compute(double_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  VectorFloat float_data;
  float_data.assign(test_data);
  hist6.compute(float_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  VectorLlong llong_data;
  llong_data.assign(test_data);
  hist6.compute(llong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  VectorLong long_data;
  long_data.assign(test_data);
  hist6.compute(long_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  VectorShort short_data;
  short_data.assign(test_data);
  hist6.compute(short_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  VectorUllong ullong_data;
  ullong_data.assign(test_data);
  hist6.compute(ullong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  VectorUlong ulong_data;
  ulong_data.assign(test_data);
  hist6.compute(ulong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  VectorUshort ushort_data;
  ushort_data.assign(test_data);
  hist6.compute(ushort_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  // test a histogram with preset bins in CENTERS mode for each data type
  //
  expected_counts.assign(L"2, 1, 1, 1, 1, 1, 1, 1, 1, 1");
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(byte_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(double_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(float_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(llong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(long_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(short_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(ullong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(ulong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(ushort_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  // test a histogram with a specified number of bins in CENTERS mode for
  // all data types
  //
  expected_counts.assign(L"2, 1, 1, 1, 1, 1, 1, 1, 1, 1");
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.compute(byte_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.compute(double_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.compute(float_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.compute(llong_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.compute(long_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.compute(short_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.compute(ullong_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.compute(ulong_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  hist6.compute(ushort_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  // test a histogram with specified bins in CENTERS mode for each data type
  //
  expected_counts.assign(L"1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1");
  VectorDouble all_bins(L"0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10");
  hist6.clear();
  hist6.setMode(CENTERS);
  byte_bins.assign(all_bins);
  hist6.compute(byte_data, byte_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  double_bins.assign(all_bins);
  hist6.compute(double_data, double_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  float_bins.assign(all_bins);
  hist6.compute(float_data, float_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  llong_bins.assign(all_bins);
  hist6.compute(llong_data, llong_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  long_bins.assign(all_bins);
  hist6.compute(long_data, long_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  short_bins.assign(all_bins);
  hist6.compute(short_data, short_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  ullong_bins.assign(all_bins);
  hist6.compute(ullong_data, ullong_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  ulong_bins.assign(all_bins);
  hist6.compute(ulong_data, ulong_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  ushort_bins.assign(all_bins);
  hist6.compute(ushort_data, ushort_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  // update a histogram with specified bins in CENTERS mode for each data type
  //
  expected_counts.assign(L"2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2");
  hist6.clear();
  hist6.setMode(CENTERS);
  byte_bins.assign(all_bins);
  hist6.compute(byte_data, byte_bins);
  hist6.update(byte_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  double_bins.assign(all_bins);
  hist6.compute(double_data, double_bins);
  hist6.update(double_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  float_bins.assign(all_bins);
  hist6.compute(float_data, float_bins);
  hist6.update(float_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  llong_bins.assign(all_bins);
  hist6.compute(llong_data, llong_bins);
  hist6.update(llong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  long_bins.assign(all_bins);
  hist6.compute(long_data, long_bins);
  hist6.update(long_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  short_bins.assign(all_bins);
  hist6.compute(short_data, short_bins);
  hist6.update(short_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  ullong_bins.assign(all_bins);
  hist6.compute(ullong_data, ullong_bins);
  hist6.update(ullong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  ulong_bins.assign(all_bins);
  hist6.compute(ulong_data, ulong_bins);
  hist6.update(ulong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(CENTERS);
  ushort_bins.assign(all_bins);
  hist6.compute(ushort_data, ushort_bins);
  hist6.update(ushort_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }

  // test all compute methods in EDGES mode
  //

  // test a new histogram in EDGES mode for each data type
  //
  test_data.assign(L"0, 1, 2, 3, 4, 5, 6, 7, 8, 9");
  expected_counts.assign(L"1, 1, 1, 1, 1, 1, 1, 1, 1, 1");
  hist6.clear();
  hist6.setMode(EDGES);
  byte_data.assign(test_data);
  hist6.compute(byte_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    hist6.debug(L"hist6");
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  double_data.assign(test_data);
  hist6.compute(double_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  float_data.assign(test_data);
  hist6.compute(float_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  llong_data.assign(test_data);
  hist6.compute(llong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  long_data.assign(test_data);
  hist6.compute(long_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  short_data.assign(test_data);
  hist6.compute(short_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  ullong_data.assign(test_data);
  hist6.compute(ullong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  ulong_data.assign(test_data);
  hist6.compute(ulong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  ushort_data.assign(test_data);
  hist6.compute(ushort_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  // test a histogram with preset bins in EDGES mode for each data type
  //
  expected_counts.assign(L"1, 1, 1, 1, 1, 1, 1, 1, 1, 1");
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(byte_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(double_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(float_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(llong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(long_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(short_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(ullong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(ulong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.setBins(test_data.min(), test_data.max(), DEF_NUM_BINS);
  hist6.compute(ushort_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  // test a histogram with a specified number of bins in EDGES mode for
  // all data types
  //
  expected_counts.assign(L"1, 1, 1, 1, 1, 1, 1, 1, 1, 1");
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.compute(byte_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.compute(double_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.compute(float_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.compute(llong_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.compute(long_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.compute(short_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.compute(ullong_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.compute(ulong_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  hist6.compute(ushort_data, DEF_NUM_BINS);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  // test a histogram with specified bins in EDGES mode for each data type
  //
  test_data.assign(L"0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11");
  expected_counts.assign(L"1, 1, 1, 1, 1, 1, 1, 1, 1, 1");
  all_bins.assign(L"0, 1, 2, 3, 4, 5, 6, 7, 8, 9");
  hist6.clear();
  hist6.setMode(EDGES);
  byte_bins.assign(all_bins);
  hist6.compute(byte_data, byte_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  double_bins.assign(all_bins);
  hist6.compute(double_data, double_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  float_bins.assign(all_bins);
  hist6.compute(float_data, float_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  llong_bins.assign(all_bins);
  hist6.compute(llong_data, llong_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  long_bins.assign(all_bins);
  hist6.compute(long_data, long_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  short_bins.assign(all_bins);
  hist6.compute(short_data, short_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  ullong_bins.assign(all_bins);
  hist6.compute(ullong_data, ullong_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  ulong_bins.assign(all_bins);
  hist6.compute(ulong_data, ulong_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  ushort_bins.assign(all_bins);
  hist6.compute(ushort_data, ushort_bins);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  // update a histogram with specified bins in EDGES mode for each data type
  //
  expected_counts.assign(L"2, 2, 2, 2, 2, 2, 2, 2, 2, 2");
  hist6.clear();
  hist6.setMode(EDGES);
  byte_bins.assign(all_bins);
  hist6.compute(byte_data, byte_bins);
  hist6.update(byte_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  double_bins.assign(all_bins);
  hist6.compute(double_data, double_bins);
  hist6.update(double_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  float_bins.assign(all_bins);
  hist6.compute(float_data, float_bins);
  hist6.update(float_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  llong_bins.assign(all_bins);
  hist6.compute(llong_data, llong_bins);
  hist6.update(llong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  long_bins.assign(all_bins);
  hist6.compute(long_data, long_bins);
  hist6.update(long_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  short_bins.assign(all_bins);
  hist6.compute(short_data, short_bins);
  hist6.update(short_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  ullong_bins.assign(all_bins);
  hist6.compute(ullong_data, ullong_bins);
  hist6.update(ullong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  ulong_bins.assign(all_bins);
  hist6.compute(ulong_data, ulong_bins);
  hist6.update(ulong_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  hist6.clear();
  hist6.setMode(EDGES);
  ushort_bins.assign(all_bins);
  hist6.compute(ushort_data, ushort_bins);
  hist6.update(ushort_data);
  hist6.getCounts(actual_counts);
  if (!actual_counts.eq(expected_counts)) {
    actual_counts.debug(L"actual counts");
    expected_counts.debug(L"expected counts");
    return Error::handle(name(), L"compute", Error::TEST, __FILE__,
			 __LINE__);
  }
  

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }

  //---------------------------------------------------------------------------
  //
  // 5. class-specific public methods:
  //     normalization methods
  //
  //---------------------------------------------------------------------------

  // set indentation
  //
  if (level_a > Integral::NONE) {
    Console::put(L"testing class-specific public methods: get methods...\n");
    Console::increaseIndention();
  }

  // test pdf
  //
  VectorDouble expected_pdf(L"0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1");
  VectorDouble actual_pdf;

  if (!hist6.pdf(actual_pdf) ||
      !actual_pdf.almostEqual(expected_pdf)) {
    actual_pdf.debug(L"actual pdf");
    expected_pdf.debug(L"expected pdf");
    return Error::handle(name(), L"pdf", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  // test cdf
  //
  VectorDouble expected_cdf(L"0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1");
  VectorDouble actual_cdf;

  if (!hist6.cdf(actual_cdf) ||
      !actual_cdf.almostEqual(expected_cdf)) {
    actual_cdf.debug(L"actual cdf");
    expected_cdf.debug(L"expected cdf");
    return Error::handle(name(), L"cdf", Error::TEST, __FILE__,
			 __LINE__);
  }
  
  
  
  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }

  //---------------------------------------------------------------------------
  //
  // 6. print completion message
  //
  //---------------------------------------------------------------------------

  // reset indentation
  //
  if (level_a > Integral::NONE) {
    Console::decreaseIndention();
  }
  
  if (level_a > Integral::NONE) {
    SysString output(L"diagnostics passed for class ");
    output.concat(name());
    output.concat(L"\n");
    Console::put(output);
  }

  // exit gracefully
  //
  return true;
}
