// file: $isip/class/stat/SupportVectorModel/svm_04.cc
// version: $Id: svm_04.cc,v 1.1 2002/10/26 19:45:55 jelinek Exp $
//

// isip include files
//
#include "SupportVectorModel.h"

// method: sofSize
//
// arguments: none
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
long SupportVectorModel::sofSize() const {

  // start with the space required for the algorithm name
  //
  long bytes = ALGO_MAP.elementSofSize();
  bytes += IMPL_MAP.elementSofSize();
    
  // add the space required for data memebers
  //
  bytes += support_vectors_d.sofSize() + kernels_d.sofSize()
    + alphas_d.sofSize() + bias_d.sofSize();

  if (implementation_d == SIGMOID) {
    bytes += sigmoid_d.sofSize();
  }

  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: boolean value indicating status
//
// this method has the object write itself to an Sof file
//
boolean SupportVectorModel::write(Sof& sof_a, long tag_a,
			     const String& name_a) const {

  long obj_size = 0;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: boolean value indicating status
//
// this method has the object write itself to an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean SupportVectorModel::writeData(Sof& sof_a,
				      const String& pname_a) const {
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm and implementation name
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (long)algorithm_d);
  IMPL_MAP.writeElementData(sof_a,
			    PARAM_IMPLEMENTATION, (long)implementation_d);

  // write the vector of means
  //
  support_vectors_d.writeData(sof_a, PARAM_SUPPORT_VECTORS);
  kernels_d.writeData(sof_a, PARAM_KERNELS);
  alphas_d.writeData(sof_a, PARAM_ALPHAS);
  bias_d.writeData(sof_a, PARAM_BIAS);

  if (implementation_d == SIGMOID) {
    sigmoid_d.writeData(sof_a, PARAM_SIGMOID);
  }

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}






