#!/bin/sh

# file: $isip/login/shell_bashrc.sh
#
# this is the standard .bashrc file.
# a user should execute this file at the top of their personal
# bashrc file.
#

# important definitions
#
PROGRAM_NAME="$0";

DEFAULT_ISIP_TERM="vt100";

DEFAULT_ISIP_ROOT="/isip/tools";
DEFAULT_ISIP="$DEFAULT_ISIP_ROOT/ifc";
DEFAULT_ISIP_PROTO="$DEFAULT_ISIP_ROOT/proto";

DEFAULT_ISIP_SERVER="isip002";
DEFAULT_ISIP_SERVER_INFO="nntp.msstate.edu";
DEFAULT_ISIP_SERVER_MAIL="isip01";
DEFAULT_ISIP_DOMAIN="isip.msstate.edu";
DEFAULT_ISIP_ORGANIZATION="institute for signal and information processing";

DEFAULT_ISIP_UMASK="002";
DEFAULT_ISIP_HISTSIZE="1024";
DEFAULT_ISIP_INDEX_FILE="$HOME/.isip_index";

DEFAULT_ISIP_EDITOR="emacs";
DEFAULT_ISIP_EDITOR_INIT_FILE="$HOME/login/editor_emacs_init.el";
DEFAULT_ISIP_PRINTER="blue";

DEFAULT_ISIP_XINITRC="$HOME/login/x11_xinitrc.sh";
DEFAULT_ISIP_TWMRC="$HOME/login/x11_twmrc.sh";
DEFAULT_ISIP_XTERM="xterm";

DEFAULT_ISIP_AUDIO_DEVICE="isip003:0";
DEFAULT_ISIP_SCANNER="isip003:/dev/ss0";

DEFAULT_CVSROOT="$DEFAULT_ISIP_ROOT/cvsroot";
DEFAULT_ISIP_VERSION="ver_0"

#------------------------------------------------------------------------------
# general isip stuff goes here:
#  this section usually sets the path, defines environment variables, etc.
#------------------------------------------------------------------------------

# define the location of the isip tools
#
ISIP=${ISIP:-"$DEFAULT_ISIP"};
if (test ! -d "$ISIP") then
    echo "<$PROGRAM_NAME> the isip environment is not properly set up!";
    exit -1;
fi

isip="$ISIP";
export ISIP isip;

# define the location of the ISIP prototype system
#
ISIP_PROTO=${ISIP_PROTO:-"$DEFAULT_ISIP_PROTO"};
if (test ! -d "$ISIP_PROTO") then
    echo "<$PROGRAM_NAME> the isip prototype environment is not properly set up!";
fi

export ISIP_PROTO;

# define environmental variables dependent upon $ISIP
#
DEFAULT_ISIP_WORDLIST="$ISIP/login/spell_ispell_wordlist.sh"
DEFAULT_ISIP_DEVEL=$ISIP

# define the location of the isip development environment. user's can
# override this definition if they set ISIP_DEVEL in their .bashrc
# file before this file is sourced.
#
ISIP_DEVEL=${ISIP_DEVEL:-"$DEFAULT_ISIP_DEVEL"};
if (test ! -d "$ISIP_DEVEL") then
    echo "<$PROGRAM_NAME> cannot find development environment!";
    echo "<$PROGRAM_NAME> using production directory $ISIP!";
    ISIP_DEVEL=$ISIP;
fi
export ISIP_DEVEL;

# define the type of machine and operating system being used:
# note: uname is not terribly portable. the command below is not backwardly
# compatible with all unix operating systems.
#
ISIP_BINARY=`sh $ISIP/config.guess`;
export ISIP_BINARY;

# define the home directories for some important tools:
#  note that these environment variables (containing the base location of the
#  tool) are required by other tools in the isip environment).
#
OPENWINHOME="/usr/openwin";
CLASSPATH=".:/usr/local/netscape:/usr/local/netscape/java_301:/usr/local/lib/java/mm.mysql.jar:$ISIP/lib/java/varmint.jar";
MOZILLA_HOME="/usr/local/netscape";
FMHOME="/usr/local/frame";
XGOHOME="/usr/local/xgo";

# export only the variables that really need to be exported
#
export OPENWINHOME CLASSPATH MOZILLA_HOME FMHOME XGOHOME;

# define paths which make the java plugin work under netscape communicator
#
#JAVA_PLUGIN_PATH="/usr/local/java/plugins"
JAVA_PLUGIN_PATH="/usr/local/java/jre/plugin"
NPX_PLUGIN_PATH="$JAVA_PLUGIN_PATH:/usr/local/netscape/plugins"
NPX_JRE_PATH="/usr/local/java/jre"
export JAVA_PLUGIN_PATH NPX_PLUGIN_PATH NPX_JRE_PATH;

# initialize the path for the isip tools starting with
# the current working directory
#
PATH=".";

# add the isip script and architecture dependent binary directories to
# the path. if ISIP_DEVEL is equal to ISIP, only add them once. if
# they are different, then add ISIP_DEVEL before ISIP.
#
if (test "$ISIP" = "$ISIP_DEVEL") then
    PATH="$PATH:$ISIP_DEVEL/bin/scripts:$ISIP/bin/scripts";
    PATH="$PATH:$ISIP/bin/$ISIP_BINARY";
else
    PATH="$PATH:$ISIP_DEVEL/bin/scripts:$ISIP/bin/scripts";
    PATH="$PATH:$ISIP_DEVEL/bin/$ISIP_BINARY:$ISIP/bin/$ISIP_BINARY";
fi

# add the isip prototype scripts and architecture dependent binary
# directories to the path.
#
PATH="$PATH:$ISIP_PROTO/bin/scripts";
PATH="$PATH:$ISIP_PROTO/bin/$ISIP_BINARY";

# add java with swing
# note: since /usr/bin also has an older version of java,
# this line must precede /usr/bin
#
PATH="$PATH:/usr/local/java/bin";

# give /usr/local/bin priority (this now includes GNU stuff)
#
PATH="$PATH:/usr/local/bin";

# add the system-level directories - give GNU priority
#
PATH="$PATH:/usr/sbin:/usr/bin:/usr/ccs/bin:/usr/ucb";
PATH="$PATH:$OPENWINHOME/bin";
PATH="$PATH:/usr/local/tk/bin";

# add some important packages
#
PATH="$PATH:$FMHOME/bin";
PATH="$PATH:/usr/local/netaudio/bin";
PATH="$PATH:/usr/local/sphere/bin";
PATH="$PATH:/usr/local/sctk/bin";
PATH="$PATH:/usr/local/srilm/bin";
PATH="$PATH:/usr/local/grace/bin";
PATH="$PATH:/usr/local/netscape/bin";
PATH="$PATH:/usr/local/openscan/bin";
PATH="$PATH:/usr/local/frame/bin";
PATH="$PATH:/usr/local/acrobat/bin";
PATH="$PATH:/usr/local/acroread/bin";
PATH="$PATH:/usr/local/fsm/bin";
PATH="$PATH:/usr/local/go_global/bin";
PATH="$PATH:/usr/local/ncd";
PATH="$PATH:/usr/local/powerchute";
PATH="$PATH:/usr/local/xdm/";
PATH="$PATH:/usr/local/xgo/bin";

# ssh for Solaris 2.7
#
PATH="$PATH:/opt/net/bin";

# sri lm toolkit
#
PATH="$PATH:/usr/local/sri/bin";

# export the path
#
export PATH;

# create a corresponding man path - give local customizations
# priority
#
DEFAULT_MANPATH="/usr/local/fsm/man";
DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/man:/usr/local/man:$OPENWINHOME/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/pbmplus/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/acroread/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/frame/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/httpd/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/powerchute/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/openscan/scanner/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/traceroute/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/tcl/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/tk/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/tm/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/go_global/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/xgo/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/unzip/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/admin/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/backups/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/login/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/phone/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/linkon/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/mail_aliases/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/nas/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/ncd/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/netscape/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/sphere/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/sctk/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/sox/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/srilm/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/top/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/rcs/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/fping/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/ftptool/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/java/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/openwin/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/proctool/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/xdm/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/netaudio/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/xmgr/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/xv/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/sysinfo/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/xspread/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/perl/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/xkeycaps/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/xpdf/man";
#DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/xrsh/man";
DEFAULT_MANPATH="$DEFAULT_MANPATH:/usr/local/grace/man";

# set the actual MANPATH
#
MANPATH=${MANPATH:-$DEFAULT_MANPATH};
export MANPATH;

# set xwindows library path: we finally decide to stick with Sun's openwin
#
LD_LIBRARY_PATH="/usr/openwin/lib:/usr/local/lib";
export LD_LIBRARY_PATH;

# define the isip libraries
#
ISIP_LIBS="-L$ISIP_DEVEL/lib/$ISIP_BINARY \
           -l_asr \
	   -l_pr \
	   -l_search \
           -l_sp \
           -l_algo \
           -l_stat \
	   -l_mmedia \
           -l_shell \
	   -l_numeric \
           -l_math_matrix \
           -l_math_vector \
           -l_math_scalar \
           -l_io \
	   -l_system \
           -lm";
export ISIP_LIBS;

# define the location of /tmp
#
ISIP_TMPDIR="/tmp";
export ISIP_TMPDIR;

# define compiler related things:
#  (1) the make file tool and priority
#  (2) the location of include files 
#  (3) the C++ compiler
#
ISIP_MAKE="nice -5 make";
ISIP_INCLUDE="-I$ISIP_DEVEL/include";
ISIP_CPLUS_COMPILER="g++";
ISIP_CPLUS_OPTIMIZE="-O2";
export ISIP_MAKE ISIP_INCLUDE ISIP_CPLUS_COMPILER ISIP_CPLUS_OPTIMIZE;

# declare the location of a scratch file to be used to index tools
#
ISIP_INDEX_FILE=$DEFAULT_ISIP_INDEX_FILE;
export ISIP_INDEX_FILE;

# define some global error codes to be used for shellscripts
#
ISIP_NOERROR="0";
ISIP_ERROR="1";

#------------------------------------------------------------------------------
# general user-level stuff goes here:
#  this section usually defines things that affect the user's interactive
#  environment.
#------------------------------------------------------------------------------

# set LOGNAME: since this is expensive, we only want to do it once
#
# define a username variable:
#  doing this in a standard way is becoming increasingly difficult
#
ISIP_USERNAME=`/usr/bin/id | cut -f2 -d'(' | cut -f1 -d')'`;
LOGNAME="$ISIP_USERNAME";
export ISIP_USERNAME LOGNAME;

# set default protection
#
umask $DEFAULT_ISIP_UMASK;

# set core file size to something small (realistic)
#
ulimit -c 0;

# set history length
#
HISTSIZE=$DEFAULT_ISIP_HISTSIZE;

# prevent storing adjacent identical lines in the bash history
#
history_control="ignoredups";

# set the preferred printer
#  note that system V seems to use lpdest
#
PRINTER=${PRINTER:-"$DEFAULT_ISIP_PRINTER"};
LPDEST="$PRINTER";
export PRINTER LPDEST;

# set audio system device number
#  make this compatible with the datlink software
#
ISIP_AUDIO_DEVICE=${ISIP_AUDIO_DEVICE:-"$DEFAULT_ISIP_AUDIO_DEVICE"};
AUDIO_DEVICE="$ISIP_AUDIO_DEVICE";
export ISIP_AUDIO_DEVICE AUDIO_DEVICE;

# set the scanner device number
#  make this compatible with the apunix scanner
#
ISIP_SCANNER=${ISIP_SCANNER:-"$DEFAULT_ISIP_SCANNER"};
SCANNER="$ISIP_SCANNER";
export ISIP_SCANNER SCANNER;

# create the mail environment:
#  since several tools require access to the mail aliases,
#  the mail environment has been isolated.
#
. $ISIP/login/mail_environment.sh;

# set up some useful host names
#
ISIP_SERVER=${ISIP_SERVER:-"$DEFAULT_ISIP_SERVER"};
ISIP_SERVER_INFO=${ISIP_SERVER_INFO:-"$DEFAULT_ISIP_SERVER_INFO"};
ISIP_SERVER_MAIL=${ISIP_SERVER_MAIL:-"$DEFAULT_ISIP_SERVER_MAIL"};
ISIP_HOST=`uname -n | cut -d. -f1`;
export ISIP_SERVER_INFO ISIP_SERVER_MAIL;
export ISIP_SERVER ISIP_HOST; 

# set up some definitions for information servers
#
ISIP_DOMAIN=$DEFAULT_ISIP_DOMAIN;
ISIP_ORGANIZATION=$DEFAULT_ISIP_ORGANIZATION;
export ISIP_DOMAIN ISIP_ORGANIZATION;

# set the editor
#
EDITOR=${EDITOR:-"$DEFAULT_ISIP_EDITOR"};
ISIP_EDITOR_INIT_FILE=$DEFAULT_ISIP_EDITOR_INIT_FILE;
export EDITOR ISIP_EDITOR_INIT_FILE;

# set x environment variables
#
ISIP_XINITRC="$DEFAULT_ISIP_XINITRC";
ISIP_TWMRC="$DEFAULT_ISIP_TWMRC";
export ISIP_XINITRC ISIP_TWMRC;

XINITRC="$ISIP_XINITRC";	
TWMRC="$ISIP_TWMRC";
XAPPLRESDIR="$DEFAULT_XAPPLRESDIR";
export XINITRC TWMRC XAPPLRESDIR;

# set their default xterm
#
ISIP_XTERM="$DEFAULT_ISIP_XTERM";
export ISIP_XTERM;

# setup version control
#
CVSROOT=${CVSROOT:-"$DEFAULT_CVSROOT"};
ISIP_VERSION=${ISIP_VERSION:-"$DEFAULT_ISIP_VERSION"};
CVSREAD=1
CVSEDITOR="emacs -l $ISIP_EDITOR_INIT_FILE"
export CVSROOT ISIP_VERSION CVSREAD CVSEDITOR;

# define a group dictionary
#
WORDLIST="$DEFAULT_ISIP_WORDLIST";
export WORDLIST;

REMOTE_USERNAME_00=${REMOTE_USERNAME_00:-"$ISIP_USERNAME"};
REMOTE_USERNAME_01=${REMOTE_USERNAME_01:-"$ISIP_USERNAME"};
REMOTE_USERNAME_02=${REMOTE_USERNAME_02:-"$ISIP_USERNAME"};
export REMOTE_USERNAME_00 REMOTE_USERNAME_01 REMOTE_USERNAME_02;

#------------------------------------------------------------------------------
# interactive stuff goes here:
#  the commands below are to be executed for interactive shells only.
#  to determine if the shell is interactive, test for PS1.
#------------------------------------------------------------------------------
if (test -n "$PS1") then

    # update prompt
    #
    PS1=$ISIP_HOST"_[$SHLVL]: ";
    PS2=$ISIP_HOST"_[$SHLVL]? ";
    export PS1 PS2;

    # set some useful stuff for non-X terminal logins - a non-X
    # term has no DISPLAY set automatically.
    #

    if (test -z "$DISPLAY") then

	# set terminal type to the lowest common denominator - a vt100
	# (crude, but reliable)
	#
	TERM=${TERM:-"$DEFAULT_ISIP_TERM"};
	export TERM;

	# set USER, which for some reason does not always get set
	#
	USER="$ISIP_USERNAME";
	export USER;
    fi

    # setup dmalloc
    #
    #function isip_dmalloc { eval dmalloc -b -f $ISIP/login/debugger_gdb_dmallocrc.text $*;}

    #--------------------------------------------------------------------------
    # define some extremely useful commands:
    #
    # the commands defined here should satisfy two requirements -
    #  (1) no standard unix alias exist for these commands
    #  (2) the command name length is limited to two characters
    #
    # all other commands in the isip environment are prefaced with "isip_".
    # since some of the commands defined below are frequently used, they
    # are allowed to exist without a standard prefix. the downside of this
    # is that they might cause a conflict with commands from another
    # environment (for example, several years ago, the show file command
    # "sf" was eliminated because it caused conflicts with some third
    # party environments.
    #--------------------------------------------------------------------------

    # define directory commands:
    #  due to the nature of shell programming, these must be defined using
    #  functions, so that they can be part of the user level environment
    #  these must also be in the bashrc so that they are available for
    #  any shell.
    #
    p  () { echo $PWD; }
    sd () { pushd $1 > /dev/null; echo $PWD; }
    td () { pushd    > /dev/null; echo $PWD; }
    pd () { popd  $1 > /dev/null; echo $PWD; }

    # define useful short unix commands:
    d  () { /usr/bin/ls -acblF $*; }
    f  () { finger -l $*; }
    m  () { more $*; }

    # make sure the tty settings are reasonable
    #  in Solaris 2.4, and various versions of X, this is required
    #  to make backspace and delete work properly. the options selected
    #  here were the result of much experimentation. we used to use stty sane,
    #  but that forces a 7-bit stream, which breaks some modem/tty connections.
    #
    stty erase \^? intr \^c echoe;

    # end of interactive shell
fi

#
# end of file
