#!/bin/sh
#
# file: $isip/login/x11_twmrc.sh
#
# the isip-standard twm init file.
#
# notes:
#
# 09/07/00 (bill chapman): change isip005 to isip004
# 07/08/00 (joe picone): modified for the new isip environment
# 07/15/95 (joe picone): first version
#

#------------------------------------------------------------------------------
# this section establishes basic twm characteristics
#------------------------------------------------------------------------------
# show the iconmanager
#
ShowIconManager
IconManagerGeometry	"156x10-0+20"

# define fonts
#
IconFont		"6x13"
MenuFont		"6x13"
ResizeFont		"6x13"
TitleFont		"6x13"
IconManagerFont		"6x13"

# define window behaviors
#
NoGrabServer
UsePPosition		"non-zero"
DecorateTransients
NoTitleFocus

Color {
    BorderColor		"Black"
    IconBackground	"Black"
    IconBorderColor	"White"
    IconForeground	"White"
    MenuBackground	"White"
    MenuForeground	"Black"
    MenuShadowColor	"Black"
    MenuTitleBackground	"Black"
    MenuTitleForeground	"White"
    TitleBackground	"Black"
    TitleForeground	"White"
}

Pixmaps
    {TitleHighlight "black"}

# define which windows appear with no title
#
NoTitle {
    "TWM Icon Manager"
    "xisipbiff"
    "Xcb"
    "xclock"
}

# define which tools are managed (displayed) by the iconmanager
#
IconifyByUnmapping

IconManagerDontShow {
    "xisipbiff"
    "Xcb"
    "xclock"
}

#------------------------------------------------------------------------------
# this section binds the mouse's button keys
#  this was changed on 6/21/95 to make the environment more user-friendly
#  to a two-button mouse
#------------------------------------------------------------------------------
# mouse in root window
#
Button1		=	: root		: f.menu "window operations"
Button2		=	: root		: f.menu "window operations"
Button3		=	: root		: f.menu "utilities"

# mouse on a title bar
#
Button1		=	: title		: f.move
Button2		=	: title		: f.menu "modify window"
Button3		=	: title		: f.menu "modify window"

# other (shift)
#
Button1		= s	: all		: f.zoom
Button2		= s	: all		: f.lower
Button3		= s	: all		: f.raise

#------------------------------------------------------------------------------
# this section defines the window operations menu hieararchy.
#------------------------------------------------------------------------------

# define the first level of window operations
#
Menu "window operations" {
    "window operations"	f.title
    "xterm local"	f.exec "isip_xterm -tool xterm &"
    "xterm server"	f.menu "xterm server"
    "xterm workstation"	f.menu "xterm workstation"
    "xterm auxiliary"	f.menu "xterm auxiliary"
    "kill window"	f.menu "kill_window"
    "modify window"	f.menu "modify_window"
}

# define the second level for each choice
#
Menu "xterm server" {
    "xterm servers" f.title
    "server"	f.exec "xrsh $ISIP_SERVER /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip223"	f.exec "xrsh isip223 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip222"	f.exec "xrsh isip222 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip221"	f.exec "xrsh isip221 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip220"	f.exec "xrsh isip220 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip219"	f.exec "xrsh isip219 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip218"	f.exec "xrsh isip218 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip217"	f.exec "xrsh isip217 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip216"	f.exec "xrsh isip216 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip215"	f.exec "xrsh isip215 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip214"	f.exec "xrsh isip214 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip213"	f.exec "xrsh isip213 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip212"	f.exec "xrsh isip212 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip211"	f.exec "xrsh isip211 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip210"	f.exec "xrsh isip210 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip209"	f.exec "xrsh isip209 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip208"	f.exec "xrsh isip208 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip207"	f.exec "xrsh isip207 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip206"	f.exec "xrsh isip206 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip205"	f.exec "xrsh isip205 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip204"	f.exec "xrsh isip204 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip203"	f.exec "xrsh isip203 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip202"	f.exec "xrsh isip202 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip201"	f.exec "xrsh isip201 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
}

Menu "xterm workstation" {
    "xterm workstations" f.title
    "isip115"	f.exec "xrsh isip115 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip114"	f.exec "xrsh isip114 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip113"	f.exec "xrsh isip113 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip112"	f.exec "xrsh isip112 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip111"	f.exec "xrsh isip111 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip110"	f.exec "xrsh isip110 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip109"	f.exec "xrsh isip109 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip108"	f.exec "xrsh isip108 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip107"	f.exec "xrsh isip107 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip106"	f.exec "xrsh isip106 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip105"	f.exec "xrsh isip105 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip104"	f.exec "xrsh isip104 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip103"	f.exec "xrsh isip103 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip102"	f.exec "xrsh isip102 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip101"	f.exec "xrsh isip101 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip100"	f.exec "xrsh isip100 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
}

Menu "xterm auxiliary" {
    "xterm auxiliary" f.title
    "isip006"	f.exec "xrsh isip006 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip005"	f.exec "xrsh isip005 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip004"	f.exec "xrsh isip004 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip003"	f.exec "xrsh isip003 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip002"	f.exec "xrsh isip002 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
    "isip200"	f.exec "xrsh isip200 /isip/tools/ifc/bin/scripts/isip_xterm -tool xterm &"
}

Menu "xterm remote" {
    "xterm remote" f.title
    "ra"	f.exec "ssh -l $REMOTE_USERNAME_00 -f ra /usr/openwin/bin/xterm -name xterm_on_ra -title xterm_on_ra"
    "leto.ece"	f.exec "ssh -l $REMOTE_USERNAME_01 -f leto.ece /usr/openwin/bin/xterm -name xterm_on_leto -title xterm_on_leto"
    "hoth0.ece"	f.exec "ssh -l $REMOTE_USERNAME_01 -f hoth0.ece /usr/openwin/bin/xterm -name xterm_on_hoth0 -title xterm_on_hoth0"
    "hoth1.ece"	f.exec "ssh -l $REMOTE_USERNAME_01 -f hoth1.ece /usr/openwin/bin/xterm -name xterm_on_hoth1 -title xterm_on_hoth1"
    "hoth2.ece"	f.exec "ssh -l $REMOTE_USERNAME_01 -f hoth2.ece /usr/openwin/bin/xterm -name xterm_on_hoth2 -title xterm_on_hoth2"
    "disney.cs"	f.exec "ssh -l $REMOTE_USERNAME_02 -f disney.cs /usr/openwin/bin/xterm -name xterm_on_disney -title xterm_on_disney"
}

Menu "emacs server" {
    "emacs on server" f.title
    "server"	f.exec "xrsh $ISIP_SERVER /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip223"	f.exec "xrsh isip223 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip222"	f.exec "xrsh isip222 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip221"	f.exec "xrsh isip221 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip220"	f.exec "xrsh isip220 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip219"	f.exec "xrsh isip219 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip218"	f.exec "xrsh isip218 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip217"	f.exec "xrsh isip217 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip216"	f.exec "xrsh isip216 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip215"	f.exec "xrsh isip215 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip214"	f.exec "xrsh isip214 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip213"	f.exec "xrsh isip213 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip212"	f.exec "xrsh isip212 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip211"	f.exec "xrsh isip211 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip210"	f.exec "xrsh isip210 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip209"	f.exec "xrsh isip209 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip208"	f.exec "xrsh isip208 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip207"	f.exec "xrsh isip207 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip206"	f.exec "xrsh isip206 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip205"	f.exec "xrsh isip205 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip204"	f.exec "xrsh isip204 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip203"	f.exec "xrsh isip203 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip202"	f.exec "xrsh isip202 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip201"	f.exec "xrsh isip201 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
}

Menu "emacs workstation" {
    "emacs on workstation" f.title
    "isip115"	f.exec "xrsh isip115 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip114"	f.exec "xrsh isip114 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip113"	f.exec "xrsh isip113 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip112"	f.exec "xrsh isip112 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip111"	f.exec "xrsh isip111 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip110"	f.exec "xrsh isip110 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip109"	f.exec "xrsh isip109 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip108"	f.exec "xrsh isip108 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip107"	f.exec "xrsh isip107 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip106"	f.exec "xrsh isip106 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip105"	f.exec "xrsh isip105 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip104"	f.exec "xrsh isip104 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip103"	f.exec "xrsh isip103 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip102"	f.exec "xrsh isip102 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip101"	f.exec "xrsh isip101 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip100"	f.exec "xrsh isip100 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
}

Menu "emacs auxiliary" {
    "emacs on auxiliary" f.title
    "isip006"	f.exec "xrsh isip006 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip005"	f.exec "xrsh isip005 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip004"	f.exec "xrsh isip004 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip003"	f.exec "xrsh isip003 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
    "isip002"	f.exec "xrsh isip002 /isip/tools/ifc/bin/scripts/isip_e -tool emacs &"
}

Menu "kill_window" {
    "kill window"	f.title
    "delete"		f.delete
    "destroy"		f.destroy
}

Menu "modify_window" {
    "modify window"	f.title
    "iconify"		f.iconify
    "move"		f.move
    "resize"		f.resize
    "refresh"		f.winrefresh
}

#------------------------------------------------------------------------------
# this section defines the utilities menu hieararchy.
#------------------------------------------------------------------------------

# define the first level of utilities
#
Menu "utilities" {
    "utilities"			f.title
    "emacs"			f.menu "emacs"
    "framemaker"		f.menu "framemaker"
    "pdf viewer"		f.menu "pdf viewer"
    "netnews"			f.menu "netnews"
    "netscape"			f.menu "netscape"
    "ftp"			f.menu "ftp"
    "mail"			f.menu "mail"
    "spreadsheet"		f.menu "spreadsheet"
    "calculator"		f.exec "/usr/openwin/bin/calctool &"
    "openscan"			f.exec "xrsh isip003 /isip/tools/ifc/bin/scripts/isip_openscan &"
    "varmint"                   f.menu "varmint"
    "gimp"                      f.menu "gimp"
    "logins"			f.menu "logins"
    "other twm utilities"	f.menu "twm"
    "screen lock"		f.exec "xterm -iconic -ls -exec xlock -remote -mode blank -nice -19 &"
    "exit twm"			f.quit
}

# define the second level for each choice
#
Menu "emacs" {
    "emacs"			f.title
    "local"			f.exec "isip_e -tool emacs &"
    "servers"                   f.menu "emacs server"
    "workstations"              f.menu "emacs workstation"
    "auxiliary"                 f.menu "emacs auxiliary"
}

Menu "framemaker" {
    "framemaker"		f.title
    "isip006"			f.exec "xrsh isip006 /usr/local/frame/bin/maker &"
    "isip004"			f.exec "xrsh isip004 /usr/local/frame/bin/maker &"
    "isip002"			f.exec "xrsh isip002 /usr/local/frame/bin/maker &"
}
	
Menu "pdf viewer" {
    "pdf viewer"		f.title
    "local"			f.exec "acroread &"
    "server"			f.exec "xrsh $ISIP_SERVER /usr/local/bin/acroread &"
}
	
Menu "netnews" {
    "netnews"			f.title
    "local"			f.exec "isip_e -tool emacs -name netnews -f gnus &"
    "server"			f.exec "xrsh $ISIP_SERVER /isip/tools/ifc/bin/scripts/isip_e -tool emacs -name netnews -f gnus &"
}

Menu "netscape" {
    "netscape"			f.title
    "local"			f.exec "netscape -no-about-splash &"
    "server"			f.exec "xrsh $ISIP_SERVER /isip/tools/ifc/bin/scripts/isip_netscape -no-about-splash &"
}

Menu "ftp" {
    "ftp"			f.title
    "local"			f.exec "ftptool &"
    "server"			f.exec "xrsh $ISIP_SERVER /usr/local/ftptool/bin/ftptool &"
}

Menu "mail" {
    "mail"			f.title
    "local"			f.exec "isip_e -tool emacs -name -mail- -f rmail &"
    "server"			f.exec "xrsh $ISIP_SERVER /isip/tools/ifc/bin/scripts/isip_e -tool emacs -name -mail- -f rmail &"
}

Menu "spreadsheet" {
    "spreadsheet"		f.title
    "local"			f.exec "xspread -fn fixed &"
    "server"			f.exec "xrsh $ISIP_SERVER /usr/local/xspread/bin/xspread -fn fixed &"
}

Menu "varmint" {
    "varmint"			f.title
    "local"			f.exec "isip_varmint &"
}

Menu "gimp" {
    "gimp"			f.title
    "local"			f.exec "isip_gimp &"
}

Menu "logins" {
    "logins"			f.title
    "local"			f.exec "isip_xterm -T rlogin &"
    "server"			f.menu "xterm server"
    "workstation"		f.menu "xterm workstation"
    "auxiliary"		        f.menu "xterm auxiliary"
    "remote"		        f.menu "xterm remote"
}

Menu "twm" {
    "other twm utilities"	f.title
    "refresh all"		f.refresh
    "restart twm"		f.restart
    "sort icon manager"		f.sorticonmgr
    "twm version"		f.version
}

#------------------------------------------------------------------------------
# this section defines the modify window menu:
#  this window was added to provide some simple window manipulation functions
#  from the mouse (it emulates a standard function of fvwm)
#------------------------------------------------------------------------------

# define the first level of window operations
#
Menu "modify window" {
    "modify window"		f.title
    "move"			f.move
    "resize"			f.resize
    "iconify"			f.iconify
    "delete"			f.delete
    "destroy"			f.destroy
    "refresh"			f.winrefresh
}

#
# end of file
