#! /usr/local/bin/perl
#
# file: $isip/tools/util/misc/isip_demo/isip_demo
#
$ISIP_HELP_FILE = <<__ISIP_HELP_HERE_FILE__;
name: isip_demo
synopsis: isip_demo -display <X display> [-start|-stop] -url <url>
descr: this program performs one of two tasks depending on the user input. 
       1. if the start option is specified the program kills the screen saver 
          on the specified terminal and starts up a given/default url on 
          the browser.
       2. if the stop option is selected the program kills the browser on the 
          specified terminal and activates the  screen saver. 
example: isip_demo -display ncd15b07.isip.msstate.edu:0.0  -start -url http://www.isip.msstate.edu/data/tour_demo/index.html

options:
 -display: displays the url on a specified terminal.
 -start: kills the screen saver and starts the browser on a specified terminal.
 -stop: kills the browser and starts the screen saver on a specified terminal. 
 -url: allows you to specify the url to be displayed on the browser.

man page: none
__ISIP_HELP_HERE_FILE__

# define the program name
#
@tmp = split(/\//, $0);
$ISIP_PROG = pop(@tmp);

# define command line options
#
$ID_OPT_DISPLAY = "-display";
$ID_OPT_STOP = "-stop";
$ID_OPT_START = "-start";
$ID_URL = "-url";

# define default url
#
$ID_DEFAULT_URL = "http://www.isip.msstate.edu/data/tour_demo/index.html";

# define time to delay after issuing an exit command to give the programs time
# to exit
#
$ID_TIME_DELAY = 30;

# define the path to the needed utilities
#
$ID_PATH = "/usr/local/bin:/usr/openwin/bin";

# define browser
# 
$ID_BROWSER = "netscape";

# define browser geometry
#
$ID_BROWSER_GEOMETRY = "1280x1024--1+-1";

# define screen saver
# 
$ID_SCREEN_SAVER = "xset";

# define path to $isip
#
$ID_ISIP_TOOLS = "/isip/tools";

# define path to the home directory to be used for running the demo
#
$ID_HOME_DIR = "\/home\/demo/";

# set the executable search path
#
$ENV{PATH} = "$ENV{PATH}:$ID_PATH";

# set the path to $isip and $HOME
#
$ENV{isip} = "$ID_ISIP_TOOLS";
$ENV{HOME} = "$ID_HOME_DIR";

# load the command_line module
#
require "$ENV{isip}/lib/scripts/perl/command_line.pm";

# parse the command line
#
my ($display, $stop, $start, $url) = 
    command_line(0, $ID_OPT_DISPLAY, 1, $ID_OPT_STOP, 0,
		 $ID_OPT_START, 0, $ID_URL, 1);

# if display is not specified the default is the users terminal
#
if ($display eq "") {
    $display = $ENV{DISPLAY};
}

# if url is not specified then use the default
#
if ($url eq "") {
    $url = $ID_DEFAULT_URL;
}

# if the stop option is selected
#
if ($stop) {
   
    # if both the stop and start options are selected exit with error
    #
    if ($start) {
        
        # the program is terminated if more that one option is selected 
        #
	isip_die ("you cannot specify both start and stop options at the same time");
    }

    # the browser is closed and the screen saver is activated
    #
    else {

        # close the browser on the specified terminal
        #
	system ("nice -19 $ID_BROWSER -display $display -remote 'exit()' > /dev/null 2>/dev/null");
	sleep $ID_TIME_DELAY;	


	# display the screen saver and turns the functionality on
	#
	system ("nice -19 $ID_SCREEN_SAVER -display $display s activate > /dev/null 2>/dev/null");
	system ("nice -19 $ID_SCREEN_SAVER -display $display s on > /dev/null 2>/dev/null");
    }
}

# if the start option is selected
#
elsif ($start) {
   
    # if both the start and stop option are selected kill the program 
    #
    if ($stop) {

        # the program is terminated if more that one option is selected
        #
	isip_die ("you cannot specify both start and stop options at the
                  same time");
    }
    
    # kill the screen saver and opens the browser with the given url 
    #
    else {

        # opens up a given url on the specified display terminal 
        #
	$browser_open = `nice -19 $ID_BROWSER -display $display -remote 'OpenURL($url)' 2>&1`;
	
	# start the browser if it has not already been started
	#
	if ($browser_open ne "") {

	    # resets the screen saver and turns the functionality off
	    #
	    system ("nice -19 $ID_SCREEN_SAVER -display $display s reset > /dev/null 2>/dev/null");
	    system ("nice -19 $ID_SCREEN_SAVER -display $display s off > /dev/null 2>/dev/null");

	    # define the geometry for the browser
	    #
	    system ("nice -19 $ID_BROWSER -display $display -geometry $ID_BROWSER_GEOMETRY > /dev/null 2>/dev/null &");

	    # create a pause before opening the given url
	    #
	    sleep $ID_TIME_DELAY;
	    system ("nice -19 $ID_BROWSER -display $display -remote \'OpenURL\($url\)\' > /dev/null 2>/dev/null &");
	}
    }
}

# if no option has been selected
#
else {

    # print an error message stating that no option has been selected 
    #
    isip_die ("no options specified. try the -help option.");
}

#
# end of file
