#! @PERL@
#
# file: $isip/scripts/perl/isip_fmconvert/isip_fmconvert.pl
#

# load the command_line source
#
require "$ENV{'ISIP'}/lib/scripts/perl/command_line.pm";

$ISIP_HELP_FILE = <<__ISIP_HELP_HERE_FILE__;

name: isip_fmconvert
synopsis: isip_fmconvert [options] file(s)
descr: convert framemaker files to specified format(s)
example: isip_fmconvert -text resume

options: 
     -text: convert files to text (with .text extension)
     -mif: convert file to MIF (with .mif extension)
     -pdf: convert files to Adobe's pdf (with .pdf extension)
     -ps: convert to postscript (with .ps extension)
     -help: display this help message

arguments: framemaker filenames to convert

man page: none
__ISIP_HELP_HERE_FILE__

$PDF_FILE = "/tmp/to_pdf_$$.fm";
$PS_FILE = "/tmp/to_ps_$$.fm";

$PDF_TEMPLATE = "$ENV{'ISIP'}/util/misc/isip_fmconvert/to_pdf.fm";
$PS_TEMPLATE = "$ENV{'ISIP'}/util/misc/isip_fmconvert/to_ps.fm";

($text, $mif, $pdf, $ps, @files) = command_line(-1, "-text",0,"-mif",
						0,"-pdf",0,"-ps",0);

open(FP, "|fmbatch") or isip_die("can't run command here");

# load pdf and/or ps files, if needed
#
if ($pdf) {

    # copy the pdf template file, open it
    #
    `cp $PDF_TEMPLATE $PDF_FILE`;
    `chmod a+w $PDF_FILE`;
    print FP "Open $PDF_FILE\n";
}

if ($ps) {

    # copy the ps template file, open it
    #
    `cp $PS_TEMPLATE $PS_FILE`;
    `chmod a+w $PS_FILE`;
    print FP "Open $PS_FILE\n";
}

for $file (@files) {
    
    print FP "Open $file\n";
    
    if ($text) {      	

	# transform .fm -> .text
	#	
	$file_out = $file;
        if ($file_out =~ /\.fm$/) {
	    $file_out =~ s/\.fm$/\.text/;
	}
	else {
	    $file_out = "$file_out.text";
	}

	print FP "echo Saving $file to $file_out\n";
	print FP "SaveAs t $file $file_out\n";
    }

    if ($mif) {
	
	# transform .fm -> .mif
	#	
	$file_out = $file;
        if ($file_out =~ /\.fm$/) {
	    $file_out =~ s/\.fm$/\.mif/;
	}
	else {
	    $file_out = "$file_out.mif";
	}

	print FP "echo Saving $file to $file_out\n";
	print FP "SaveAs m $file $file_out\n";
    }
    
    if ($pdf) {

	print FP "echo Printing $file as pdf\n";
	print FP "Print $file $PDF_FILE\n";
    }

    if ($ps) {

	print FP "echo Printing $file as ps\n";
	print FP "Print $file $PS_FILE\n";
    }
    
    print FP "Quit $file\n" ;
    
}


if ($pdf) {
    print FP "Quit $PDF_FILE\n";
}
if ($ps) {
    print FP "Quit $PS_FILE\n";
}

print FP "Quit\n" ;
close(FP);

if ($pdf) {
    `rm $PDF_FILE`;
}
if ($ps) {
    `rm $PS_FILE`;
}
