// file: $isip/util/asr/isip_lm_converter.cc
// version: $Id: isip_lm_converter.cc,v 1.2 2002/12/09 17:21:47 alphonso Exp $
//

// isip include files
//
#include <LanguageModel.h>
#include "isip_lm_converter.h"

// isip_lm_converter: language model converter utility
//
// this is the language model converter utility. it can be used for
// convert input of VJSGF(visualization of JSGF) or JSGF format into
// output of VJSGF(visualization of JSGF) or JSGF format
//
int main(int argc, const char** argv) {
 
 CommandLine cmdl;
 
 cmdl.setUsage(
#include "usage_message.text"
                );
 cmdl.setHelp(
#include "help_message.text"
               );
 
 cmdl.setIdent("$Revision: 1.2 $",
	       "$Name: isip_r00_n11 $",
	       "$Date: 2002/12/09 17:21:47 $");
    

 // add a command line option for input file
 //
 Filename input_lm_file;
  Filename def_lm_input_file(L"word_level_input.sof");
 cmdl.addOptionParam(input_lm_file, OPTION_INPUT_FILE, def_lm_input_file);

 // add a command line option for the input mode
 //
 String input_mode;
 cmdl.addOptionParam(input_mode, OPTION_INPUT_MODE, EMPTY);
 
 // add a command line option for output file
 //
 Filename output_lm_file;
 Filename def_lm_output_file(L"word_level_output.sof");
 cmdl.addOptionParam(output_lm_file, OPTION_OUTPUT_FILE, def_lm_output_file);
 
 // add a command line option for the output mode
 //
 String output_mode;
 cmdl.addOptionParam(output_mode, OPTION_OUTPUT_MODE, EMPTY);

 // parse the command line
 //
 if (!cmdl.parse(argc, argv)) {
   cmdl.printUsage();
 };

 // print the command line
 //
 cmdl.printCommandLine();
 cmdl.printVersion();

 // check the input file argument
 //
 if(input_lm_file.eq(NULL)) {
   cmdl.printUsage();
 };

 if(input_mode.eq(NULL)) {
   cmdl.printUsage();
 };
  
 if(output_lm_file.eq(NULL)) {
   cmdl.printUsage();
 };

 if(output_mode.eq(NULL)) {
   cmdl.printUsage();
 };

 Vector<SearchLevel> lev;
 LanguageModel lm2;
 
 // load file from input_lm_file and store it into vector of SearchLevel
 //
 if(input_mode.eq(L"JSGF") || input_mode.eq(L"VJSGF")) {
   String model_file_a;
   model_file_a.assign(input_lm_file); 
   lm2.load(model_file_a, lev);
 }
 else {
   cmdl.printUsage();
 }

 // output_lm_file
 //
 if(output_mode.eq(L"VJSGF")) {

   // convert JSGF format of input_lm_file to NATIVE format of output_lm_file
   //
   String model_file_a;
   model_file_a.assign(output_lm_file);
   lm2.store(model_file_a,lev, LanguageModel::NATIVE);
 }
 else if(output_mode.eq(L"JSGF")) {

   // convert VJSGF format of input_lm_file to JSGF format of output_lm_file
   //
   String model_file_a;
   model_file_a.assign(output_lm_file);
   lm2.store(model_file_a,lev, LanguageModel::JSGF);
 }
 else {
    cmdl.printUsage();
 }
 
 // exit gracefully
 //
 return Integral::exit();

}










