// file: DiGraph.java
//

// import java libraries
//
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.util.*;
import java.io.*;
import java.lang.*;
import java.net.*;

public class DiGraph extends JFrame implements ActionListener, MouseListener,
					       MouseMotionListener, Constants {

    // ---------------------------------------------------
    //
    // declare class member data
    //
    // ---------------------------------------------------

    // keep track the curren vertex in focus
    //
    public Vertex systemVertex = null;
    public Vertex inVertex = null;
    public Vertex outVertex = null;
    public Vertex initVertexFocus = null;
    public Vertex cfgVertexFocus = null;
    public Vertex currVertexFocus = null;

    // differentiate between parent and child 
    //
    public Vertex vertexParent = null;
    public Vertex vertexChild = null;

    // various types of been in the work area
    //
    public Vector vertices = null;
    public Vector coeffLabels = null;
    public Vector dataLabels = null;

    // temporary vertex arc parameters
    //
    public String tmpepi = null;
    public String tmpweight = null;

    // flag that enables insertion and removal of arcs
    //
    public boolean enableArcInsertion = false;
    public boolean enableSelfArcInsertion = false;
    public boolean enableArcDeletion = false;

    // flag that determines if input block exists
    //
    public boolean inputBlockExist = false; 

    // flag that determines if output block exists
    //
    public boolean outputBlockExist = false; 

    // target of the front end for output data type
    //
    public String targetStr = null;

    // input data type
    //
    public String inputDataStr = SAMPLED_DATA_STR;

    // current save status flag
    //
    public boolean saveWork = false;

    // vector of selected vertices
    //
    public Vector selectedVertices = null;

    // clipboard to handle things like cut, copy and paste
    //
    public Vector clipboard = null;

    // hash tables for quick reference
    //
    public Hashtable tagToVertex = null;

    // holds the maximum width and height of the scroller
    //
    public int maxWidth = WORKAREA_WIDTH;
    public int maxHeight = WORKAREA_HEIGHT;

    // holds the current location of the mouse
    //
    public int mouseXloc = 0;
    public int mouseYloc = 0;
    public int line_index = 0;

    // border for the viewport of the scrollpane
    //
    public Border border = null;

    // the label of the vertex that has been selected form the tool bar
    //
    public String currVertexLabel= null;

    // record the maximum number of inputs generated
    //
    public int maxInputVertices = 0;

    // algorithm association for the processes
    //
    public Association currAssociation = null;

    // maximum width and height of the work area
    //
    public int loadWidthMax = Integer.MIN_VALUE;
    public int loadHeightMax = Integer.MIN_VALUE;

    // input/output block type and impl
    //
    public int iotype = -1;
    public int ioimpl = -1;

    // varaible name in input block 
    //
    public String inputVariable = null;

    // varaible name in output block 
    //
    public String outputVariable = null;

    // creates a fram to manipulate the configuration options
    //
    public JFrame config = null;

    // textfields and labels for the connector properties
    //
    public JLabel toLabel = null;
    public JLabel fromLabel = null;
    public JLabel nameLabel = null;

    public JTextField toField = null;
    public JTextField fromField = null;
    public JTextField nameField = null;

    // main menu for functionality
    //
    public MainMenu mainMenu = null;

    // work area to drag and drop components
    //
    public WorkArea workArea = null;

    // paramenet file to be written to
    //
    public File paramFile = null;

    // graph file to be written to
    //
    public File graphFile = null;

    // JSGF file to be written to
    //
    public File JSGFFile = null;

    // lmTester file to be tested
    //
    public File lmtesterFile = null;

    // scroll bars for the work area
    //
    public JScrollPane scroller = null;

    // declare the popup menu objects
    //
    public JPopupMenu popup = null;
    public JMenuItem menuItem = null;

    // default strings for the popup menus
    //
    public String ordering = null;
    public String delete = null;
    public String expand = null;
    public String configuration = null;
    public String label = null;
    public String apply = null;
    public String update = null;
    public String cancel = null;
    public String saveApprove = null;
    public String loadApprove = null;

    // data object for reading the algorithm resource file
    //
    public Data data = null;
    
    // arc objects for stroring vertex (from), vertex (to), weights, and 
    // epsilon 
    public GraphArc graphArc = null;

    // wordSymbol
    //
    public Vector wordSymbol = null;
    
    // wordVertex
    //
    public Vector wordVertex = null;

    // nodeInfo in the digraph
    //
    public Vector graphInfo = null;


    // ---------------------------------------------------
    //
    // declare class constructors
    //
    // ---------------------------------------------------

    // method: DiGraph
    //
    // arguments: none
    // returns  : none
    //
    public DiGraph(String title) {
	
	// invoke the parent default constructor
	//
	super(title);

	
	//instantiate the data class
	//
	data = new Data();
	
	// initialize the arc class
	//
	graphArc = new GraphArc();

	// initialize the vectors data members
	//
	vertices = new Vector(100, 50);
	coeffLabels = new Vector(100, 50);
	dataLabels = new Vector(100, 50);
	clipboard = new Vector(10, 10);
	selectedVertices = new Vector(100, 50);

	// initialize the hashtables 
	//
	tagToVertex = new Hashtable(50);

	// instantiate the process algorithm association
	//
	currAssociation = new Association();

	// initialize the popup menu strings
	//
	ordering = new String("Ordering");
	delete = new String("Delete");
	configuration = new String("Configuration");

	// create the popup menu for the process vertex
	//
	popup = new JPopupMenu();
	menuItem = new JMenuItem(configuration);
	menuItem.addActionListener(this);
	popup.add(menuItem);
	popup.addSeparator();

	menuItem = new JMenuItem(ordering);
	menuItem.addActionListener(this);
	popup.add(menuItem);

	menuItem = new JMenuItem(delete);
	menuItem.addActionListener(this);
	popup.add(menuItem);

	// set the font of the popup menu
	//
	popup.setFont(newCoeffFont);

	// create the menu to control functionality
	//
	mainMenu = new MainMenu(this);

	// create a work area to drag and drop components
	//
	workArea = new WorkArea(vertices, this);
	workArea.addMouseListener(this);
	workArea.addMouseMotionListener(this);

	// add scroll panes to the workarea
	//
	scroller = new JScrollPane(workArea, 
				   JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				   JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
	
	// set the dimensions of the scroller
	//
	scroller.setPreferredSize(new Dimension(MAX_WIDTH, MAX_HEIGHT));

	// add the components
	//
	this.getContentPane().add(mainMenu, BorderLayout.NORTH);
	this.getContentPane().add(scroller, BorderLayout.CENTER);
	mainMenu.add_components();

	// create and initialize input configuration vertex
	//
	inVertex = new Vertex("system", ALGORITHM, NONE);
	inVertex.association = "Audio_Input";
	
	// to simplify the creation, the data vectors will be trimmed to
	// only include the transform we're looking at and saved into
	// these three vectors for use in a bit
	//
	Vector labels2 = new Vector(5,5);
	Vector types2 = new Vector(5,5);
	Vector values2 = new Vector(5,5);
	
	for(int j=data.association.indexOf(inVertex.association); 
	    j <= data.association.lastIndexOf(inVertex.association);
	    j++) {
	    if(j != -1) {
		labels2.add(data.name.get(j));
		types2.add(data.type.get(j));
		values2.add(data.values.get(j));
	    }
	}	
	
	// initialize the vertex to the default values for its type
	//
	inVertex.initializeConfig(labels2, values2, types2,
				    inVertex.association, data);

	// create and initialize output configuration vertex
	//
	outVertex = new Vertex("system", ALGORITHM, NONE);
	outVertex.association = "Audio_Output";
	
	// to simplify the creation, the data vectors will be trimmed to
	// only include the transform we're looking at and saved into
	// these three vectors for use in a bit
	//
	Vector labels3 = new Vector(5,5);
	Vector types3 = new Vector(5,5);
	Vector values3 = new Vector(5,5);
	
	for(int j=data.association.indexOf(outVertex.association); 
	    j <= data.association.lastIndexOf(outVertex.association);
	    j++) {
	    if(j != -1) {
		labels3.add(data.name.get(j));
		types3.add(data.type.get(j));
		values3.add(data.values.get(j));
	    }
	}	
	
	// initialize the vertex to the default values for its type
	//
	outVertex.initializeConfig(labels3, values3, types3,
				    outVertex.association, data);

    }

    // method: resetVertexFocus
    //
    // arguments: none
    // return: none
    //
    // resets the parent and child relationships
    //
    public void resetVertexFocus() {
	
	// reset the parent and child focus
	//
	vertexChild = null;
	vertexParent = null;

	// reset the current vertex selected
	//
	initVertexFocus = null;
	currVertexFocus = null;

	// reset the arc enables
	//
	enableArcInsertion = false;
	enableSelfArcInsertion = false;
	enableArcDeletion = false;

	// repaint the workarea
	//
	workArea.repaint();
    }

    // method: updateVertexFocus
    //
    // arguments:
    //    MouseEvent e: (input) input mouse event
    // return: none
    //
    // updates the parent and child relationships
    //
    public void updateVertexFocus(MouseEvent e) {
	
	// check to make sure that the vertex parent is valid
	//
	if (vertexParent == null) {
	    
	    // highligh the current vertex parent
	    //
	    vertexParent = (Vertex)e.getSource();
	}

	// initialize the child if the vertex parent is valid
	//
	else {

	    // initialize the child of the connection
	    //
	    vertexChild = (Vertex)e.getSource();
	}
    }

    // method: createAlgorithm
    //
    // arguments:
    //   String algorithmstr: name of the algorithm that fired the event
    // returns: none
    //
    // this method creates an algorithm object
    //
    public void createAlgorithm(String algorithmstr) {

	// determine which algorithm fired the event
	//
	int action = data.association.indexOf(algorithmstr);
	
	// if the action is found in the list of transforms
	//
	if (action != -1) {
	    
	    String image_name = null;
	    String image_association= null;
	    String icon_name = null;
	    
	    // tells the program which transform it is looking to match
	    //
	    image_association = algorithmstr;
	    
	    // scans through the association until 
	    // it finds the largeIcon key
	    //
	    for(int j=data.association.indexOf(image_association); 
		j <= data.association.lastIndexOf(image_association); 
		j++) {
		
		if (j != -1) {
		    if (((String)(data.name.get(j))).equals(ALGORITHM_ICON_BIG)) {
			
			// sets the image_name to the value held 
			// in large icon
			//
			image_name = (String)data.values.elementAt(j);
		    }
		    if (((String)(data.name.get(j))).equals("iconName")) {
			
			// sets the image_name to the value held 
			// in large icon
			//
			icon_name = (String)data.values.elementAt(j);
		    }  
		}
	    }
	    
	    // check to see if the file exists. If it does not exist,
	    // use a default image.
	    //
	    if(image_name != null) {
		
		File image_file = new File(image_name);
		if (image_file.exists() == false)
		    image_name = CONTAINER_IMAGE;
	    }
	    else {
		image_name = CONTAINER_IMAGE;
	    }
	    
	    // create the vertex for the work area
	    //
	    ImageIcon vertex_icon = new ImageIcon(image_name);
	    Vertex  vertex = new Vertex(vertex_icon , ALGORITHM);
	    vertex.setVertexName(algorithmstr);
	    if (algorithmstr.length() > 4) {
		vertex.setText(algorithmstr.substring(0, 4));
	    }
	    else {
		vertex.setText(algorithmstr);
	    }
	
            if(icon_name != null) {
		vertex.setText(icon_name);
		vertex.setIconName(icon_name);
	    }

	    // to simplify the creation, the data vectors will be
	    // trimmed to only include the transform we're looking
	    // at and saved into these three vectors for use in a
	    // bit
	    //
	    Vector labels = new Vector(5,5);
	    Vector types = new Vector(5,5);
	    Vector values = new Vector(5,5);
	    
	    for(int j=data.association.indexOf(image_association); 
		j <= data.association.lastIndexOf(image_association); 
		j++) {
		
		if(j != -1) {
		    labels.add(data.name.get(j));
		    types.add(data.type.get(j));
		    values.add(data.values.get(j));
		}
	    }	 
	    
	    // initialize the vertex to the default values for its type
	    //
	    vertex.initializeConfig(labels, values, types, 
				    image_association, data);
	    
	    // set the tool tip for the algorithm vertex
	    //
	    vertex.setToolTipText(vertex.association);
	    
	    // attach a mouse motion listener to the vertex
	    //
	    vertex.addMouseListener(this);
	    vertex.addMouseMotionListener(this);
	    
	    // set the current vertex selected
	    //
	    initVertexFocus = vertex;
	
	    // Create an instance of java.awt.Cursor
	    Cursor c = new Cursor ( Cursor.HAND_CURSOR );

	    // Set cursor for the frame component
	    this.setCursor (c);
	}
    }


    // method: createIOData
    //
    // arguments:
    //   String label: data label
    //
    // this method creates a name data node (SAMPLE_DATA or FEATURES)
    //
    public void createIOData(int type, int impl) {
	iotype = type;
	ioimpl = impl;
	
	// Create an instance of java.awt.Cursor
	Cursor c = new Cursor ( Cursor.HAND_CURSOR );

	// Set cursor for the frame component
	this.setCursor (c);
    }

    // method: createData
    //
    // arguments:
    //   String label: data label
    //
    // this method creates a name data node (SAMPLE_DATA or FEATURES)
    //
    public void createData() {

	// declare local variables
	//
	Vertex vertex = null;

	// currently we only allow one sample input/output and one
	// feature input/output but not one of both
	//

	if ((dataLabels.contains("Start")) &&  
	     (iotype == START) && (ioimpl == START)) {
	    JFrame frame = new JFrame("Warning");
	    JOptionPane.showMessageDialog(frame, "An start** block is already present.");		

	    // Create an instance of java.awt.Cursor
	    //
	    Cursor c = new Cursor ( Cursor.DEFAULT_CURSOR );
	    this.setCursor (c);

	    return;
	}
	
	if ((dataLabels.contains("Stop")) && 
	    (iotype == STOP) && (ioimpl == STOP)) {
	    JFrame frame = new JFrame("Warning");
	    JOptionPane.showMessageDialog(frame, "An stop block is already present.");
	   
	    // Create an instance of java.awt.Cursor
	    //
	    Cursor c = new Cursor ( Cursor.DEFAULT_CURSOR );
	    this.setCursor (c);
	    
	    return;	    
	}
	
	if ((dataLabels.contains("Start")) && 
	    (iotype == START) && (ioimpl == START)) {
	    JFrame frame = new JFrame("Warning");
	    JOptionPane.showMessageDialog(frame, "An start block is already present.");
	    // Create an instance of java.awt.Cursor
	    //
	    Cursor c = new Cursor ( Cursor.DEFAULT_CURSOR );
	    this.setCursor (c);
	    
	    return;	    
	}
	if ((dataLabels.contains("Stop")) && 
	    (iotype == STOP) && (ioimpl == STOP)) {
	    JFrame frame = new JFrame("Warning");
	    JOptionPane.showMessageDialog(frame, "An stop block is already present.");
	    // Create an instance of java.awt.Cursor
	    //
	    Cursor c = new Cursor ( Cursor.DEFAULT_CURSOR );
	    this.setCursor (c);
	    
	    return;   
	}
	
	// when the type is SAMPLED_DATA
	//
	if (iotype == START) {

	    // when the implementation is START
	    //
	    if (ioimpl == START) {

		// create a unique label for the input vertex
		//
		currVertexLabel = new String("Start");
		
		// create a vertex to be added in the work area
		//
		ImageIcon vertex_icon = new ImageIcon(START_IMAGE);
		vertex = new Vertex(vertex_icon, currVertexLabel, 
				    START, START);
		dataLabels.addElement("Start");

		// set the tool tip for the vertex
		//
		vertex.setToolTipText("Start");

		// set flag for input block
		//
		inputBlockExist = true;
	    }

	    // when the implementation is stop
	    //
	    if (ioimpl == STOP) {

		// create a unique label for the input vertex
		//
		currVertexLabel = new String("Stop");
		
		// create a vertex to be added in the work area
		//
		ImageIcon vertex_icon = new ImageIcon(TERM_IMAGE);
		vertex = new Vertex(vertex_icon, currVertexLabel, 
				    STOP, STOP);
		dataLabels.addElement("Stop");

		// set the tool tip for the vertex
		//
		vertex.setToolTipText("Stop");

		// set flag for input block
		//
		outputBlockExist = true;
		
		// add the output block name as the system target
		//
		targetStr = currVertexLabel;
       	    }
	}

	// when the type is FEATURE_DATA
	//
	else if (iotype == STOP) {

	    // when the implementation is START
	    //
	    if (ioimpl == START) {

		// create a unique label for the input vertex
		//
		currVertexLabel = new String("Start");
		
		// create a vertex to be added in the work area
		//
		ImageIcon vertex_icon = new ImageIcon(START_IMAGE);
		vertex = new Vertex(vertex_icon, currVertexLabel, 
				    START, START);
		dataLabels.addElement("Start");

		// set the tool tip for the vertex
		//
		vertex.setToolTipText("Start");
		
		// set flag for input block
		//
		inputBlockExist = true;
	    }
	    
	    // when the implementation is STOP
	    //
	    if (ioimpl == STOP) {

		// create a unique label for the input vertex
		//
		currVertexLabel = new String("Stop");
		
		// create a vertex to be added in the work area
		//
		ImageIcon vertex_icon = new ImageIcon(TERM_IMAGE);
		vertex = new Vertex(vertex_icon, currVertexLabel,
				    STOP, STOP);
		dataLabels.addElement("Stop");

		// set the tool tip for the vertex
		//
		vertex.setToolTipText("Stop");

		// set flag for input block
		//
		outputBlockExist = true;

		// add the output block name as the system target
		//
		targetStr =  currVertexLabel;	      
	    }
	}
	
	// when the type is CONTAINER
	//
	else if (iotype == CONTAINER) {
	    
	    // generate a new feature label if the original is not unique
	    //
	    int tmpInputVertices = 0;
	    currVertexLabel = new String("S" + tmpInputVertices);

	    while (coeffLabels.contains(currVertexLabel)) {
		tmpInputVertices++;
		currVertexLabel = new String("S" + tmpInputVertices);
	    }

	    if(tmpInputVertices > maxInputVertices)
		maxInputVertices = tmpInputVertices;
	    /*
	    // create a unique label for the input vertex
	    //
	    currVertexLabel = new String("S" + maxInputVertices);
	    
	    // generate a new feature label if the original is not unique
	    //
	    while (coeffLabels.contains(currVertexLabel)) {
		maxInputVertices++;
		currVertexLabel = new String("S" + maxInputVertices);
	    }	    
	    */

	    // create a vertex to be added in the work area
	    //
	    ImageIcon vertex_icon = new ImageIcon(CONTAINER_IMAGE);
	    vertex = new Vertex(vertex_icon, currVertexLabel, 
				CONTAINER, NONE);
	    coeffLabels.addElement(currVertexLabel);

	    // set the tool tip for the vertex
	    //
	    vertex.setToolTipText("Node");

	    // initialize Node parameters
	    //
	    vertex.names.add("Word_List");
	    vertex.types.add("enum");

	    // add the token that has already been taken
	    //

	    // vertex.values.add("word1");
	}

	// we have an undefined type
	//
	else {
	    return;
	}
	
	// attach a mouse motion listener to the vertex
	//
	vertex.addMouseListener(this);
	vertex.addMouseMotionListener(this);       
	
	// add the vertex to the vector
	//
	vertex.setFont(newCoeffFont);

	// set the current vertex selected
	//
	initVertexFocus = vertex;
	
    }

    // method: initializeDataBuffer
    //
    // arguments:
    // returns  : new data buffer vertex
    //
    // method initializes the container object
    //
    public Vertex initializeDataBuffer() {
	return (new Vertex());
    }

    // method: initializeContainer
    //
    // arguments:
    //      String name: (input) name of the container
    // returns  : new container vertex
    //
    // method initializes the container object
    //
    public Vertex initializeContainer(String name) {
	ImageIcon vertex_icon = new ImageIcon(CONTAINER_IMAGE);
	return (new Vertex(vertex_icon, name, CONTAINER, NONE));
    }

    // method: initializeAlgorithm
    //
    // arguments:
    //   String name: (input) name of the algorithm object
    //   BufferedReader file: (input) input file buffered reader
    // returns  : new algorithm vertex
    //
    // method initializes the algorithm
    //
    public Vertex initializeAlgorithm(String name, BufferedReader file, String typeAlgo) {

	// initialize variables
	//
	StringTokenizer tokens = null;
	String line = null;
	String type = typeAlgo;
	String image_name = null;
	String image_association= null;	
	
	// when the name indicates a start or term vertex
	//
	if (name.equals(BUFFER_TAG)) {
	
	    // switch based on the type of the object
	    //
	    if (type.equals("Start") || type.equals(INPUTGEN_STR)) {

		// declare local variables
		//
		Vertex START_VERTEX = new Vertex(new ImageIcon(START_IMAGE), ALGORITHM);
		
		// set the location of the vertices
		//
		START_VERTEX.setLocation(new Point(INIT_START_XLOC, 
						   INIT_START_YLOC));
		
		// set the dimensions of the vertices
		//
		START_VERTEX.setSize(ICON_WIDTH, ICON_HEIGHT);
		START_VERTEX.setPreferredSize(new Dimension(ICON_WIDTH, 
							    ICON_HEIGHT));
		START_VERTEX.setMinimumSize(new Dimension(ICON_WIDTH, 
							  ICON_HEIGHT));
		START_VERTEX.setVertexWidth(ICON_WIDTH);
		START_VERTEX.setVertexHeight(ICON_HEIGHT);
		
		// set the font of the vertices
		//
		START_VERTEX.setFont(newCoeffFont);
		
		// set the type of the vertex
		//
		START_VERTEX.setType(START);
		
		// set the vertex papameters
		//
		START_VERTEX.association = new String(BUFFER_TAG);
		START_VERTEX.names = new Vector();
		START_VERTEX.types = new Vector();
		
		// return the vertex
		//
		return START_VERTEX;
	    }
	     
	    if (type.equals("Stop")) {
		
		// declare local variables
		//
		Vertex TERM_VERTEX = new Vertex(new ImageIcon(TERM_IMAGE), ALGORITHM);
		
		// set the location of the vertices
		//
		TERM_VERTEX.setLocation(new Point(INIT_TERM_XLOC, 
						  INIT_TERM_YLOC));
		
		// set the dimensions of the vertices
		//
		TERM_VERTEX.setSize(ICON_WIDTH, ICON_HEIGHT);
		TERM_VERTEX.setPreferredSize(new Dimension(ICON_WIDTH, 
							   ICON_HEIGHT));
		TERM_VERTEX.setMinimumSize(new Dimension(ICON_WIDTH, 
							 ICON_HEIGHT));
		TERM_VERTEX.setVertexWidth(ICON_WIDTH);
		TERM_VERTEX.setVertexHeight(ICON_HEIGHT);
		
		// set the font of the vertices
		//
		TERM_VERTEX.setFont(newCoeffFont);
		
		// set the type of the vertex
		//
		TERM_VERTEX.setType(STOP);
		
		// set the vertex papameters
		//
		TERM_VERTEX.association = new String(BUFFER_TAG);
		TERM_VERTEX.names = new Vector();
		TERM_VERTEX.types = new Vector();	

		// return the vertex
		//
		return TERM_VERTEX;
	    }	
	}

	// which transform it is looking to match?
	//
	name = name.replace('\"',' ');
	name = name.replace(';', ' ');
	name = name.trim();
	image_association = name;
	
	
	// create a vertex to be added in the work area
	//
	ImageIcon vertex_icon = new ImageIcon(CONTAINER_IMAGE);
	Vertex algoVertex = new Vertex(vertex_icon, name, 
				CONTAINER, NONE);

	// set the tool tip for the vertex
	//
	algoVertex.setToolTipText("Node");

	algoVertex.setVertexName(name);
	if (name.length() > 4) {
	    algoVertex.setText(name.substring(0, 4));
	}
	else {
	    algoVertex.setText(name);
	}

	coeffLabels.addElement(name);
	
	// initialize Node parameters
	//
	
	algoVertex.names.add("Word_List");
	algoVertex.types.add("enum");
	
	Vector enum_var = new Vector(5,5);
	//enum_var.add(typeAlgo);
		
	tokens = new StringTokenizer(typeAlgo);
	String ww = ""; 
	while(tokens.hasMoreTokens()) {
	    String xvar = new String(tokens.nextToken());
	    xvar.trim();
	    ww =ww+xvar + " ";
	    enum_var.addElement(xvar);
	}
		
	// add the token that has already been taken
	//
	algoVertex.values.addElement(typeAlgo);

	// return the initialize vertex
	//
	return algoVertex;
    }

    // method: removeVertex
    //
    // arguments:
    //   Vertex target: (input) vertex to be removed
    // returns  : none
    //
    // method that removes the specified vertex from the work area as
    // well as removing all parent and corresponding children links
    //
    public void removeVertex(Vertex target) {

	// verify that the vertex to be deleted is valid
	//
	if (target == null) {
	    return;
	}

	// reset the vertex focus
	//
	resetVertexFocus();

	// get the parents of the traget and all accurances of target
	//
	Vector parents = target.getParents();
	for (int i=0; i<parents.size(); i++) {
	    if (((Vertex)(parents.elementAt(i))).containsChild(target)) {
		((Vertex)(parents.elementAt(i))).removeChild(target);
	    }
	}

	// get the children of the traget and all accurances of target
	//
	Vector children = target.getChildren();
	for (int i=0; i<children.size(); i++) {
	    if (((Vertex)(children).elementAt(i)).containsParent(target)) {
		((Vertex)(children.elementAt(i))).removeParent(target);
	    }
	}
	
	// remove the node form the work area
	//
	workArea.remove(target);
	vertices.remove(target);

	// remove the label from the vector depending on the implementation
	//
	if (target.getVertexImpl() == START) {
	    dataLabels.remove(target.getText()); 
	    dataLabels.remove(START_STR);
	    iotype = -1;
	    inputBlockExist = false;	 
	}
	else if (target.getVertexImpl() == STOP) {
	    dataLabels.remove(target.getText());
	    iotype = -1;
	    outputBlockExist = false;	
	    dataLabels.remove(STOP_STR);
	}
	else {
	    coeffLabels.remove(target.getText());
	    maxInputVertices--;
	}

	String targetStr = target.getVertexName();
	for(int k = 0; k < graphArc.from.size(); k++) {
	    Vertex from = (Vertex)graphArc.from.get(k);
	    String fromStr = from.getVertexName();
	    Vertex to  = (Vertex)graphArc.to.get(k);
	    String toStr = to.getVertexName();	    
	    if (targetStr.equals(toStr) || targetStr.equals(fromStr)) {
		graphArc.from.remove(k);
                graphArc.to.remove(k);
		graphArc.weights.remove(k);
		graphArc.epsilon.remove(k);
		graphArc.xloc.remove(k);
		graphArc.yloc.remove(k);
		graphArc.width.remove(k);
		graphArc.height.remove(k);
		k = k-1;
	    }	
	}
		  
	// repaint the work area
	//
	workArea.repaint();
    }
  
    // method: cut
    //
    // arguments: none
    // returns  : none
    //
    // method cuts the current vertex and adds it to the clipboard
    //
    public void cut() {

	// check for a valid vertex
	//
	if (currVertexFocus == null) {
	    return;
	}

	// clear the current contents of the clipboard
	//
	clipboard.removeAllElements();

	// add the vertex to the clipboard
	//
	clipboard.addElement(currVertexFocus);

	// remove the vertex 
	//
	removeVertex(currVertexFocus);

	// repaint the work area
	//
	workArea.repaint();
    }

    // method: copy
    //
    // arguments: none
    // returns  : none
    //
    // method copies the current vertex and adds it to the clipboard
    //
    public void copy() {

	// check for a valid vertex
	//
	if (currVertexFocus == null) {
	    return;
	}

	// we cannot copy and paste SAMPLED_DATA and FEATURE
	// vertices, hence, we check for them here
	//
	if ((currVertexFocus.getVertexType() == START) || 
	    (currVertexFocus.getVertexType() == STOP)) {
	    JFrame frame = new JFrame("Warning");
	    JOptionPane.showMessageDialog(frame, 
					  "You cannot paste Input or output block");
	    return;
	}

	// clear the current contents of the clipboard
	//
	clipboard.removeAllElements();

	// add the vertex to the clipboard
	//
	clipboard.addElement(currVertexFocus);

	// repaint the work area
	//
	workArea.repaint();
    }

    // method: paste
    //
    // arguments: none
    // returns  : none
    //
    // method pastes the current vertex from the clipboard
    //
    public void paste() {

	// loop through each element in the clipboard and add them back
	//
	for(int i=0; i < clipboard.size(); i++) {

	    // get the vertex that we intend to paste
	    //
	    Vertex v = (Vertex)clipboard.elementAt(i);
	    
	    // create a copy of the vertex and remove all arcs
	    //
	    Vertex vertex = new Vertex(v);

	    // remove all parents and children associated with the vertex
	    //
	    vertex.vertexParents.removeAllElements();
	    vertex.vertexChildren.removeAllElements();

	    // attach a mouse motion listener to the vertex
	    //
	    vertex.addMouseListener(this);
	    vertex.addMouseMotionListener(this);

	    // set the font for the vertex
	    //
	    vertex.setFont(newCoeffFont);

	    // set the location of the vertex
	    //
	    Point coordinates = new Point(mouseXloc, mouseYloc);
	    vertex.setLocation(coordinates);
	    
	    // we need create a unique name for the current vertex
	    // if it is of type CUSTOM or CONTAINER
	    //
	    if ((vertex.getVertexType() == CUSTOM) ||
		(vertex.getVertexType() == CONTAINER)) {

		// when the old name does not exist reuse it
		//
		if (!coeffLabels.contains(vertex.getText())) {
		    currVertexLabel = vertex.getText();
		}

		// when the old name exist create a new copy
		//
		else {
		    int offset = -1;
		    currVertexLabel = vertex.getText();
		    while (coeffLabels.contains(currVertexLabel)) {
			offset++;
			currVertexLabel = vertex.getText() + offset;
		    }
		}

		maxInputVertices++;
		currVertexLabel = "S"+maxInputVertices;

		// set the name of the vertex
		//
		if (currVertexLabel.length() > 3) {
		    vertex.setText(currVertexLabel.substring(0, 3));
		}
		else {
		    vertex.setText(currVertexLabel);
		}
		vertex.setVertexName(currVertexLabel);
		coeffLabels.addElement(currVertexLabel);
	    }

	    // add the element to the data structures
	    //
	    vertices.addElement(vertex);
	    workArea.add(vertex);
	}
	
	// repaint the work area
	//
	workArea.repaint();
    }
 
    // method: clearAllMemory
    //
    // arguments: none
    // returns  : none
    //
    // method clears out all memory, i.e., vectors and hash tables
    //
    public void clearAllMemory() {

	// reset the maximum number of vertices
	//
	maxInputVertices = 0;
	
	// reset the vertices
	//
	initVertexFocus = null;
	cfgVertexFocus = null;
	currVertexFocus = null;
	vertexParent = null;
	vertexChild = null;	

	inputBlockExist = false;
	outputBlockExist = false;

	// clear out all vectors
	//
	vertices.removeAllElements();
	coeffLabels.removeAllElements();
	dataLabels.removeAllElements();
	clipboard.removeAllElements();
	selectedVertices.removeAllElements();
	
	// reset the current line index
	//
	line_index = 0;

	// clear out all hash tables
	//
	tagToVertex.clear();

	// clear out the work area
	//
	workArea.removeAll();

	// clear out graphArc object
	//
	graphArc = new GraphArc();
    }

    // method: writeGraph
    //
    // arguments: none
    // return   : none
    //
    // writes the sof file
    //
    public void writeGraph() {
	getVertexSymbol();

	// convert the Digraph format to JSGF format
	//
	convertJSGF();
    }
    
    // method: getVertexSymbol
    //
    // arguments: none
    // return   : none
    //
    // get Digraph inforamtion
    //
    public void getVertexSymbol() {
	
	// define tempory vertex collection
	//
	Vector vertexCollection = null; 
	wordVertex = new Vector(100, 50);
	wordSymbol = new Vector(100, 50);

	for (int i=0; i < vertices.size(); i++) {
	    
	    //get the start vertex
	    //
	    Vertex vertex = (Vertex)vertices.elementAt(i);
	    
	    if(vertex.getVertexType() != START && vertex.getVertexType() != STOP) {
		String datavar = "" + vertex.values.get(0);
		StringTokenizer tokens = new StringTokenizer(datavar, " ");
		
		String xvar = (tokens.nextToken()).trim();
		if(wordSymbol == null) {
		    wordSymbol.add(xvar);
		}
		else {
		    if(!wordSymbol.contains(xvar)) {    
			wordSymbol.add(xvar);
		    }
		}
		SymbolTable s1 = new SymbolTable();
		s1.vName = xvar;
		s1.nodeIndex = vertex.vertexName;
		s1.symbolIndex = wordSymbol.indexOf(xvar);
		wordVertex.add(s1);
		
		while(tokens.hasMoreTokens()) {

		    xvar = (tokens.nextToken()).trim();
		    if(!wordSymbol.contains(xvar)) {    
		    wordSymbol.add(xvar);
		    }

		    SymbolTable s2 = new SymbolTable();
		    s2.vName = xvar;
		    s2.nodeIndex = vertex.vertexName;
		    s2.symbolIndex = wordSymbol.indexOf(xvar);
		    wordVertex.add(s2);
		}				
	    }
	}

	// declare local variables
	//
	FileOutputStream fout = null;
	PrintWriter file = null;
	String line = null;

	// reset the save flag to indicate the obvious
	// 
	saveWork = false;

	// open the selected file for writing
	//
	try {

	    // initialize the variables
	    //
	    line = new String();
	    
	    // initialize the file output stream
	    //
	    fout = new FileOutputStream(graphFile);
	    file = new PrintWriter(fout);

	    // write the file header
	    //
	    file.println(HEADER_TAG_00);
	    file.println(HEADER_TAG_05);
		
	    // write search symbol
	    //
	    writeSearchSymbol(file);
	    writeSymbolVertices(file);
	    writeSymbolArcs(file);
	    file.println("};");
	    // close the file descriptor
	    //
	    file.close();
	}

	// catch and deal with any file I/O exceptions
	//
	catch(IOException e1) {}

    }

    // method: writeSearchSymbol
    //
    // arguments:
    //    BufferedReader file: (input) file descriptor
    // return   : none
    //
    // writes the sof file
    //
    public void writeSearchSymbol(PrintWriter file) {
	
	String line = null;
	file.println("values = {");
	for(int i=0; i < wordSymbol.size(); i++) {
	    String tmp = (String)wordSymbol.elementAt(i);
	    tmp = tmp.toUpperCase();
	    line = "  \""+tmp+"\"";
	    file.println(line);
	    if(i < wordSymbol.size() - 1)
		line = "}, {";
	    else
		line = "};"; 
	    file.println(line);
	}
    }

    // method: writeSymbolVertices
    //
    // arguments:
    //    BufferedReader file: (input) file descriptor
    // return   : none
    //
    // writes the sof file
    //
    public void writeSymbolVertices(PrintWriter file) {
	
	String line = null;
	
	file.println(HEADER_TAG_06);
	file.println("values = {");
	file.println("  weighted = true;");
	file.println("  vertices =");	
	for(int i=0; i < wordVertex.size(); i++) {
	    SymbolTable s1 = (SymbolTable)wordVertex.elementAt(i);
	    if(i < wordVertex.size() - 1)
		line = "        {" + i  + ", {" + s1.symbolIndex + "}},";
	    else
		line = "        {" + i  + ", {" + s1.symbolIndex + "}};";
	    file.println(line);
	}
    }

    // method: writeSymbolArcs
    //
    // arguments:
    //    BufferedReader file: (input) file descriptor
    // return   : none
    //
    // writes the sof file
    //
    public void writeSymbolArcs(PrintWriter file) {
	
	// declare local variables
	//
	String src = null;
	String dst = null;
	String line = null;
	String wei = null;
	String epi = null;

	file.println("  arcs =");
	boolean start = false;

	for (int i=0; i < vertices.size(); i++) {
	    
	    //get the start vertex
	    //
	    Vertex vertex = (Vertex)vertices.elementAt(i);
	    
	    if(vertex.getVertexType() == START) {
		
		int index = i;
					
		// check if the vertex has any children
		//
		
		if (vertex.getOutDegree() > 0) {
		
		    // get the vector of children
		    //
		    Vector children = (Vector)vertex.getChildren();
		    
		    // loop over all children and write the arcs
		    //
		    for (int j=0; j < children.size(); j++) {
		
			// get the current child
			//
			Vertex child = (Vertex)children.elementAt(j);
			
			for(int k = 0; k < graphArc.from.size(); k++) {
			    if((Vertex)graphArc.from.get(k) == vertex && (Vertex)graphArc.to.get(k) == child) {
			    wei = (String)graphArc.weights.get(k);
			    double tmpw;
			    if(wei==null)
				wei=""+"1.0";
			    tmpw = Double.valueOf(wei).doubleValue();
			    if(wei.equals("0.0") || tmpw == 0.0)
				tmpw = -23.02585093;

			    else
				tmpw = Math.log(tmpw);
			    wei = Double.toString(tmpw);
			    epi = (String)graphArc.epsilon.get(k);
			    break;
			    }
			}
			
			src = new String("S");
			for(int w=0; w < wordVertex.size(); w++) {

			    SymbolTable s1 = (SymbolTable)wordVertex.elementAt(w);

			    if(s1.nodeIndex.equals(child.vertexName)) {
				if (start) {
				    file.println(",");
				}

				dst = new String("" + w);
			    
				if(epi == "true") {
				    line = "        {" + src + ", " + dst + ", " + "epi" + 
					", " + "true" +"}";
				}
				else {
				    line = "        {" + src + ", " + dst + ", " 
					+ wei +"}";
				}		    
				file.print(line);
				start = true;
			    }
			} // end for loop w
		    }// end for loop j
		}
		break;
	    }

	} // end for loop i

	writeSubSymbolArcs(file, vertices);
	file.println(";");	
    }

    public void writeSubSymbolArcs(PrintWriter file, Vector vec) {

	// declare local variables
	//
	String src = null;
	String dst = null;
	String wei = null;
	String epi = null;
	String line = null;
	boolean start = false;
	boolean selfloop = false;

	for (int i=0; i < vec.size(); i++) {
	
	    // get the current vertex
	    //
	    
	    Vertex vertex = (Vertex)vec.elementAt(i);
	    if(vertex.getVertexType() != START && vertex.getVertexType() != 
	       STOP) {
		file.println(",");
		start = false;

		//check if the vertex has self arc
		//
		for(int k = 0; k < graphArc.from.size(); k++) {
		    selfloop = false;
		    if((Vertex)graphArc.from.get(k) == vertex && (Vertex)graphArc.to.get(k) == vertex) {
			selfloop = true;
			wei = (String)graphArc.weights.get(k);
			double tmpw;		
			if(wei==null)
			    wei=""+"1.0";
			tmpw = Double.valueOf(wei).doubleValue();
			if(wei.equals("0.0") || tmpw == 0.0)
			    tmpw = -23.02585093;
			else
			    tmpw = Math.log(tmpw);
			wei = Double.toString(tmpw);
			epi = (String)graphArc.epsilon.get(k);

			for(int w=0; w < wordVertex.size(); w++) {
			    
			    SymbolTable s1 = (SymbolTable)wordVertex.elementAt(w);

			    if(s1.nodeIndex.equals(vertex.vertexName)) {
				src = new String("" + w);
				
				for(int n = 0; n < wordVertex.size(); n++) {
				      SymbolTable s2 = (SymbolTable)wordVertex.elementAt(n);

				      if(s2.nodeIndex.equals(vertex.vertexName)) {
					  if (start) 
					      file.println(",");
					  
					  dst = new String("" + n);
				
					  if(epi == "true") {
					      line = "        {" + src + ", " + dst + ", " + "1.0" + 
						  ", " + "true" +"}";
					  }
					  else {
					      line = "        {" + src + ", " + dst + ", " 
						  + wei +"}";
					  }		    
					  file.print(line);
					  start = true;
				      }
				} //end for loop n
			    }
			}// end for loop w
			break;
		    }
		}
		
		if (selfloop == true) {
		    file.println(",");
		}
		selfloop =false;

		// check if the vertex has any children
		//
	    
		if (vertex.getOutDegree() > 0) {
		    if(selfloop ==true)
			file.println(",");
		    start = false;
		    // get the vector of children
		    //
		    Vector children = (Vector)vertex.getChildren();
		
		    // loop over all children and write the arcs
		    //
		    for (int j=0; j < children.size(); j++) {
		
			// get the current child
			//
			Vertex child = (Vertex)children.elementAt(j);

			for(int k = 0; k < graphArc.from.size(); k++) {
			    if((Vertex)graphArc.from.get(k) == vertex && (Vertex)graphArc.to.get(k) == child) {
			    wei = (String)graphArc.weights.get(k);
			    if(wei==null)
				wei=""+"1.0";
			    double tmpw;
			    tmpw = Double.valueOf(wei).doubleValue();
			    if(wei.equals("0.0") || tmpw == 0.0)
			       tmpw = -23.02585093;
			    else
				tmpw = Math.log(tmpw);
			    wei = Double.toString(tmpw);
			    epi = (String)graphArc.epsilon.get(k);
			    break;
			    }
			}
			
			for(int p = 0; p < wordVertex.size(); p++) {

			    SymbolTable s2 = (SymbolTable)wordVertex.elementAt(p);
			    if(s2.nodeIndex.equals(vertex.vertexName)) {

				src = new String("" + p);
				if(child.getVertexType() == STOP) {
				    dst = new String("T");
				    if (start == true) {
					file.println(",");
				    }
				    if(epi == "true") {
					line = "        {" + src + ", " + dst + ", " + "1.0" + 
					    ", " + "true" +"}";
				    }
				    else {
					line = "        {" + src + ", " + dst + ", " 
					    + wei +"}";
				    }		    
				    file.print(line);
				    
				    start = true;
				}
				for(int w = 0; w < wordVertex.size(); w++) {

				    SymbolTable s1 = (SymbolTable)wordVertex.elementAt(w);

				    if(s1.nodeIndex.equals(child.vertexName)) {
					if (start) {
					    file.println(",");
					}

					dst = new String("" + w);
			    
					if(epi == "true") {
					    line = "        {" + src + ", " + dst + ", " + "1.0" + 
						", " + "true" +"}";
					}
					else {
					    line = "        {" + src + ", " + dst + ", " 
						+ wei +"}";
					}		    
					file.print(line);
					start = true;
				    }
				} // end for loop w
			    }//end if
			}//end foor loop p=0
		    }//end for loop j=0
		}
	    }
	}
	
    }


    // method: write
    //
    // arguments: none
    // return   : none
    //
    // writes the sof file
    //
    public void write() {

	// declare local variables
	//
	FileOutputStream fout = null;
	PrintWriter file = null;
	String line = null;

	// reset the save flag to indicate the obvious
	// 
	saveWork = false;

	// open the selected file for writing
	//
	try {

	    // initialize the variables
	    //
	    line = new String();

	    // initialize the file output stream
	    //
	    fout = new FileOutputStream(paramFile);
	    file = new PrintWriter(fout);

	    // write the file header
	    //
	    file.println(HEADER_TAG_00);
	    file.println(HEADER_TAG_01);
	    file.println();

	    // write the system configuration parameters
	    //
	    file.println();

	    // write the vertex data
	    //
	    for (int i=0; i < vertices.size(); i++) {

		// get the current vertex
		//
		Vertex vertex = (Vertex)vertices.elementAt(i);
 
		// write the data data if the vertex is a data
		//
		if ((vertex.getVertexType() == START) ||
		    (vertex.getVertexType() == STOP) ||
		    (vertex.getVertexType() == CUSTOM)) {
		    
		    // write the data tag
		    //
		    line = ALGO_TAG + " " + vertices.indexOf(vertex) + " @";
		    file.println(line);
		    
		    // write the data's location
		    //
		    Point point = vertex.getLocation();
		    file.println(LOCATION_KEY + " " + point.x + " " + point.y);
		    
		    // write the name tag
		    //
		    line = " name = " + BUFFER_TAG + ";";
		    file.println(line);

		    // write the variable tag
		    //
		    if (vertex.getVertexType() == CUSTOM) {
			line = " variable = " + vertex.getVertexName() + ";";
			file.println(line);
		    }
		    else if (vertex.getVertexType() == STOP ) {
			line = " variable = " + "Stop" + ";";
			file.println(line);
		    }
		    else {
			line = " variable = " + "Start" + ";";
			file.println(line);
		    }

		    // write the implementation
		    //
		    if (vertex.getVertexImpl() == START) {
			line = " type = " + START_STR + ";";
			file.println(line);
			file.println();
		    }
		    else if (vertex.getVertexImpl() == STOP) {
			line = " type = " + STOP_STR + ";";
			file.println(line);
			file.println();
		    }
		    else {
			line = " type = " + INTERNAL_STR + ";";
			file.println(line);
			file.println();
		    }
		}
	    }


	    // write the algorithm data if the vertex is a algorithm
	    //
	    for (int i=0; i < vertices.size(); i++) {

		// get the current vertex
		//
		Vertex vertex = (Vertex)vertices.elementAt(i);

		if (vertex.getVertexType() == CONTAINER) {

		    // write the algorithm tag
		    //
		    line = ALGO_TAG + " " + vertices.indexOf(vertex) + " @";
		    file.println(line);
		    
		    // write the algorithm data
		    //
		    writeAlgorithmData(file, vertex);
		    file.println();
		}		
	    }

	    // write the DiGraph header
	    //
	    file.println(DIGRAPH_TAG + "0 @");

	    // indicated that we have a weighted arc
	    //
	    file.println(WEIGHTED_KEY);

	    // write the vertex information
	    //
	    writeDiGraphVertices(file, vertices, false, -1);

	    // write the arcs information
	    //
	    writeDiGraphArcs(file, vertices, false, -1);

	    // close the file descriptor
	    //
	    file.close();
	}

	// catch and deal with any file I/O exceptions
	//
	catch(IOException e1) {}
    }

    // method: writeDiGraphVertices
    //
    // arguments:
    //    BufferedReader file: (input) file descriptor
    //    Vector vec: (input) vector of input vertices
    //    Boolean flag: indicating a container vertex
    //    Boolean offset: container sof tag 
    // return: none
    //
    // write the vertices that correspond to the digraph
    //
    public void writeDiGraphVertices(PrintWriter file, Vector vec, 
				     boolean flag, int offset) {

	// declare local variables
	//
	String line = null;

	// write the vertex tag
	//
	file.println(VERTICES_TAG);
	
	// loop over all vertices and write the vertex tags
	//
	for (int i=0; i < vec.size(); i++) {
	    int id = i+(offset * CONT_OFFSET);
	    if (i == vec.size() - 1) {
		if (flag) {
		    line = "        {" + id + ", {" + id + "}};";
		}
		else {
		    line = "        {" + i + ", {" + i + "}};";
		}
	    }
	    else {
		if (flag) {
		    line = "        {" + id + ", {" + id + "}},";
		}
		else {
		    line = "        {" + i + ", {" + i + "}},";
		}
	    }
	    file.println(line);		
	}	
    }

    // method: writeDiGraphArcs
    //
    // arguments:
    //    BufferedReader file: (input) file descriptor
    //    Vector vec: (input) vector of input vertices
    //    Boolean flag: indicating a container vertex
    //    Boolean offset: container sof tag 
    // return: none
    //
    // write the arcs that correspond to the digraph
    //
    public void writeDiGraphArcs(PrintWriter file, Vector vec, 
				 boolean flag, int offset) {

	// declare local variables
	//
	String src = null;
	String dst = null;
	String line = null;
	String wei = null;
	String epi = null;

	// write the arc information
	//
	file.println(ARCS_TAG);
	
	// loop over all vertices and examine the children
	//
	boolean start = false;
	for (int i=0; i < vec.size(); i++) {
	    
	    //get the start vertex
	    //
	    Vertex vertex = (Vertex)vec.elementAt(i);
	    
	    if(vertex.getVertexType() == START) {
		
		int index = i;
					
		// check if the vertex has any children
		//
		
		if (vertex.getOutDegree() > 0) {
		
		    // get the vector of children
		    //
		    Vector children = (Vector)vertex.getChildren();
		    
		    // loop over all children and write the arcs
		    //
		    for (int j=0; j < children.size(); j++) {
		
			// get the current child
			//
			Vertex child = (Vertex)children.elementAt(j);
			
			for(int k = 0; k < graphArc.from.size(); k++) {
			    if((Vertex)graphArc.from.get(k) == vertex && (Vertex)graphArc.to.get(k) == child) {
			    wei = (String)graphArc.weights.get(k);
			    if(wei==null)
				wei="1.0";
			    epi = (String)graphArc.epsilon.get(k);
			    break;
			    }
			}
			
			if (start) {
			    file.println(",");
			}
			src = new String("" + vec.indexOf(vertex));
			dst = new String("" + vec.indexOf(child));
			if(epi == "true") {
			line = "        {" + src + ", " + dst + ", " + "epi" + 
			    ", " + "true" +"}";
			}
			else {
			    line = "        {" + src + ", " + dst + ", " 
				+ wei + ", " + "true" +"}";
			}		    
			file.print(line);
			start = true;
		    }
		}
		break;
	    }

	}
	//file.println(",");
	writeSubDiGraphArcs(file, vec);
	file.println(";");
    }
    

    public void writeSubDiGraphArcs(PrintWriter file, Vector vec) {

	// declare local variables
	//
	String src = null;
	String dst = null;
	String wei = null;
	String epi = null;
	String line = null;
	boolean start = false;

	for (int i=0; i < vec.size(); i++) {
	
	    // get the current vertex
	    //
	    
	    Vertex vertex = (Vertex)vec.elementAt(i);
	    if(vertex.getVertexType() != START && vertex.getVertexType() != 
	       STOP) {
		file.println(",");
		start = false;

		//check if the vertex has self arc
		//
		for(int k = 0; k < graphArc.from.size(); k++) {
		    if((Vertex)graphArc.from.get(k) == vertex && (Vertex)graphArc.to.get(k) == vertex) {
			wei = (String)graphArc.weights.get(k);
			if(wei==null)
			    wei="1.0";
			epi = (String)graphArc.epsilon.get(k);
			src = new String("" + i);
			dst = new String("" + i);
			if(epi == "true") {
			line = "        {" + src + ", " + dst + ", " + "epi" + 
			    ", " + "true" +"}";
			}
			else {
			    line = "        {" + src + ", " + dst + ", " 
				+ wei + ", " + "true" +"}";
			}		    
			file.print(line);
			file.println(",");
			break;
		    }
		}
	
		// check if the vertex has any children
		//
	    
		if (vertex.getOutDegree() > 0) {
		
		    // get the vector of children
		    //
		    Vector children = (Vector)vertex.getChildren();
		
		    // loop over all children and write the arcs
		    //
		    for (int j=0; j < children.size(); j++) {
		
			// get the current child
			//
			Vertex child = (Vertex)children.elementAt(j);

			for(int k = 0; k < graphArc.from.size(); k++) {
			    if((Vertex)graphArc.from.get(k) == vertex && (Vertex)graphArc.to.get(k) == child) {
			    wei = (String)graphArc.weights.get(k);
			    if(wei==null)
				wei="1.0";
			    epi = (String)graphArc.epsilon.get(k);
			    break;
			    }
			}

			if (start) {
			    file.println(",");
			}
			src = new String("" + vec.indexOf(vertex));
			dst = new String("" + vec.indexOf(child));

			if(epi == "true") {
			line = "        {" + src + ", " + dst + ", " + "epi" + 
			    ", " + "true" +"}";
			}
			else {
			    line = "        {" + src + ", " + dst + ", " 
				+ wei + ", " + "true" +"}";
			}		    

			file.print(line);
			start = true;
		    }
		}
	    }
	}
	
    }

    // method: writeAlgorithmData
    //
    // arguments:
    //    PrintWriter file: (input) file descriptor
    //    Vertex vertex: (input) algorithm vertex
    // return: none
    //
    // write the data associated with each algorithm in the algorithm vertex
    //
    public void writeAlgorithmData(PrintWriter file, Vertex vertex) {

	// declare local variables
	//
	Point position = null;

	// store the location of the connector 
	//
	position = (Point)vertex.getLocation();

	// write the location of the output to file
	//
	if (position != null) {
	    file.print(LOCATION_KEY + " " + position.x + " " + position.y);
	    file.println();
	}

	// write the algorithm name to the file
	//
	file.println(" name = \"" + vertex.vertexName + "\";"); 
	
	// write the algorithm type to the file
	//
	if ((vertex.getType() == "Start") || 
	    (vertex.getType() == "Stop")) {
	    file.println(" type = \"" + vertex.getType() + "\";"); 
	}

	// write all other data associated with algorithm
	//
	for(int i=0; i < vertex.names.size(); i++) {
	    
	    String type = (String)(vertex.types.get(i));
	    String name = (String)(vertex.names.get(i));

	    // write nothing if the type is image
	    //
	    if (type.equals(ALGORITHM_IMAGE)) {}

	    // write nothing if the variable name is buffer_mode
	    //
	    else if (name.equals(ALGORITHM_BUFF_MODE)) {}

	    // add quotes to the value for output if the value is text
	    //
	    else if (type.equals("enum")) {

		if(!vertex.values.isEmpty()) {
		    Vector enum_values = new Vector(5,5);
		    String ww="" + vertex.values.get(0);
		    file.println(" " + vertex.names.get(i) 
				 + " = " + ww + ";");
		}

	    }
	    
	    // output the subclass if the value is a class
	    //
	    else if(type.equals(ALGORITHM_CLASS)) {

		writeSubClassData(file, vertex, i);
	
		file.println(ALGORITHM_BREAK_00);
	    }

	    // output the subparam if the value is a param
	    //
	    else if(type.equals(ALGORITHM_PARAM)) {
		writeSubParamData(file, vertex, i);
	    }
	    
	    else {
		if(!vertex.values.get(i).equals(""))
		    file.println(" " + vertex.names.get(i) 
				 + " = " + vertex.values.get(i) + ";");
	    }
	}
    }

    // method: writeSubParamData
    //
    // arguments:
    //    PrintWriter file: (input) file descriptor
    //    Vertex vertex: (input) algorithm vertex
    // return: none
    //
    // write the data associated with each algorithm in the algorithm vertex
    //
    public void writeSubParamData(PrintWriter file, Vertex vertex, int index) {

	// get a copy of the algorithm used to configure the subclass
	//
	int subparam_index = 0;
	
	for(int j=0; j < vertex.subparam.size(); j++) {
	    Vertex temp = (Vertex)vertex.subparam.get(j);
	    if (temp.association.equals((String)vertex.values.get(index))) {
		subparam_index = j;
		//break;
	    }

	}
	
	Vertex subparam_copy = 
	    (Vertex)vertex.subparam.get(subparam_index);

	// output all of the values of the subparam
	//
	for(int j=0; j < subparam_copy.values.size(); j++){
	    
	    String subtype = (String)(subparam_copy.types.get(j));
	    
	    if (subtype.equals(ALGORITHM_IMAGE)) { 
		
	    }
	    else if (subtype.equals(ALGORITHM_STRING)) {
		if(!subparam_copy.values.get(index).equals("")) {
		    file.println("   " + subparam_copy.names.get(j) 
				 + " = \"" + subparam_copy.
				 values.get(j) + "\";");
		}
	    }
	    else if(subtype.equals(ALGORITHM_PARAM)) {
		writeSubParamData(file, subparam_copy, j);
	    }
	    else {
		file.println(" " + subparam_copy.names.get(j) 
			     + " = " + subparam_copy.values.get(j) 
			     + ";");
	    }
	}
    }

    // method: writeSubClassData
    //
    // arguments:
    //    PrintWriter file: (input) file descriptor
    //    Vertex vertex: (input) algorithm vertex
    // return: none
    //
    // write the data associated with each algorithm in the algorithm vertex
    //
    public void writeSubClassData(PrintWriter file, Vertex vertex, int i) {

		// get a copy of the algorithm used to configure the subclass
		//
		int subclass_index = 0;

		for(int j=0; j < vertex.subclass.size(); j++) {
		    Vertex temp = (Vertex)vertex.subclass.get(j);
		    if (temp.association.equals((String)vertex.values.get(i)))
			subclass_index = j;
		}

		Vertex subclass_copy = 
		    (Vertex)vertex.subclass.get(subclass_index);

		// output the class name and open the brackets for outputting
		// the values of the subclass
		//
		file.println(" " + vertex.names.get(i) + " = {");

		// output all of the values of the subclass
		//
		for(int j=0; j < subclass_copy.values.size(); j++){

		    String subtype = (String)(subclass_copy.types.get(j));
		    
		    if (subtype.equals(ALGORITHM_IMAGE)) { 
			
		    }
		    else if (subtype.equals(ALGORITHM_STRING)) {
			if(!subclass_copy.values.get(i).equals("")) {
			    file.println("   " + subclass_copy.names.get(j) 
					 + " = \"" + subclass_copy.
					 values.get(j) + "\";");
			}
		    }
		    else if(subtype.equals(ALGORITHM_PARAM)) {
			writeSubParamData(file, subclass_copy, j);
		    }
		    else {
			file.println(" " + subclass_copy.names.get(j) 
				     + " = " + subclass_copy.values.get(j) 
				     + ";");
		    }
		}
}


    // method: readGraph
    //
    // arguments: none
    // return   : none
    //
    // reads the sof file
    //
    public void readGraph() {
	convertGraph();
		   
	String name = null;
	FileReader fin = null;
	BufferedReader file = null;
	StringTokenizer tokens = null;

	// clear out all existing memory
	//
	clearAllMemory();

	// open the selected file for reading
	//
	try {

	    // initialize the file output stream
	    //
	    fin = new FileReader(graphFile);
	    file = new BufferedReader(fin);
	    setVertexSymbol(file);
	    setVertex(file);
	    setArc(file);

	    // close the file descriptor
	    //
	    file.close();
	}
	// catch and deal with any file I/O exceptions
	//
	catch(IOException e1) {}
	graphFile.delete();
	graphFile = null;
    }

    // method: setVertexSymbol
    //
    // arguments: BufferedReader file
    // return   : none
    //
    // reads the sof file
    //
    public void setVertexSymbol(BufferedReader file) {
	
	String line = null;
	wordSymbol = new Vector(100, 50);

	// open the selected file for reading
	//
	try {

	    // read the lines form the file
	    //
	    while ((line = file.readLine()) != null) {
	
		String name = null;
		
		// remove white spaces at the ends of the string
		//
		line = line.trim();
		line_index++;
	
		// check for end of symbol_table
		//
		if (line.regionMatches(false, 0, "};", 0, 2)) {
		    break;
		}

		// check for symbols
		//
		if (line.regionMatches(false, 0, "\"", 0, 1)) {
		   
		        name = line.replace('\"',' ');
			name = name.trim();		 
			wordSymbol.add(name);
		}
	    }
	}
	catch(IOException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught IOException: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}
	catch(ArrayIndexOutOfBoundsException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught ArrayIndexOutOfBoundsException: 1" + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}
	catch(NullPointerException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught NullPointerException: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}	
	catch(Exception e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught Exception: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}
    }


    // method: setVertex
    //
    // arguments: BufferedReader file
    // return   : none
    //
    // reads the sof file
    //
    public void setVertex(BufferedReader file) {
	
	String line = null;
	String name = null;
	Point location = null;
	StringTokenizer tokens = null;

	// declare local variables
	//
	int xloc = 0;
	int yloc = 0;
		    
	// set the location of the object
	//
	location = new Point(xloc, WORKAREA_WIDTH/2);
	
	// set Start node
	//
	Vertex vertex = new Vertex();
	dataLabels.addElement(START_STR);
	vertex.setVertexName("Start");
	 vertex.setText("Start");
	 if ((vertex != null) && 
	     (vertex.getVertexName() != null)) {
	     vertex.setType("Start");
	 }

	 // set the location of the object
	 //
	 vertex.setLocation(location);
	 loadWidthMax = location.x > loadWidthMax ? 
	     location.x : loadWidthMax;
	 loadHeightMax = location.y > loadHeightMax ? 
	     location.y : loadHeightMax;

	 // set the vertex font
	 //
	 vertex.setFont(newCoeffFont);

	 // attach a mouse motion listener to the vertex
	 //
	 if (!vertices.contains(vertex)) {
	     vertex.addMouseListener(this);
	     vertex.addMouseMotionListener(this);
	 }
			
	 // add the vertex to the required data structures
	 //
	 workArea.add(vertex);
	 vertices.addElement(vertex);
	 tagToVertex.put("0", vertex);
	 	 
	 // add internal nodes
	 //
	 try {

	    // read the lines form the file
	    //
	    while ((line = file.readLine()) != null) {
	
		name = null;

		// remove white spaces at the ends of the string
		//
		line = line.trim();
		line_index++;

		// check for end of symbol_table
		//
		if (line.regionMatches(false, 0, "arcs", 0, 4)) {
		    break;
		}

		// check for symbols
		//
		if (line.regionMatches(false, 0, "{", 0, 1)) {
		    
		    tokens = new StringTokenizer(line, ",");
		    String typestr = "" + tokens.nextToken();
		    typestr = typestr.replace('{',' ');
		    typestr = typestr.trim();		
		    String typeAlgo = null;
		    
		    // get the type of the object
		    //
		    if (tokens.hasMoreTokens()) {
			typeAlgo = "" + tokens.nextToken();
			typeAlgo = typeAlgo.replace('{',' ');
			typeAlgo = typeAlgo.replace('}',' ');
			typeAlgo = typeAlgo.replace(';',' ');
			typeAlgo = typeAlgo.trim();
		    }
		    
		    // get the integer equivalent of the tag
		    //
		    int nodeIndex = Integer.valueOf(typestr).intValue();
		    int nodeValue = Integer.valueOf(typeAlgo).intValue();

		    String tmp = (String)wordSymbol.elementAt(nodeValue);
		    maxInputVertices++;
		    name = "S"+typestr;
		    vertex = initializeAlgorithm(name, file, tmp);
		    
		    // set the location of the object
		    //
		    vertex.setLocation(location);
		    loadWidthMax = location.x > loadWidthMax ? 
			location.x : loadWidthMax;
		    loadHeightMax = location.y > loadHeightMax ? 
			location.y : loadHeightMax;
		    
		    // set the vertex font
		    //
		    vertex.setFont(newCoeffFont);
		    
		    // attach a mouse motion listener to the vertex
		    //
		    if (!vertices.contains(vertex)) {
			vertex.addMouseListener(this);
			vertex.addMouseMotionListener(this);
		    }
		    
		    // add the vertex to the required data structures
		    //
		    workArea.add(vertex);
		    vertices.addElement(vertex);
		    int index = nodeIndex + 1;
		    tmp = "" + index;
		    tagToVertex.put(tmp, vertex);
		}		
	
	    }
	}
	catch(IOException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught IOException: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}
	catch(ArrayIndexOutOfBoundsException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught ArrayIndexOutOfBoundsException: 1" + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}
	catch(NullPointerException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught NullPointerException: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}	
	catch(Exception e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught Exception: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}

	// set Stop node
	//
	vertex = new Vertex();
	dataLabels.addElement(START_STR);
	vertex.setVertexName("Stop");
	 vertex.setText("Stop");
	 if ((vertex != null) && 
	     (vertex.getVertexName() != null)) {
	     vertex.setType("Stop");
	 }

	 // set the location of the object
	 //
	 vertex.setLocation(location);
	 loadWidthMax = location.x > loadWidthMax ? 
	     location.x : loadWidthMax;
	 loadHeightMax = location.y > loadHeightMax ? 
	     location.y : loadHeightMax;

	 // set the vertex font
	 //
	 vertex.setFont(newCoeffFont);

	 // attach a mouse motion listener to the vertex
	 //
	 if (!vertices.contains(vertex)) {
	     vertex.addMouseListener(this);
	     vertex.addMouseMotionListener(this);
	 }
			
	 // add the vertex to the required data structures
	 //
	 workArea.add(vertex);
	 vertices.addElement(vertex);
	 int index = vertices.size()-1;
	 String tmp = ""+index;
	 tagToVertex.put(tmp, vertex);	

    }
    // method: setArc
    //
    // arguments: BufferedReader file
    // return   : none
    //
    // reads the sof file
    //
    public void setArc(BufferedReader file) {

	String line = null;
	String name = null;
	Point location = null;
	StringTokenizer tokens = null;

	graphInfo = new Vector(100, 50);
	for(int i = 0; i < vertices.size(); i++) {
	    
	    NodeInfo tmp = new NodeInfo();
	    tmp.childIndex = new Vector(50,5);
	    graphInfo.add(tmp);
	}

	// add internal nodes
	//
	try {

	    //	    line = file.readLine();
	    
	    // read the rest of the data for the object
	    //
	    while ((line = file.readLine()) != null) {
		
		String parentTag = null;
		String childTag = null;
		String indexTag = null;
		String weightTag = null;
		String isWeight = null;
		Vertex childVertex = null;
		Vertex parentVertex = null;
		int index = 0;
		int nodeValue;
		int parentIn = -1;
		int childIn = -1;

		// remove leading and trailing white space letters
		//
		line = line.trim();
		line_index++;

		// check for end of search_model
		//
		if (line.regionMatches(false, 0, "};", 0, 2)) {
		    break;
		}

		// parse the parent and child tags
		//
		line = line.replace('{',' ');
		line = line.replace('}',' ');
		line = line.replace(',',' ');
		line = line.replace(';',' ');
		line = line.trim();
		
		if (!line.equals("")) {
		
		    // tokenize the line on white spaces
		    //
		    tokens = new StringTokenizer(line);
		    if (tokens.hasMoreTokens()) {
			parentTag = tokens.nextToken();
			if(parentTag.equals("S")) {
			    parentTag="0";
			    parentIn =0;
			}
			else {  
			    nodeValue = Integer.valueOf(parentTag).intValue();
			    int tmp = nodeValue + 1;
			    parentIn = tmp;
			    parentTag="" + tmp;
			}
		    }

		    if (tokens.hasMoreTokens()) {
			childTag = tokens.nextToken();
			if(childTag.equals("T")) {
			    int tmp = vertices.size() - 1;
			    childIn = tmp;
			    childTag=""+ tmp;
			}
			else {
			    nodeValue = Integer.valueOf(childTag).intValue();
			    int tmp = nodeValue + 1;
			    childIn = tmp;
			    childTag="" + tmp;
			}
		    }

		    if(parentIn >= 0 && childIn >= 0) {
			
			NodeInfo node1 = (NodeInfo)graphInfo.get(parentIn);
			node1.childIndex.add(childTag);
			graphInfo.set(parentIn, node1);
			if(parentIn == childIn) { 
			    node1.selfLoop = true;
			    graphInfo.set(parentIn, node1);
			}
		    }

		    if (tokens.hasMoreTokens()) {
			weightTag = tokens.nextToken();
		    }
		    if (tokens.hasMoreTokens()) {
			isWeight = tokens.nextToken();
		    }

		    // get the parent and child vertices
		    //
		    if (parentTag != null) {
			parentVertex = (Vertex)tagToVertex.get(parentTag);
		    }
		    if (childTag != null) {
			childVertex = (Vertex)tagToVertex.get(childTag);
		    }
			    
		    // add the child to the parents children list
		    //
		    if ((parentVertex != null) && 
			(childVertex != null)) {
			if(parentVertex != childVertex) {
			    parentVertex.addChild(childVertex);
			    childVertex.addParent(parentVertex, 0);
			}
			graphArc.from.addElement(parentVertex);
			graphArc.to.addElement(childVertex);
			weightTag.trim();	
			double tmpw;
			String wei = null;
			if(weightTag != null && !weightTag.equals("epi")) {
			    tmpw = Double.valueOf(weightTag).doubleValue();
			    if(tmpw < -23) {
				wei = ""+0.0;
			    }
			    else {
				tmpw = Math.exp(tmpw);
				tmpw = tmpw + 0.000001;
				wei = ""+tmpw;
			    }
			    if(wei.length() > 6)
				wei = wei.substring(0,6);
			}

			if(weightTag.equals("epi")) {
			    graphArc.epsilon.add("true");
			    graphArc.weights.add("1.0");
			}
			else {    
			    graphArc.weights.add(wei);
			    graphArc.epsilon.add("false");
			}
			graphArc.xloc.add("0");
			graphArc.yloc.add("0");
			graphArc.width.add("0");
			graphArc.height.add("0");
		    }
		}
	    }
	}	
	catch(IOException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught IOException: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}
	catch(ArrayIndexOutOfBoundsException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught ArrayIndexOutOfBoundsException: 1" + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}
	catch(NullPointerException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught NullPointerException: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}	
	catch(Exception e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught Exception: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}


	setCoordinate();

	// update client's preferred size because the area 
	// taken up by the work area has gotten larger
	//
	workArea.setPreferredSize(new Dimension(loadWidthMax+100, 
						loadHeightMax+100));
	
	// let the scroll pane know to update itself
	// and its corresponding scroll bars
	//
	workArea.revalidate();

	// repaint the work area
	//
	workArea.repaint();

    }

   // method: setCoordinate
    //
    // arguments: none
    // return   : none
    //
    public void setCoordinate() {

	Vector openList = new Vector(50,5);
	Vector backList = new Vector(50,5);
	Vector closeList = new Vector(50,5);
	Point location = null;
	int nodeIndex = -1;
	int currentX;
	int currentY;
	int maxX = -1;

	// declare local variables
	//
	int xloc = 0;
	int yloc = 0;
		    
	// set the location of the object
	//
	location = new Point(xloc, WORKAREA_WIDTH/2);

	 // set the location of the object
	 //
	Vertex vertex = (Vertex)vertices.get(0);
	vertex.setLocation(location);
	vertices.set(0, vertex);
	 loadWidthMax = location.x > loadWidthMax ? 
	     location.x : loadWidthMax;
	 loadHeightMax = location.y > loadHeightMax ? 
	     location.y : loadHeightMax;
	 
	 currentX = xloc;

	 //currentY = WORKAREA_WIDTH/2;
	 NodeInfo node = (NodeInfo)graphInfo.get(0);
	 for(int i = 0; i < node.childIndex.size(); i++) {
	     
	     String tmp = (String) node.childIndex.get(i);
	     openList.add(tmp);
	 }

	 int k = 0;
	 currentX = ALGORITHM_WIDTH * 3 / 2 + 20;	
	 maxX = currentX;

	 while(openList.size() >0) {

	     String tmp = (String)openList.get(k);
	     nodeIndex = Integer.valueOf(tmp).intValue();
	     NodeInfo nodeTmp = (NodeInfo)graphInfo.get(nodeIndex);
	     nodeTmp.xloc = currentX;
	     graphInfo.set(nodeIndex, nodeTmp);
	     
	     if(k == 0) {
		 /*
		 currentX = currentX + ALGORITHM_WIDTH * 3 / 2;		 
		 nodeTmp.xloc = currentX;
		 graphInfo.set(nodeIndex, nodeTmp);
		 */
	     }
	     else {
		 coordXAdd(k, openList);
	     }   
	     
	     NodeInfo nodeTmp1 = (NodeInfo)graphInfo.get(nodeIndex);

	     if(maxX < nodeTmp1.xloc)
		 maxX = nodeTmp1.xloc;
	     closeList.add(tmp);	     

	     for(int i = 0; i < nodeTmp.childIndex.size(); i++) {		 
		 String tmp1 = (String) nodeTmp.childIndex.get(i);
		 backList.add(tmp1);
	     }

	     k++;
	     if(k == openList.size()) {
		 coordYAdd(openList);
		 k=0;
		 openList.removeAllElements();
		 for(int j = 0; j < backList.size(); j++) {
		     boolean isAtCloseList = false;
		     String backTmp = (String)backList.get(j);
		     for(int m = 0; m < closeList.size(); m++) {
			 String closeTmp = (String)closeList.get(m);
			 if(backTmp.equals(closeTmp)) {
			     isAtCloseList = true;
			     break;
			 }
		     }
		     if(isAtCloseList == false) {
			 boolean isAtOpenList = false;
			 for(int n = 0; n < openList.size(); n++) {
			     String backTmp1 = (String)openList.get(n);
			     if(backTmp1.equals(backTmp)) {
				 isAtOpenList = true;
				 break;
			     }
			 }
			 if(isAtOpenList == false)
			     openList.add(backTmp);
		     }
		 } // end for loop
		 currentX = maxX + ALGORITHM_WIDTH * 3 / 2 + 20;
		 maxX = currentX;
	     } // end if
	}
	 
	 for(int i = 1; i < graphInfo.size(); i++) {
	
	     // set the location of the object
	     //
	     NodeInfo nodeTmpi = (NodeInfo)graphInfo.get(i);
	     location = new Point(nodeTmpi.xloc, nodeTmpi.yloc);

	     // set the location of the object
	     //
	     Vertex vertex1 = (Vertex)vertices.get(i);
	     vertex1.setLocation(location);
	     vertices.set(i, vertex1);
	     loadWidthMax = location.x > loadWidthMax ? 
		 location.x : loadWidthMax;
	     loadHeightMax = location.y > loadHeightMax ? 
		 location.y : loadHeightMax;
	     
	 } 
	 
    }

    // method: coordYAdd
    //
    // arguments: Vector openList
    // return   : none
    //
    // read the sof file
    //
    public void coordYAdd(Vector openList) {

	int currentY = 0;
	int middleIndex = -1;
	int yOffset = -1;

	for(int k = 0; k < openList.size(); k++) {

	    String tmp = (String)openList.get(k);
	    int nodeIndex = Integer.valueOf(tmp).intValue();
	    NodeInfo nodeTmp = (NodeInfo)graphInfo.get(nodeIndex);
	     if(k == 0) {
		 if(nodeTmp.selfLoop == true) 
		     nodeTmp.yloc += ALGORITHM_WIDTH/2;
		 currentY = nodeTmp.yloc;
		 graphInfo.set(nodeIndex, nodeTmp);
	     }
	     else {
		 if(nodeTmp.selfLoop == true) {
		     nodeTmp.yloc += currentY + ALGORITHM_WIDTH ;
		     graphInfo.set(nodeIndex, nodeTmp);
		 }
		 else {
		     nodeTmp.yloc += currentY + ALGORITHM_WIDTH/2;
		     graphInfo.set(nodeIndex, nodeTmp);
		 }
		 currentY = nodeTmp.yloc;
	     }	    
	}

	// move the y location to align the center of the graph
	//
	if(openList.size() > 0) {
	    middleIndex = openList.size()/2;
	    String tmpMiddle = (String)openList.get(middleIndex);
	    int nodeMiddleIndex = Integer.valueOf(tmpMiddle).intValue();
	    NodeInfo nodeMiddleTmp = (NodeInfo)graphInfo.get(nodeMiddleIndex);
	    if(WORKAREA_WIDTH/2 > nodeMiddleTmp.yloc)
		yOffset = WORKAREA_WIDTH/2 - nodeMiddleTmp.yloc;	    
	}
	
	if(yOffset > 0) {
	    
	    for(int k = 0; k < openList.size(); k++) {
		
		String tmp = (String)openList.get(k);
		int nodeIndex = Integer.valueOf(tmp).intValue();
		NodeInfo nodeTmp = (NodeInfo)graphInfo.get(nodeIndex);
		nodeTmp.yloc+=yOffset;
		graphInfo.set(nodeIndex, nodeTmp);
	    }
	}
    }

    // method: coordXAdd
    //
    // arguments: int k
    // arguments: Vector openList
    // return   : none
    //
    // read the sof file
    //
    public void coordXAdd(int k, Vector openList) {

	String tmp1 = (String)openList.get(k);
	int nodeIndex1 = Integer.valueOf(tmp1).intValue();
	String tmp0 = (String)openList.get(k-1);
	int nodeIndex0 = Integer.valueOf(tmp0).intValue();
	boolean connectArc = false;

	NodeInfo nodeTmp1 = (NodeInfo)graphInfo.get(nodeIndex1);

	for(int i = 0; i < nodeTmp1.childIndex.size(); i++) {

	    String tmp = (String)nodeTmp1.childIndex.get(i);
	    if(tmp.equals(tmp0)) {
		 nodeTmp1.yloc += ALGORITHM_WIDTH/2;
		 graphInfo.set(nodeIndex1, nodeTmp1);
		 connectArc = true; 
		 break;
	    }
	}

	NodeInfo nodeTmp0 = (NodeInfo)graphInfo.get(nodeIndex0);
	nodeTmp1 = (NodeInfo)graphInfo.get(nodeIndex1);
	
	for(int i = 0; i < nodeTmp0.childIndex.size(); i++) {

	    String tmp = (String)nodeTmp0.childIndex.get(i);
	    if(tmp.equals(tmp1)) {
		if(connectArc == false) {
		    nodeTmp1.yloc += ALGORITHM_WIDTH/2;
		    graphInfo.set(nodeIndex1, nodeTmp1);
		}
		else {
		    nodeTmp1.xloc = nodeTmp0.xloc + ALGORITHM_WIDTH/2;
		    graphInfo.set(nodeIndex1, nodeTmp1);
		}
		break;
	    }
	}

    }

    // method: read
    //
    // arguments: none
    // return   : none
    //
    // read the sof file
    //
    public void read() {

	// declare local variables
	//
	int xloc = 0;
	int yloc = 0;
	
	//Vector to store information for determing subclass
	//
	Vector paramInfo =  new Vector(5,5);
	
	String name = null;
	String tag = null;
	String line = null;
	String tmpName = null;

	Vertex vertex = null;
	FileReader fin = null;
	BufferedReader file = null;
	Point location = null;
	StringTokenizer tokens = null;
	String type = null;
	boolean dataBufferFlag = false;
	boolean algorithmFlag = false;
	boolean containerFlag = false;

	// clear out all existing memory
	//
	clearAllMemory();

	// open the selected file for reading
	//
	try {

	    // initialize the file output stream
	    //
	    tag = new String();
	    fin = new FileReader(paramFile);
	    file = new BufferedReader(fin);
	    
	    // read the lines form the file
	    //
	    while ((line = file.readLine()) != null) {

		// remove white spaces at the ends of the string
		//
		line = line.trim();
		line_index++;

		// check for the system configuration properties
		//
		if (line.equals(HEADER_TAG_03)) {
		    readSystem(file);
		}

		// check for algorithm objects
		//
		if (line.regionMatches(false, 0, ALGO_TAG, 0, 11)) {

		    // reset the flags
		    //
		    algorithmFlag = false;
		    containerFlag = false;
		    dataBufferFlag = false;

		    // parse the objects unique sof tag
		    //
		    tokens = new StringTokenizer(line);
		    tokens.nextToken();
		    tokens.nextToken();
		    tag = tokens.nextToken();

		    // mark the current position in the buffer stream
		    //
		    file.mark(100);

		    // read the rest of the data for the object
		    //
		    while ((line = file.readLine()) != null) {

			// reset the variables
			//
			name = null;
			type = null;
			boolean isCoeff = false;
			
			// remove leading and trailing white space letters
			//
			line = line.trim();
			line_index++;

			// break the loop when we have read the object 
			//
			if (line.regionMatches(false, 0, ALGO_TAG, 0, 11) ||
			    line.regionMatches(false, 0, ARCS_TAG, 0, 6) ||
			    line.regionMatches(false, 0, DIGRAPH_TAG, 0,15) ||
			    line.regionMatches(false, 0, VERTICES_TAG,0,10)) {

			    // reset the buffer to the last marked position
			    //
			    file.reset();
			    break;
			}

			// parse the location of the data object
			//
			if (line.regionMatches(false, 0, LOCATION_KEY, 0, 2)) {

			    // parse the X and Y coordinates
			    //
			    tokens = new StringTokenizer(line);
			    tokens.nextToken();
			    if (tokens.hasMoreTokens()) {
				String xstr = (tokens.nextToken()).trim();
				xloc = Integer.valueOf(xstr).intValue();
			    }
			    if (tokens.hasMoreTokens()) {
				String ystr = (tokens.nextToken()).trim();
				yloc = Integer.valueOf(ystr).intValue();
			    }

			    // set the location of the object
			    //
			    location = new Point(xloc, yloc);
			}

			// parse the name of the data object
			//
			if (line.regionMatches(false,0, NAME_KEY, 0,6)) {

			    // tokenize the line
			    //
			    tokens = new StringTokenizer(line, "=");
			    tokens.nextToken();
			    
			    // get the tag value of the object
			    //
			    int tagValue = Integer.valueOf(tag).intValue();

			    // get the name of the object
			    //
			    name = null;
			 
			    if (tokens.hasMoreTokens()) {
				name = tokens.nextToken(); 
				name = name.replace(';', ' ');
				name = name.trim();
				if (name.equals("\"CoefficientLabel\""))
				    isCoeff =true;
				name = name.replace('\"',' ');
				name = name.trim();
			    }

			    // determine if the name is valid
			    //
			    if (name == null) { break; }
			    
			    // determine if we are reading a data buffer object
			    //
			    if (name.equals(BUFFER_TAG) && !isCoeff &&
				(tagValue < CONT_OFFSET)) {
				dataBufferFlag = true;
				vertex = initializeDataBuffer();
				continue;
			    }
			    
			    // determine if we are reading a container object
			    // 
			    else if (name.equals(CONTAINER_TAG) &&
				     (tagValue < CONT_OFFSET)) {
				containerFlag = true;
				vertex = initializeContainer(name);
				continue;
			    }

			    // we must be reading a algorithm object
			    //
			    else {
				algorithmFlag = true;
				paramInfo = new Vector(5,5);

				// read the next line of the file
				//
				try {
				    line = file.readLine();
				    line_index++;
				}

				catch(IOException e) {
				    System.err.println();
				    System.err.println("---------------------------------------");
				    System.err.println("Caught IOException: " + e.getMessage());
				    System.err.println("Line Number: " + line_index);
				    System.err.println("File Line: " + line);
				    System.err.println("---------------------------------------");
				    System.exit(0);
				}
				catch(ArrayIndexOutOfBoundsException e) {
				    System.err.println();
				    System.err.println("---------------------------------------");
				    System.err.println("Caught ArrayIndexOutOfBoundsException: 1" + e.getMessage());
				    System.err.println("Line Number: " + line_index);
				    System.err.println("File Line: " + line);
				    System.err.println("---------------------------------------");
				    System.exit(0);
				}
				catch(NullPointerException e) {
				    System.err.println();
				    System.err.println("---------------------------------------");
				    System.err.println("Caught NullPointerException: " + e.getMessage());
				    System.err.println("Line Number: " + line_index);
				    System.err.println("File Line: " + line);
				    System.err.println("---------------------------------------");
				    System.exit(0);
				}	
				catch(Exception e) {
				    System.err.println();
				    System.err.println("---------------------------------------");
				    System.err.println("Caught Exception: " + e.getMessage());
				    System.err.println("Line Number: " + line_index);
				    System.err.println("File Line: " + line);
				    System.err.println("---------------------------------------");
				    System.exit(0);
				}
	    
				// tokenize the line
				//
				line = line.trim();
				tokens = new StringTokenizer(line, "=");
				String typestr = "" + tokens.nextToken();
				typestr = typestr.trim();
				String typeAlgo = null;

				// get the type of the object
				//
				if (tokens.hasMoreTokens()) {
				    typeAlgo = "" + tokens.nextToken();
				    typeAlgo = typeAlgo.replace('\"',' ');
				    typeAlgo = typeAlgo.replace(';', ' ');   
				    typeAlgo = typeAlgo.trim();
				}
				if(typestr.equals("type")) 
				   vertex = initializeAlgorithm(name, file, typeAlgo);
				else {
				    maxInputVertices++;
				    vertex = initializeAlgorithm(name, file, typeAlgo);		
				}
				    int updateValue = vertex.names.indexOf(typestr);
				if(updateValue>=0) {
				    String type1 =""+vertex.types.get(updateValue);
				    if(type1.equals("enum")) {
					paramInfo.add(typeAlgo);   
				    }
				}
				if(isItemOfVertex(typestr, vertex)) {
				    readAlgorithmParams(typeAlgo, line, file, 
							vertex, typestr, updateValue);
				}

				continue;
			    }
			}

			// when the line contains data buffer parameters
			//
			if (!line.startsWith(COMMENT_TAG) &&
			    !line.equals("") && dataBufferFlag && !isCoeff) {

			    // get the key value pair
			    //
			    tokens = new StringTokenizer(line, "=");

			    // parse the name of the data object
			    //
			    if (line.regionMatches(false, 0, VARIABLE_KEY, 0, 10)) {
				// set the name of the object
				//
				tokens = new StringTokenizer(line, "=");
				tokens.nextToken();
				if (tokens.hasMoreTokens()) {
				    name = tokens.nextToken();
				    name = name.replace('\"',' ');
				    name = name.replace(';', ' ');
				    name = name.trim();
				    tmpName = name;
				}
			    }
			    
			    // parse the type of the data object
			    //
			    if (line.regionMatches(false, 0, TYPE_KEY, 0, 6)) {
				
				// set the type of the object
				//
				tokens = new StringTokenizer(line, "=");
				tokens.nextToken();
				if (tokens.hasMoreTokens()) {
				    type = tokens.nextToken();
				    type = type.replace('\"',' ');
				    type = type.replace(';', ' ');
				    type = type.trim();
				    if (type.equals("Start")) {
					tmpName = "Start";
					iotype = START;
					ioimpl = START;
					dataLabels.addElement(START_STR);
				    }
				    else if (type.equals("Stop")){
					dataLabels.addElement(STOP_STR);
					tmpName = "Stop";
					iotype = STOP;
					ioimpl = STOP;
				    }
				    if (vertex != null) {
					vertex.setVertexName(tmpName);
					if (tmpName.length() > 5) {
					    vertex.setText(tmpName.substring(0, 4));
					}
					else {
					    vertex.setText(tmpName);
					}			
				    }
				    
				    if ((vertex != null) && 
					(vertex.getVertexName() != null)) {
					vertex.setType(type);
				    }
				}			    
			    }
			}

			// when the line contains container parameters
			//
			if (!line.startsWith(COMMENT_TAG) &&
			    !line.equals("") && containerFlag) {

			    // parse the name of the data object
			    //
			    if (line.regionMatches(false, 0, VARIABLE_KEY, 0, 10)) {
				
				// set the name of the object
				//
				tokens = new StringTokenizer(line, "=");
				tokens.nextToken();
				if (tokens.hasMoreTokens()) {
				    name = tokens.nextToken();
				    name = name.replace('\"',' ');
				    name = name.replace(';', ' ');
				    name = name.trim();
				    if (vertex != null) {
					vertex.setVertexName(name);
					if (name.length() > 4) {
					    vertex.setText(name.substring(0, 3));
					}
					else {
					    vertex.setText(name);
					}

				    }
				}
			    }   
			}

			// when the line contains algorithm parameters
			//
			if (!line.startsWith(COMMENT_TAG) &&
			    !line.equals("") && algorithmFlag) {

			    // get the key value pair
			    //
			    tokens = new StringTokenizer(line, "=");

			    // make sure there is a key value pair
			    //
			    if (tokens.countTokens() == 2) {
				String key = (tokens.nextToken()).trim();
				int updateValue = vertex.names.indexOf(key);
				String copyKey = new String(key);
				key = tokens.nextToken();
				key = key.replace('\"',' ');
				key = key.replace(';', ' ');   
				key = key.trim();
				if(updateValue>=0) {
				    String type1 =""+vertex.types.get(updateValue);
				    if(type1.equals("enum")) {
					paramInfo.add(key);   
				    }
				}
				if(isItemOfVertex(copyKey, vertex)) {
				    readAlgorithmParams(key, line, file, 
							vertex, copyKey, updateValue);
				}
				else {
				    if(vertex.subparam.size() > 0) {
					readSubParams(key, line, 
						      file, vertex, copyKey, updateValue, paramInfo);
				    }
				}
			    }
			}

			// mark the current position in the buffer stream
			//
			file.mark(100);
		    }

		    // make sure that the vertex is valid
		    //
		    if (vertex == null) {
			break;
		    }

		    // set the location of the object
		    //
		    vertex.setLocation(location);
		    loadWidthMax = location.x > loadWidthMax ? 
			location.x : loadWidthMax;
		    loadHeightMax = location.y > loadHeightMax ? 
			location.y : loadHeightMax;

		    // set the vertex font
		    //
		    vertex.setFont(newCoeffFont);

		    // get the integer equivalent of the tag
		    //
		    int tagValue = Integer.valueOf(tag).intValue();

		    // the vertex is part of some container
		    //
		    if (tagValue >= CONT_OFFSET) {
			
			// determine the container the vertex belongs to
			//
			int cval = (int)(tagValue/CONT_OFFSET);
			
			// get the container in order to add the vertex
			//
			String sval = new String("" + cval);
			Vertex cobj = (Vertex)tagToVertex.get(sval);
			
			// add the vertex to the container
			//
			cobj.insertContainerElement(vertex);
		    }
		    
		    // the vertex is part of the main panel
		    //
		    else {

			// attach a mouse motion listener to the vertex
			//
			if (!vertices.contains(vertex)) {
			    vertex.addMouseListener(this);
			    vertex.addMouseMotionListener(this);
			}
			
			// add the vertex to the required data structures
			//
			workArea.add(vertex);
			vertices.addElement(vertex);
		    }
		    tagToVertex.put(tag, vertex);
		}

		// check for directed arcs
		//
		if (line.regionMatches(false, 0, ARCS_TAG, 0, 6)) {

		    // mark the current position in the buffer stream
		    //
		    file.mark(100);

		    // read the rest of the data for the object
		    //
		    while ((line = file.readLine()) != null) {

			String parentTag = null;
			String childTag = null;
			String indexTag = null;
			String weightTag = null;
			String isWeight = null;
			Vertex childVertex = null;
			Vertex parentVertex = null;
			int index = 0;

			// remove leading and trailing white space letters
			//
			line = line.trim();
			line_index++;

			// break the loop when we have read the object 
			//
			if (line.regionMatches(false, 0, ALGO_TAG, 0, 11) ||
			    line.regionMatches(false, 0, ARCS_TAG, 0, 6) ||
			    line.regionMatches(false, 0, DIGRAPH_TAG, 0,15) ||
			    line.regionMatches(false, 0, VERTICES_TAG,0,10)) {

			    // reset the buffer to the last marked position
			    //
			    file.reset();
			    break;
			}

			// parse the parent and child tags
			//
			line = line.replace('{',' ');
			line = line.replace('}',' ');
			line = line.replace(',',' ');
			line = line.replace(';',' ');
			line = line.trim();

			if (!line.equals("")) {

			    // tokenize the line on white spaces
			    //
			    tokens = new StringTokenizer(line);
			    if (tokens.hasMoreTokens()) {
				parentTag = tokens.nextToken();
			    }
			    if (tokens.hasMoreTokens()) {
				childTag = tokens.nextToken();
			    }
			    if (tokens.hasMoreTokens()) {
				weightTag = tokens.nextToken();
			    }
			    if (tokens.hasMoreTokens()) {
				isWeight = tokens.nextToken();
			    }

			    // get the parent and child vertices
			    //
			    if (parentTag != null) {
				parentVertex = (Vertex)tagToVertex.get(parentTag);
			    }
			    if (childTag != null) {
				childVertex = (Vertex)tagToVertex.get(childTag);
			    }
			    
			    // add the child to the parents children list
			    //
			    if ((parentVertex != null) && 
				(childVertex != null)) {
				if(parentVertex != childVertex) {
				    parentVertex.addChild(childVertex);
				    childVertex.addParent(parentVertex, 0);
				}
				graphArc.from.addElement(parentVertex);
				graphArc.to.addElement(childVertex);
				weightTag.trim();	
				if(weightTag.equals("epi")) {
				    graphArc.epsilon.add("true");
				    graphArc.weights.add("1.0");
				}
				else {    
				    graphArc.weights.add(weightTag);
				    graphArc.epsilon.add("false");
				}
				graphArc.xloc.add("0");
				graphArc.yloc.add("0");
				graphArc.width.add("0");
				graphArc.height.add("0");
			    }
			}

			// mark the current position in the buffer stream
			//
			file.mark(100);
		    }
		}
	    }
	    
	    // close the file 
	    //
	    file.close();
	}

	catch(IOException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught IOException: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}
	catch(ArrayIndexOutOfBoundsException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught ArrayIndexOutOfBoundsException:2 " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}
	catch(NullPointerException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught NullPointerException: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}	
	catch(Exception e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught Exception: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}

	// update client's preferred size because the area 
	// taken up by the work area has gotten larger
	//
	workArea.setPreferredSize(new Dimension(loadWidthMax+100, 
						loadHeightMax+100));
	
	// let the scroll pane know to update itself
	// and its corresponding scroll bars
	//
	workArea.revalidate();

	// repaint the work area
	//
	workArea.repaint();
	
	iotype = -1;
    }


    // method:isItemOfVertex
    //
    // arguments:
    //   String copykey: (input) token from the input line
    //   Vertex vertex: (input) current recipe
    // return : boolean
    //
    boolean isItemOfVertex(String copyKey, Vertex vertex) {
	for(int i=0; i < vertex.names.size(); i++) {

	    // if enumerated type, get the first value
	    //	
	    if(vertex.names.get(i).equals(copyKey)){
		return true;
	    }
	}
	return false;
    }


    // method:readAlgorithmParams
    //
    // arguments:
    //   String key: (input) token from the input line
    //   String line: (input) current line that was read
    //   BufferedReader file: (input) file handler
    //   Vertex vertex: (input) current recipe
    //   String token4: (input) more tokens
    //   int updateValue: (input) updated values
    // return: none
    //
    // reads the configuration information for any subclasses
    // 
    public void readAlgorithmParams(String key, String line, 
				    BufferedReader file, Vertex vertex, 
				    String token4, int updateValue) {
	
	Vector subparamInfo =  new Vector(5,5);

	// take care of any subclasses
	//
	if (key.equals(ALGORITHM_BREAK_02)) {

	    try {

		// read the next line of the file
		//
		line = file.readLine();
		line_index++;

		// keep reading until we find a delimiter
		//
		while (!line.trim().equals(ALGORITHM_BREAK_01) &&
		       !line.trim().equals(ALGORITHM_BREAK_00)) {
		    
		    StringTokenizer subToken3 = 
			new StringTokenizer(line, "=");
		    
		    key = subToken3.nextToken();
		    key = key.trim();
		    String copyKey = new String(key);

		    if(!key.startsWith(COMMENT_TAG)){ 

			int n1 = vertex.names.indexOf(token4);
			int n2 = -1;

			for (int k=0; k < vertex.subclass.size(); k++){

			    if (((Vertex)vertex.subclass.get(k)).association.
				equals((String)vertex.values.get(n1))) {
				n2 = k;
			    }
			}
				
			int updateValue2 = ((Vertex)vertex.subclass.get(n2)).names.indexOf(key);
		       
			key = subToken3.nextToken();
			key = key.replace('\"',' ');
			key = key.replace(';', ' ');   
			key = key.trim(); 
				
			if(updateValue2>=0) {
			    String type1 =""+((Vertex)vertex.subclass.get(n2)).types.get(updateValue2);
			    if(type1.equals("enum")) {
				subparamInfo.add(key);   
			    }
			}
			if(isItemOfVertex(copyKey, (Vertex)vertex.subclass.get(n2))) {
			    ((Vertex)vertex.subclass.get(n2)).values.set(updateValue2, key); 
			    readAlgorithmParams(key, line, file, 
						(Vertex)vertex.subclass.get(n2), copyKey, updateValue2);
			}
			else {
			    if(((Vertex)vertex.subclass.get(n2)).subparam.size() > 0) {
				readSubParams(key, line, 
					      file, (Vertex)vertex.subclass.get(n2), copyKey, updateValue2, subparamInfo);
			    }
			}

			//((Vertex)vertex.subclass.get(n2)).values.set(updateValue2, key); 
		    }

		    // read the next line of the file
		    //
		    line = file.readLine();
		    line_index++;
		}
	    }
	    catch(IOException e) {
		System.err.println();
		System.err.println("---------------------------------------");
		System.err.println("Caught IOException: " + e.getMessage());
		System.err.println("Line Number: " + line_index);
		System.err.println("File Line: " + line);
		System.err.println("---------------------------------------");
		System.exit(0);
	    }
	    catch(ArrayIndexOutOfBoundsException e) {
		System.err.println();
		System.err.println("---------------------------------------");
		System.err.println("Caught ArrayIndexOutOfBoundsException:3 " + e.getMessage());
		System.err.println("Line Number: " + line_index);
		System.err.println("File Line: " + line);
		System.err.println("---------------------------------------");
		System.exit(0);
	    }
	    catch(NullPointerException e) {
		System.err.println();
		System.err.println("---------------------------------------");
		System.err.println("Caught NullPointerException: " + e.getMessage());
		System.err.println("Line Number: " + line_index);
		System.err.println("File Line: " + line);
		System.err.println("---------------------------------------");
		System.exit(0);
	    }	
	    catch(Exception e) {
		System.err.println();
		System.err.println("---------------------------------------");
		System.err.println("Caught Exception: " + e.getMessage());
		System.err.println("Line Number: " + line_index);
		System.err.println("File Line: " + line);
		System.err.println("---------------------------------------");
		System.exit(0);
	    }	    
	}
	else {

	    // update the value with the corresponding key
	    //
	    vertex.values.set(updateValue, key); 
	}
    }

    // method:readSubParams
    //
    // arguments:
    //   String key: (input) token from the input line
    //   String line: (input) current line that was read
    //   BufferedReader file: (input) file handler
    //   Vertex vertex: (input) current recipe
    //   String token4: (input) more tokens
    //   int updateValue: (input) updated values
    //   Vector: parameter identity  
    // return: none
    //
    // reads the configuration information for any subclasses
    // 
    public void readSubParams(String key, String line, 
			      BufferedReader file, Vertex vertex, 
			      String token4, int updateValue, 
			      Vector subInfo) {
	Vertex subparam_copy = 
		    (Vertex)vertex.subparam.get(0);
	String transform = subparam_copy.association;
	boolean isParentType = false;
	int     subStart = -1;
	int     tmpStart = -1;
	int     subEnd = -1;
	String delimit = null; 

	for(int index = 0; index < subInfo.size(); index++) {
	    subparam_copy.identify.add(subInfo.get(index));
	}

	for(int j=data.association.indexOf(transform); 
	    j <= data.association.lastIndexOf(transform);
	    j++) {
	    if(j != -1) { 
		if(!data.association.get(j).equals(transform) && subStart >=0) {
		    subEnd = j;
		    break;
		}
		if(subStart < 0) {
		    for(int k=0; k< subInfo.size(); k++) {
			String tmp1 ="["+ subInfo.get(k) + "]";
			tmp1.trim();
			String tmp2 = ""+data.values.get(j);
			if(tmp2.equals(tmp1)) {			
			    if(k == 0) {
				tmpStart =j;
				if(0 != subInfo.size()-1) {
				   
				}
			    }
			    if(k == subInfo.size()-1) {
				delimit = (String)data.name.get(tmpStart);
				subStart = tmpStart;
				j++;
				break;
			    }
			    j++;
			}
			else {
			    break;
			}
		    }
		}	     
		if(subEnd < 0) {
		    String tmp3 = ""+data.name.get(j);
		    if(tmp3.equals(delimit)) {
			subEnd =j;
			break;
		    }
		    if(subEnd < 0 && j == data.association.lastIndexOf(transform)) {
			subEnd = j+1;
		    }
		}
	    }
	}
	
	subparam_copy.values = new Vector(5,5);
	subparam_copy.names = new Vector(5,5);
	subparam_copy.types = new Vector(5,5);

	// keep reading until we find a delimiter
	//
	for(int index = subStart+subInfo.size(); index < subEnd; index++) {
	    
	    // get the key value pair
	    //
	    StringTokenizer tokens = new StringTokenizer(line, "=");
	    
	    // make sure there is a key value pair
	    //
	    if (tokens.countTokens() == 2) {
		String key1 = (tokens.nextToken()).trim();
		String copyKey1 = new String(key1);
		key1 = tokens.nextToken();
		key1 = key1.replace('\"',' ');
		key1 = key1.replace(';', ' ');   
		key1 = key1.trim();
		subparam_copy.values.add(key1);
		subparam_copy.names.add(data.name.get(index));
		subparam_copy.types.add(data.type.get(index));
	    }
	    if(index == subEnd - 1)
		break;

	    // read the next line of the file
	    //
	    try {
	    line = file.readLine();
	    line_index++;
	    }    
	    catch(IOException e) {
		System.err.println();
		System.err.println("---------------------------------------");
		System.err.println("Caught IOException: " + e.getMessage());
		System.err.println("Line Number: " + line_index);
		System.err.println("File Line: " + line);
		System.err.println("---------------------------------------");
		System.exit(0);
	    }
	    catch(ArrayIndexOutOfBoundsException e) {
		System.err.println();
		System.err.println("---------------------------------------");
		System.err.println("Caught ArrayIndexOutOfBoundsException:3 " + e.getMessage());
		System.err.println("Line Number: " + line_index);
		System.err.println("File Line: " + line);
		System.err.println("---------------------------------------");
		System.exit(0);
	    }
	    catch(NullPointerException e) {
		System.err.println();
		System.err.println("---------------------------------------");
		System.err.println("Caught NullPointerException: " + e.getMessage());
		System.err.println("Line Number: " + line_index);
		System.err.println("File Line: " + line);
		System.err.println("---------------------------------------");
		System.exit(0);
	    }	
	    catch(Exception e) {
		System.err.println();
		System.err.println("---------------------------------------");
		System.err.println("Caught Exception: " + e.getMessage());
		System.err.println("Line Number: " + line_index);
		System.err.println("File Line: " + line);
		System.err.println("---------------------------------------");
		System.exit(0);
	    }	    
	}
    }
	

    // method: readSystem
    //
    // arguments:
    //    PrintWriter file: (input) file descriptor
    // return: none
    //
    // read the system configuration properties from the sof file
    //
    public void readSystem(BufferedReader file) {

	// declare local variables
	//
	String token1 = null;
	String token2 = null;
	String token3 = null;
	String token4 = null;
	String line = null;
	String line_backup = null;

	char commaToken = ';';
	char nullToken = ' ';

	int index = -1;
	int subclassIndex = -1;

	Point point = null;
	Vertex algorithmVertex = null;
	boolean firstProcess = true;

	try {

	    // read the line from the file and remove end white spaces
	    // 
	    // EX: name = AudioFrontEnd;
	    //
	    line = file.readLine();
	    line = line.trim();
	    line_index++;

	    // read the rest of the system block description
	    //
	    // EX: name = AudioFrontEnd;
	    // EX: output_mode = RAW_FEATURES;
	    // EX: audio = {
	    // EX: audio_output = {
	    //
	    while (!line.equals("") && line != null) {

		// create a sub tokenizer parse the algorithm fields
		//
		StringTokenizer subToken = new StringTokenizer(line, "=");
		index = -1;
		
		// get the key associated with the field
		//
		// EX: name
		// EX: output_mode
		// EX: audio
		//
		if (subToken.hasMoreTokens()) {
		    token1 = subToken.nextToken();
		    token1 = token1.replace(commaToken, nullToken);
		    token1 = token1.trim();
		    //index = systemVertex.names.indexOf(token1);
		}

		// get the value associated with the field
		//
		// EX: AudioFrontEnd
		// EX: RAW_FEATURES
		// EX: {
		//
		if(subToken.hasMoreTokens()) {

		    token2 = subToken.nextToken();
		    token2 = token2.replace('\"', nullToken);
		    token2 = token2.replace(commaToken, nullToken);
		    token2 = token2.trim();

		    if (token1.equals("inputflag")) {
			if(token2.equals("true"))
			    inputBlockExist = true;
			else
			    inputBlockExist = false; 
		    }	    
		    if (token1.equals("outputflag")) {
			if(token2.equals("true"))
			    outputBlockExist = true;
			else
			    outputBlockExist = false; 
		    }
		    if (token1.equals("target")) {
			targetStr = token2;
		    }
		    
		    if (token1.equals("frame_duration")) {
			 int index_1 =inVertex.names.indexOf(token1);
			 inVertex.values.set(index_1, token2);
		    }
		    if (token1.equals("signal_duration")) {
			 int index_2 =inVertex.names.indexOf(token1);
			 inVertex.values.set(index_2, token2);
		    }
		    if (token1.equals("number_of_features")) {
			 int index_3 =inVertex.names.indexOf(token1);
			 inVertex.values.set(index_3, token2);
		    }


		    if (token2.equals("{")) {
			if(token1.equals("audio_input")) {
			    line = file.readLine();
			    line = line.trim();
			    line_index++;

			    // read the audio and audio_output parameters
			    //
			    // EX: byte_order = BIG_ENDIAN;
			    // EX: sample_frequency = 8000.0;
			    //
			    while (!line.equals("") && line != null && 
				   !line.equals(ALGORITHM_BREAK_00)) {

				// create a sub tokenizer parse the algorithm
				//
				StringTokenizer subToken2 = 
				    new StringTokenizer(line, "=");
				
				// get the key associated with the field
				//
				// EX: byte_order
				// EX: sample_frequency
				//
				if (subToken2.hasMoreTokens()) {
				    token3 = subToken2.nextToken();
				    token3 = token3.replace(commaToken, nullToken);
				    token3 = token3.trim();
				    index =inVertex.names.indexOf(token3);
				}
				
				// get the value associated with the field
				//
				// EX: BIG_ENDIAN
				// EX: 8000.0
				//
				if(subToken2.hasMoreTokens()) {
				    token4 = subToken2.nextToken();
				    token4 = token4.replace(commaToken, nullToken);
				    token4 = token4.replace('\"', nullToken);
				    token4 = token4.trim();
				    if((index != -1) && 
				       (!token4.equals("{"))) {
					inVertex.values.set(index, token4);
				    }
				}
					
				if((line = file.readLine()) != null) {
				    line = line.trim();
				    line_index++;
				}
			    }
			} //end "audio_input"
			else if(token1.equals("audio_output")) { 
			    line = file.readLine();
			    line = line.trim();
			    line_index++;

			    // read the audio and audio_output parameters
			    //
			    // EX: byte_order = BIG_ENDIAN;
			    // EX: sample_frequency = 8000.0;
			    //
			    while (!line.equals("") && line != null && 
				   !line.equals(ALGORITHM_BREAK_00)) {

				// create a sub tokenizer parse the algorithm
				//
				StringTokenizer subToken2 = 
				    new StringTokenizer(line, "=");
				
				// get the key associated with the field
				//
				// EX: byte_order
				// EX: sample_frequency
				//
				if (subToken2.hasMoreTokens()) {
				    token3 = subToken2.nextToken();
				    token3 = token3.replace(commaToken, nullToken);
				    token3 = token3.trim();
				    index =outVertex.names.indexOf(token3);
				}
				
				// get the value associated with the field
				//
				// EX: BIG_ENDIAN
				// EX: 8000.0
				//
				if(subToken2.hasMoreTokens()) {
				    token4 = subToken2.nextToken();
				    token4 = token4.replace(commaToken, nullToken);
				    token4 = token4.replace('\"', nullToken);
				    token4 = token4.trim();
				    if((index != -1) && 
				       (!token4.equals("{"))) {
					outVertex.values.set(index, token4);
				    }
				}
					
				if((line = file.readLine()) != null) {
				    line = line.trim();
				    line_index++;
				}
			    }
			}// end "audio_out"
		   }
		}
		else {
		    System.out.println("Unhandled line: " + line);
		}
		
		if((line = file.readLine()) != null) {
		    line = line.trim();
		    line_index++;
		}
	    }
	}

	catch(IOException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught IOException: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}
	catch(ArrayIndexOutOfBoundsException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught ArrayIndexOutOfBoundsException:4 " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}
	catch(NullPointerException e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught NullPointerException: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}	
	catch(Exception e) {
	    System.err.println();
	    System.err.println("---------------------------------------");
	    System.err.println("Caught Exception: " + e.getMessage());
	    System.err.println("Line Number: " + line_index);
	    System.err.println("File Line: " + line);
	    System.err.println("---------------------------------------");
	    System.exit(0);
	}
    }

    // ---------------------------------------------------
    //
    // class required methods
    //
    // ---------------------------------------------------

    // method: mouseClicked
    //
    // arguments:
    //    MouseEvent e: (input) event that was fired
    // return: none
    //
    public void mouseClicked(MouseEvent e) {

	// declare local variables
	//
	boolean forwardLink = false;
	boolean reverseLink = false;

	Vertex conChild = null;
	Vertex conParent = null;

	// did the user click on the WorkArea with the left mouse button?
	//
	if (e.getSource().getClass().getName().equals("WorkArea")) {
	    
	    // reset the current focus
	    //
	    currVertexFocus = null;

	    if (SwingUtilities.isLeftMouseButton(e)) {

		// declare local variables
		//
		double smallestDistance = Double.MAX_VALUE;
		Vertex smallestParent = null;
		Vertex smallestChild = null;

		// has the arc deletion option been enabled?
		//
		if (enableArcDeletion) {		   

		    // set the save flag to indicate the obvious
		    //
		    saveWork = true;		    

		    // get the current cursor's location
		    //
		    Point location = e.getPoint();
		    for(int k = 0; k < graphArc.from.size(); k++) {
			if((Vertex)graphArc.from.get(k) == (Vertex)graphArc.to.get(k)) {
	
			    String xtmp = "" + graphArc.xloc.get(k);
			    int xloc1 = Integer.valueOf(xtmp).intValue();
			    String ytmp = "" + graphArc.yloc.get(k);
			    int yloc1 = Integer.valueOf(ytmp).intValue();
			    String wtmp = "" + graphArc.width.get(k);
			    int wleng = Integer.valueOf(wtmp).intValue();
			    String htmp = "" + graphArc.height.get(k);
			    int hleng = Integer.valueOf(htmp).intValue();
	
			    if(location.x >= xloc1 && location.x <= xloc1 + wleng && location.y >= yloc1 && location.y <= yloc1 + hleng) {
				graphArc.from.remove(k);
				graphArc.to.remove(k);
				graphArc.weights.remove(k);
				graphArc.epsilon.remove(k);
				graphArc.xloc.remove(k);
				graphArc.yloc.remove(k);
				graphArc.width.remove(k);
				graphArc.height.remove(k);
				break;
			    }
			}
			else {
			
			    String xtmp = "" + graphArc.xloc.get(k);
			    int xloc1 = Integer.valueOf(xtmp).intValue();
			    String ytmp = "" + graphArc.yloc.get(k);
			    int yloc1 = Integer.valueOf(ytmp).intValue();
	    
			    String wtmp = "" + graphArc.width.get(k);
			    int xloc2 = Integer.valueOf(wtmp).intValue();
			    String htmp = "" + graphArc.height.get(k);
			    int yloc2 = Integer.valueOf(htmp).intValue();
			    
			    float angle = (float)(yloc2 - yloc1) / (float)(xloc2 - xloc1);
			    int lowx = 0;
			    int highx = 0;
			    int lowy = 0;
			    int highy = 0;
			    if(xloc1 >= xloc2) {
				highx = xloc1;
				lowx = xloc2;
			    }
			    else {
				highx = xloc2;
				lowx = xloc1;
			    }

			    if(yloc1 >= yloc2) {
				highy = yloc1;
				lowy = yloc2;
			    }
			    else {
				highy = yloc2;
				lowy = yloc1;
			    }
			    
			    float rule1 = yloc2 - (float)location.y - angle * ( xloc2 - (float)location.x); 
			    float rule2 = yloc1 -1 - (float)location.y - angle * ( xloc1 -(float) location.x);
			    float rule3 = yloc1 - (float)location.y - angle * ( xloc1 - 1 - (float)location.x);
			    float rule4 = yloc1 - 2 - (float)location.y - angle * ( xloc1  - (float)location.x);
			    float rule5 = yloc1 + 1 - (float)location.y - angle * ( xloc1  - (float)location.x);
			    float rule6 = yloc1 + 2 - (float)location.y - angle * ( xloc1  - (float)location.x);
	
			    if(((rule1 >= -2 && rule1 <= 2) || (rule2 >= -2 && rule2 <=2) || (rule3 >= -2 && rule3 <= 2) || (rule4 >=-2 && rule4 <= 2) || (rule5 >= -2 && rule5 <= 2) || (rule6 >= -2 && rule6 <= 2)) && location.x >= lowx && location.x <= highx && location.y >= lowy && location.y <= highy) {
				Vertex tmp1 = (Vertex)graphArc.from.elementAt(k);
				Vertex tmp2 = (Vertex)graphArc.to.elementAt(k);
				if ((tmp1 != null) && 
				    (tmp2 != null)) {
				    tmp1.removeChild(tmp2);
				    tmp2.removeParent(tmp1);
				}	

				graphArc.from.remove(tmp1);
				graphArc.to.remove(tmp2);
				graphArc.weights.remove(k);
				graphArc.epsilon.remove(k);
				graphArc.xloc.remove(k);
				graphArc.yloc.remove(k);
				graphArc.width.remove(k);
				graphArc.height.remove(k);
				break;
			    }

			}
		    }
	     
		    // reset and repaint
		    //
		    resetVertexFocus();
		    return;
		}
		
		if (iotype >=0) {
		    createData();
		    iotype = -1;
		}

		// is the newly created vertex valid?
		//
		if (initVertexFocus != null) {		    

		    // set the save flag to indicate the obvious
		    //
		    saveWork = true;		    

		    // get the current cursor's location
		    //
		    Point location = e.getPoint();
		    
		    // retrieve the vertex dimensions
		    //
		    int vertexWidth = initVertexFocus.getVertexWidth();
		    int vertexHeight = initVertexFocus.getVertexHeight();
		    
		    // center the vertex around the mouse cursor
		    //
		    location.x = location.x - (vertexWidth / 2);
		    location.y = location.y - (vertexHeight / 2);
		    
		    // set the location of the vertex
		    //
		    initVertexFocus.setLocation(location);
		    
		    // add the vertex to the work area
		    //
		    workArea.add(initVertexFocus);
		    vertices.add(initVertexFocus);
		    
		    // Create an instance of java.awt.Cursor
		    Cursor c = new Cursor ( Cursor.DEFAULT_CURSOR );
		    this.setCursor (c);
		    
		    // reset the focus
		    //
		    resetVertexFocus();		    
		    return;
		}

		// get the current cursor's location
		//
	        Point location = e.getPoint();
		for(int k = 0; k < graphArc.from.size(); k++) {
		    if((Vertex)graphArc.from.get(k) == (Vertex)graphArc.to.get(k)) {
	
			String xtmp = "" + graphArc.xloc.get(k);
			int xloc1 = Integer.valueOf(xtmp).intValue();
			String ytmp = "" + graphArc.yloc.get(k);
			int yloc1 = Integer.valueOf(ytmp).intValue();	    
			String wtmp = "" + graphArc.width.get(k);
			int wleng = Integer.valueOf(wtmp).intValue();
			String htmp = "" + graphArc.height.get(k);
			int hleng = Integer.valueOf(htmp).intValue();	
			if(location.x >= xloc1 && location.x <= xloc1 + wleng && location.y >= yloc1 && location.y <= yloc1 + hleng) {
			    ConfigBoxArc arcConfig = new ConfigBoxArc(this,data,outVertex, workArea, 0, k);
			    arcConfig.pack();
			    arcConfig.setVisible(true);
			    graphArc.weights.set(k, tmpweight);
			    graphArc.epsilon.set(k, tmpepi);
			    tmpweight = null;
			    tmpepi = null;
			    break;
			}
		    }
		    else {
			
			String xtmp = "" + graphArc.xloc.get(k);
			int xloc1 = Integer.valueOf(xtmp).intValue();
			String ytmp = "" + graphArc.yloc.get(k);
			int yloc1 = Integer.valueOf(ytmp).intValue();	    
			String wtmp = "" + graphArc.width.get(k);
			int xloc2 = Integer.valueOf(wtmp).intValue();
			String htmp = "" + graphArc.height.get(k);
			int yloc2 = Integer.valueOf(htmp).intValue();

			float angle = (float)(yloc2 - yloc1) / (float)(xloc2 - xloc1);
			int lowx = 0;
			int highx = 0;
			int lowy = 0;
			int highy = 0;
			if(xloc1 >= xloc2) {
			    highx = xloc1;
			    lowx = xloc2;
			}
			else {
			    highx = xloc2;
			    lowx = xloc1;
			}

			if(yloc1 >= yloc2) {
			    highy = yloc1;
			    lowy = yloc2;
			}
			else {
			    highy = yloc2;
			    lowy = yloc1;
			}

			float rule1 = yloc2 - (float)location.y - angle * ( xloc2 - (float)location.x); 
			float rule2 = yloc1 -1 - (float)location.y - angle * ( xloc1 -(float) location.x);
			float rule3 = yloc1 - (float)location.y - angle * ( xloc1 - 1 - (float)location.x);
			float rule4 = yloc1 - 2 - (float)location.y - angle * ( xloc1  - (float)location.x);
			float rule5 = yloc1 + 1 - (float)location.y - angle * ( xloc1  - (float)location.x);
			float rule6 = yloc1 + 2 - (float)location.y - angle * ( xloc1  - (float)location.x);
	
			if(((rule1 >= -2 && rule1 <= 2) || (rule2 >= -2 && rule2 <=2) || (rule3 >= -2 && rule3 <= 2) || (rule4 >=-2 && rule4 <= 2) || (rule5 >= -2 && rule5 <= 2) || (rule6 >= -2 && rule6 <= 2)) && location.x >= lowx && location.x <= highx && location.y >= lowy && location.y <= highy) {
			    
			    ConfigBoxArc arcConfig = new ConfigBoxArc(this,data,outVertex, workArea, 0, k);
			    arcConfig.pack();
			    arcConfig.setVisible(true);
			    graphArc.weights.set(k, tmpweight);
			    graphArc.epsilon.set(k, tmpepi);
			    tmpweight = null;
			    tmpepi = null;
			    break;
			}

		    }
		}
	    }
	}

	// did the user click on a vertex with the right mouse button?
	//
	if (e.getSource().getClass().getName().equals("Vertex")) {
	    if(SwingUtilities.isRightMouseButton(e)) {

		// set the save flag to indicate the obvious
		//
		saveWork = true;

		// get the vertex that is being referenced
		//
		cfgVertexFocus = (Vertex)e.getSource();

		// display the popup menu based on the vertex type
		//
		//popup.show(e.getComponent(), e.getX(), e.getY());

		// create the configuration window for the CONTAINER
		//
		if (cfgVertexFocus.getVertexType() == CONTAINER) {
		    ConfigBoxWord config_box = new ConfigBoxWord(this,data, 
							 cfgVertexFocus, 
							 workArea, 0);
		    config_box.pack();
		    config_box.setVisible(true);
		}

		// create the configuration window for the start/stop data
		//
		else {

		    // create a frame to display the properties
		    //
		    if (config != null) {
			config.setVisible(false);
		    }
		    config = new JFrame("Label");
		    Point xy = getLocation();
		    config.setLocation((int)xy.getX() + 100,(int)xy.getY() + 200);

		    nameField = new JTextField(10);
		    
		    // add a key listener to the textfield
		    //
		    nameField.addKeyListener(new KeyAdapter() { 
			    public void keyPressed(KeyEvent e) { 
				
				// determine if the enter key was pressed
				//
				if (e.getKeyCode() == e.VK_ENTER) {
				    
				    // check to see if a vertex with
				    // this label exists
				    //
				    if (coeffLabels.contains(nameField.getText())) {
					// display a warning message
					// indicating that you cannot
					// have two vertices with the
					// same name
					//
					JFrame frame = new JFrame("Warning");
					JOptionPane.showMessageDialog(frame, 
					       "A node with the label " + 
					            nameField.getText() + 
						       " already exists");
					return;
				    }
				    if (cfgVertexFocus.getVertexImpl() == NONE) {
					if (dataLabels.contains(nameField.getText())) {
					    // display a warning message
					    // indicating that you cannot
					    // have two vertices with the
					    // same name
					    //
					    JFrame frame = new JFrame("Warning");
					    JOptionPane.showMessageDialog(frame, 
						    "A node with the label " + 
						         nameField.getText() + 
							    " already exists");
					    return;
					}
				    }

				    // remove the old and add the new label
				    //
				    if (cfgVertexFocus.getVertexImpl() == NONE) {
					coeffLabels.remove(cfgVertexFocus.getVertexName());
					coeffLabels.addElement(nameField.getText());
				    }
				    else {
					dataLabels.remove(cfgVertexFocus.getVertexName());
					dataLabels.addElement(nameField.getText());
				    }

				    // update the vertex text with
				    // what the user entered
				    //
				    if (nameField.getText().length() > 3) {
					cfgVertexFocus.setText(nameField.getText().substring(0, 3));
				    }
				    else {
					cfgVertexFocus.setText(nameField.getText());
				    }
				    cfgVertexFocus.setVertexName(nameField.getText());
				    
				    // if the block is of type STOP then
				    // update the system target
				    //
				    if (cfgVertexFocus.getVertexImpl() == STOP) {
					targetStr = "" + cfgVertexFocus.getVertexName();
				    }
				    
				    // remove the frame from the screen
				    //
				    config.setVisible(false);
				}
			    }
			});
		    
		    // create labels to be included in the frame
		    //
		    nameLabel = new JLabel("Label: ");
		    nameLabel.setFont(newFont);
		    nameLabel.setForeground(Color.black);
		    
		    // create buttons for the frame
		    //
		    update = new String("Update");
		    JButton done = new JButton(update);
		    done.addActionListener(this);
		    
		    cancel = new String("Cancel");
		    JButton ignore = new JButton(cancel);
		    ignore.addActionListener(this);

		    // retrieve the text from the vertex and set the textfield
		    //
		    String text = cfgVertexFocus.getVertexName();
		    nameField.setText(text);
		    
		    // declare the default layout for the frame
		    //
		    config.getContentPane().setLayout(new GridLayout(2,2));

		    // add the textfields, labels and buttons to the frame
		    //
		    config.getContentPane().add(nameLabel);
		    config.getContentPane().add(nameField);
		    config.getContentPane().add(done);
		    config.getContentPane().add(ignore);
		    
		    // display the frame
		    //
		    config.pack();
		    config.setVisible(true);
		}

		// reset the focus
		//
		resetVertexFocus();
		return;
	    }
	}

	// did the user click on a vertex with the left mouse button?
	//
	if (e.getSource().getClass().getName().equals("Vertex")) {
	    if (SwingUtilities.isLeftMouseButton(e)) {	    

		// set the save flag to indicate the obvious
		//
		saveWork = true;

		// updates the vertex focus to highlight the current parent
		//
		updateVertexFocus(e);
		
		// exit the method if the vertex child is not instantiated
		//
		if (vertexChild == null && ! enableSelfArcInsertion) { 
		    return; 
		}
		else if( enableSelfArcInsertion) {

		    // the input vertices cannot have incoming arcs
		    //
		    if (vertexParent.getVertexImpl() == START || vertexParent.getVertexImpl() == STOP ) {
			Toolkit.getDefaultToolkit().beep();
			resetVertexFocus();
			return;
		    }
		    else {
			graphArc.from.addElement(vertexParent);
			graphArc.to.addElement(vertexParent);
			graphArc.weights.add(tmpweight);
			graphArc.epsilon.add(tmpepi);
			graphArc.xloc.add("0");
			graphArc.yloc.add("0");
			graphArc.width.add("0");
			graphArc.height.add("0");
			
			// reset the vertex parent and vertex child pointers
			//
			resetVertexFocus();
			
			// update the work area
			//
			workArea.repaint();
			return;
		    }
		}

		// the input vertices cannot have incoming arcs
		//
		if (vertexChild.getVertexImpl() == START) {
		    Toolkit.getDefaultToolkit().beep();
		    resetVertexFocus();
		    return;
		}

		// the output vertices cannot have outgoing arcs
		//
		if(vertexParent.getVertexImpl() == STOP) {
		    Toolkit.getDefaultToolkit().beep();
		    resetVertexFocus();
		    return;
		}
		
		// make sure are drawing is enabled
		//
		if (!enableArcInsertion) {
		    resetVertexFocus();
		    return;
		}
		
		else {

		    // check for forward and backward edges
		    //
		    if (vertexParent.containsChild(vertexChild)) {
			forwardLink = true;
		    }
		    if (vertexChild.containsChild(vertexParent)) {
			reverseLink = true;
		    } 
		    // if neither a forward nor backward link exist add
		    // the connector
		    //
		    if (!forwardLink || !reverseLink) {
			
			if (vertexChild == vertexParent) {
			    resetVertexFocus();
			    return;
			}
			else {
			    graphArc.from.addElement(vertexParent);
			    graphArc.to.addElement(vertexChild);
			    graphArc.weights.add(tmpweight);
			    graphArc.epsilon.add(tmpepi);
			    graphArc.xloc.add("0");
			    graphArc.yloc.add("0");
			    graphArc.width.add("0");
			    graphArc.height.add("0");

			    // add the parent from the child lis and vice-versa
			    //
			    if(vertexChild != null) {
				vertexParent.addChild(vertexChild);
			    }
			    if(vertexChild != null) {
				vertexChild.addParent(vertexParent);
			    }
			}
		    }
		}
		
		// reset the vertex parent and vertex child pointers
		//
		resetVertexFocus();
	    }
	}

	// update the work area
	//
	workArea.repaint();
    }

    // method: distance
    //
    // arguments: 
    //    int x1: (input) x-coordinate of the first point
    //    int y1: (input) y-coordinate of the first point
    //    int x2: (input) x-coordinate of the second point
    //    int y2: (input) y-coordinate of the second point
    // return: distance between the points
    //
    // calculate the euclidean distance between the two points
    //
    public double distance(int x1, int y1, int x2, int y2) {

        double distance = 0.0;

        // claculate the euclidean distance
        //
        double deltaX = x2 - x1;
        double deltaY = y2 - y1;
        double sqrX = deltaX * deltaX;
        double sqrY = deltaY * deltaY;

        distance = Math.sqrt((double)sqrX + (double)sqrY);

        // return the distance
        //
        return distance;
    }

    // method: mousePressed
    //
    // arguments:
    //    MouseEvent e: (input) event that was fired
    // return: none
    //
    public void mousePressed(MouseEvent e) {

	// did the user click on a vertex with the left mouse button?
	//
	if (e.getSource().getClass().getName().equals("Vertex")) {
	    if (SwingUtilities.isLeftMouseButton(e)) {
		if (currVertexFocus != null) {
		    currVertexFocus.getModel().setSelected(false);
		}
		currVertexFocus = (Vertex)e.getSource();
		currVertexFocus.getModel().setSelected(true);
	    }
	}

	// repaint the work area
	//
	workArea.repaint();
    }
        
    // method: mouseDragged
    //
    // arguments:
    //    MouseEvent e: (input) event that was fired
    // return: none
    //
    public void mouseDragged(MouseEvent e) {	

	// declare local variables
	//
	int xdelta = 0;
	int ydelta = 0;
	int newWidth = 0;
	int newHeight = 0;
	boolean forwardLink = false;
	boolean reverseLink = false;

	// did the user click on a vertex with the left mouse button?
	//
	if (e.getSource().getClass().getName().equals("Vertex")) {
	    if (SwingUtilities.isLeftMouseButton(e)) {

		// get the current location of the vertex
		//
		if (currVertexFocus != null) {

		    // determine the location and width of the vertex
		    //
		    Point loc = currVertexFocus.getVertexLocation();
		    
		    int vertexWidth = currVertexFocus.getVertexWidth();
		    int vertexHeight = currVertexFocus.getVertexHeight();
		    
		    int xloc = (loc.x + e.getX() - (vertexWidth / 2));
		    int yloc = (loc.y + e.getY() - (vertexHeight / 2));
		    
		    int xloc1 = loc.x + e.getX() + (vertexWidth / 2);
		    int yloc1 = loc.y + e.getY() + (vertexHeight / 2);

		    // make sure the user does not move the vertex out of
		    // the work area
		    //
		    if ((xloc >= 0) && (yloc >= 0)) {
			
			// determine the new width for the work area
			//
			if (xloc1 > workArea.getWidth()) {

			    xdelta = xloc1 - workArea.getWidth();
			    newWidth = workArea.getWidth() + xdelta;
			}
			
			// determine the new height for the work area
			//
			if (yloc1 > workArea.getHeight()) {
                            
			    ydelta = yloc1 - workArea.getHeight();
			    newHeight = workArea.getHeight() + ydelta;
			}

			// update the scroll bars of the work area
			//
			if ((xloc1 >= workArea.getWidth()) || 
			    (yloc1 >= workArea.getHeight())) {

			    // scroll the work area to the new region
			    //
			    Rectangle rect = new Rectangle(xloc, yloc, 
							   vertexWidth, 
							   vertexHeight);
			    workArea.scrollRectToVisible(rect);
			    
			    // make sure that the width and does not
			    // decrease, i.e., it does not scroll back
			    //
			    if (newWidth <= maxWidth) {
				newWidth = maxWidth;
			    } else {
				maxWidth = newWidth;
			    }
			    
			    // make sure that the height and does not
			    // decrease, i.e., it does not scroll back
			    //
			    if (newHeight <= maxHeight) {
				newHeight = maxHeight;
			    } else {
				maxHeight = newHeight;
			    }
			    
			    // update client's preferred size because the area 
			    // taken up by the work area has gotten larger
			    //
			    workArea.setPreferredSize(new Dimension(newWidth, 
								   newHeight));
			    
			    // let the scroll pane know to update itself
			    // and its corresponding scroll bars
			    //
			    workArea.revalidate();
			}
			
			// drag the vertex to the location the user specifies
			//
			currVertexFocus.setLocation(xloc, yloc);
			currVertexFocus.updateLocation(xloc, yloc);
		    }
		}

		// when the arc insertion is enabled
		//
		if (enableArcInsertion) {
		    
		    // set the save flag to indicate the obvious
		    //
		    saveWork = true;
		    
		    // updates the vertex focus to highlight the current parent
		    //
		    updateVertexFocus(e);
		    
		    // exit the method if the vertex child is not instantiated
		    //
		    if (vertexChild == null) { 
			return; 
		    }
		    
		    // the input vertices cannot have incoming arcs
		    //
		    if (vertexChild.getVertexImpl() == START) {
			Toolkit.getDefaultToolkit().beep();
			resetVertexFocus();
			return;
		    }
		    
		    // the output vertices cannot have outgoing arcs
		    //
		    if(vertexParent.getVertexImpl() == STOP) {
			Toolkit.getDefaultToolkit().beep();
			resetVertexFocus();
			return;
		    }
		    /*
		    // check for self loops and exit if it exists
		    //
		    if (vertexChild == vertexParent) {
			
			if ((vertexParent.getVertexType() == CONTAINER) &&
			    (vertexChild.getVertexType() == CONTAINER)) {
			    expandContainer();
			    resetVertexFocus();
			    return;
			}
			else {
			    Toolkit.getDefaultToolkit().beep();
			resetVertexFocus();
			return;
			}
		    }
		    */
		    // check for forward and backward edges
		    //
		    if (vertexParent.containsChild(vertexChild)) {
			forwardLink = true;
		    }
		    if (vertexChild.containsChild(vertexParent)) {
			reverseLink = true;
		    }
		    
		    // if neither a forward nor backward link exist add
		    // the connector
		    //
		    if (!forwardLink && !reverseLink) {
			
			// add the parent from the child lis and vice-versa
			//
			vertexParent.addChild(vertexChild);
			vertexChild.addParent(vertexParent);
		    }
		    
		    // reset the vertex parent and vertex child pointers
		    //
		    resetVertexFocus();
		}

		// repaint the work area
		//
		workArea.repaint();
	    }
        }
    }
    
    // method: mouseReleased
    //
    // arguments:
    //    MouseEvent e: (input) event that was fired
    // return: none
    //
    public void mouseReleased(MouseEvent e) {
	// not yet implemented
    }
    
    // method: mouseMoved
    //
    // arguments:
    //    MouseEvent e: (input) event that was fired
    // return         : none
    //
    public void mouseMoved(MouseEvent e) {
	
	// update the location of the mouse coordinates
	//
	mouseXloc = e.getX();
	mouseYloc = e.getY();
    }
    
    // method: mouseExited
    //
    // arguments:
    //    MouseEvent e: (input) event that was fired
    // return   : none
    //
    public void mouseExited(MouseEvent e) {
	// this method is required to be present
    }
    
    // method: mouseEntered
    //
    // arguments:
    //    MouseEvent e: (input) event that was fired
    // return   : none
    //
    public void mouseEntered(MouseEvent e) {
	// this method is required to be present
    }

    // method: actionPerformed
    //
    // arguments:
    //    ActionEvent e: event that was fired
    //
    // return   : none
    //
    // listens for actions taking place on the components
    //
    public void actionPerformed(ActionEvent e) {

	// detremine if the delete option was selected
	//
	if (e.getActionCommand().equals(delete)) {
	    removeVertex(cfgVertexFocus);
	}

	// determine if the configuration option was selected
	//
	else if (e.getActionCommand().equals(configuration)) {

	    // create a new configuration box and display in a new frame
	    //
	    if (cfgVertexFocus != null) {
	
		// create the configuration window for the CONTAINER
		//
		if (cfgVertexFocus.getVertexType() == CONTAINER) {
		    ConfigBoxWord config_box = new ConfigBoxWord(this,data, 
							 cfgVertexFocus, 
							 workArea, 0);
		    config_box.pack();
		    config_box.setVisible(true);
		}

		// create the configuration window for the start/stop data
		//
		else {

		    // create a frame to display the properties
		    //
		    if (config != null) {
			config.setVisible(false);
		    }
		    config = new JFrame("Label");
		    nameField = new JTextField(10);
		    
		    // add a key listener to the textfield
		    //
		    nameField.addKeyListener(new KeyAdapter() { 
			    public void keyPressed(KeyEvent e) { 
				
				// determine if the enter key was pressed
				//
				if (e.getKeyCode() == e.VK_ENTER) {
				    
				    // check to see if a vertex with
				    // this label exists
				    //
				    if (coeffLabels.contains(nameField.getText())) {
					// display a warning message
					// indicating that you cannot
					// have two vertices with the
					// same name
					//
					JFrame frame = new JFrame("Warning");
					JOptionPane.showMessageDialog(frame, 
					       "A node with the label " + 
					            nameField.getText() + 
						       " already exists");
					return;
				    }
				    if (cfgVertexFocus.getVertexImpl() == NONE) {
					if (dataLabels.contains(nameField.getText())) {
					    // display a warning message
					    // indicating that you cannot
					    // have two vertices with the
					    // same name
					    //
					    JFrame frame = new JFrame("Warning");
					    JOptionPane.showMessageDialog(frame, 
						    "A node with the label " + 
						         nameField.getText() + 
							    " already exists");
					    return;
					}
				    }

				    // remove the old and add the new label
				    //
				    if (cfgVertexFocus.getVertexImpl() == NONE) {
					coeffLabels.remove(cfgVertexFocus.getVertexName());
					coeffLabels.addElement(nameField.getText());
				    }
				    else {
					dataLabels.remove(cfgVertexFocus.getVertexName());
					dataLabels.addElement(nameField.getText());
				    }

				    // update the vertex text with
				    // what the user entered
				    //
				    if (nameField.getText().length() > 3) {
					cfgVertexFocus.setText(nameField.getText().substring(0, 3));
				    }
				    else {
					cfgVertexFocus.setText(nameField.getText());
				    }
				    cfgVertexFocus.setVertexName(nameField.getText());
				    
				    // if the block is of type STOP then
				    // update the system target
				    //
				    if (cfgVertexFocus.getVertexImpl() == STOP) {
					targetStr = "" + cfgVertexFocus.getVertexName();
				    }
				    
				    // remove the frame from the screen
				    //
				    config.setVisible(false);
				}
			    }
			});
		    
		    // create labels to be included in the frame
		    //
		    nameLabel = new JLabel("Label: ");
		    nameLabel.setFont(newFont);
		    nameLabel.setForeground(Color.black);
		    
		    // create buttons for the frame
		    //
		    update = new String("Update");
		    JButton done = new JButton(update);
		    done.addActionListener(this);
		    
		    cancel = new String("Cancel");
		    JButton ignore = new JButton(cancel);
		    ignore.addActionListener(this);

		    // retrieve the text from the vertex and set the textfield
		    //
		    String text = cfgVertexFocus.getVertexName();
		    nameField.setText(text);
		    
		    // declare the default layout for the frame
		    //
		    config.getContentPane().setLayout(new GridLayout(2,2));

		    // add the textfields, labels and buttons to the frame
		    //
		    config.getContentPane().add(nameLabel);
		    config.getContentPane().add(nameField);
		    config.getContentPane().add(done);
		    config.getContentPane().add(ignore);
		    
		    // display the frame
		    //
		    config.pack();
		    config.setVisible(true);
		}
	    }	    
	}

	// detremine if the update was selected
	//
	else if (e.getActionCommand().equals(update)) {

	    // check to see if a vertex with this label exists
	    //
	    if (coeffLabels.contains(nameField.getText())) {

		// display a warning message indicating that you cannot have
		// two vertices with the same name
		//
		JFrame frame = new JFrame("Warning");
		JOptionPane.showMessageDialog(frame, "#A node with the label " 
					      + nameField.getText() + 
					      " already exists");
		return;
	    }
	    if (cfgVertexFocus.getVertexImpl() == NONE) {
		if (dataLabels.contains(nameField.getText())) {
		    
		    // display a warning message
		    // indicating that you cannot
		    // have two vertices with the
		    // same name
		    //
		    JFrame frame = new JFrame("Warning");
		    JOptionPane.showMessageDialog(frame, 
						  "A node with the label " + 
						  nameField.getText() + 
						  " already exists");
		    return;
		}
	    }
	    
	    // remove the old and add the new label
	    //
	    if (cfgVertexFocus.getVertexImpl() == NONE) {
		coeffLabels.remove(cfgVertexFocus.getVertexName());
		coeffLabels.addElement(nameField.getText());
	    }
	    else {
		dataLabels.remove(cfgVertexFocus.getVertexName());
		dataLabels.addElement(nameField.getText());
	    }

	    // update the vertex text with what the user entered
	    //
	    if (nameField.getText().length() > 3) {
		cfgVertexFocus.setText(nameField.getText().substring(0, 3));
	    }
	    else {
		cfgVertexFocus.setText(nameField.getText());
	    }
	    cfgVertexFocus.setVertexName(nameField.getText());

	    // if the block is of type STOP then
	    // update the system target
	    //
	    if (cfgVertexFocus.getVertexImpl() == STOP) {
		targetStr = "" + cfgVertexFocus.getVertexName();
	    }
	    
	    // remove the frame from the screen
	    //
	    config.setVisible(false);
	}

	// detremine if the cancel was selected
	//
	else if (e.getActionCommand().equals(cancel)) {

	    // remove the frame from the screen
	    //
	    config.setVisible(false);
	}

	// repaint the work area
	//
	workArea.repaint();
    }

    // method: convertJSGF
    //
    // arguments: none
    // return   : none
    //
    // writes the sof file
    //
    public boolean convertJSGF() {
	
       String s = null;

        try {
	    String tmpOutputPath = JSGFFile.getAbsolutePath();
	    String tmpInputPath = graphFile.getAbsolutePath();
	    String[] command = {"","-input_mode", "VJSGF", "-input_file", "", "-output_mode", "JSGF","-output_file", ""};
	    command[0] = GUI_PATH + "/util/speech/isip_lm_converter/isip_lm_converter.exe";
	    command[4] = tmpInputPath;
	    command[8] = tmpOutputPath;

            // run the Unix command
            //
            Process p = Runtime.getRuntime().exec(command);
	    p.waitFor();
	    System.out.println("return code: " + p.exitValue());
	    graphFile.delete();
	    graphFile = null;
	    return true;  
        }
        catch (IOException e) {
            System.out.println("exception happened - here's what I know: ");
            e.printStackTrace();
            System.exit(-1);
        }
	
	catch(InterruptedException e1) {
	    System.err.println("Exception:" + e1.getMessage());
	}
	return false;
    }

    // method: convertGraph
    //
    // arguments: none
    // return   : none
    //
    // writes the sof file
    //
    public boolean convertGraph() {
	
       String s = null;

        try {            
	    String tmpInputPath = JSGFFile.getAbsolutePath();
	    String tmpOutputPath = "tmp_lm_002.sof";
	    String[] command = {"","-input_mode", "JSGF", "-input_file", "", "-output_mode", "VJSGF","-output_file", ""};
	  command[0] = GUI_PATH + "/util/speech/isip_lm_converter/isip_lm_converter.exe";
	    command[4] = tmpInputPath;
	    command[8] = tmpOutputPath;

            // run the Unix command
            //
            Process p = Runtime.getRuntime().exec(command);
	    p.waitFor();
	    System.out.println("return code: " + p.exitValue());
	    graphFile = new File(tmpOutputPath);
	    return true;
        }
        catch (IOException e) {
            System.out.println("exception happened - here's what I know: ");
            e.printStackTrace();
            System.exit(-1);
        }
	
	catch(InterruptedException e1) {
	    System.err.println("Exception:" + e1.getMessage());
	}
	return false;
    }

    // method: lmTest
    //
    // arguments: none
    // return   : none
    //
    // test the JSGF file
    //
    public boolean lmTest() {
	
	JFrame frame = new sentencesGen(this, workArea);
	
        frame.pack();
        frame.setVisible(true);

	return true;	
    }


    // method: fullNode
    //
    // arguments: none
    // return   : none
    //
    // test if each node has word list
    //
    public boolean fullNode() {

	// check if each node has word list 
	//
	for (int i=0; i < vertices.size(); i++) {

	    // get the current vertex
	    //
	    Vertex vertex = (Vertex)vertices.elementAt(i);
	    
	    // write the data if the vertex is a data
	    //
	    if(vertex.getVertexType() != START && vertex.getVertexType() != STOP) {
		if(vertex.values.isEmpty()){
		    return false;
		}
	    }
	}

	return true;
    }

}

//
// end of file






















