// file: $isip/util/asr/isip_lm_converter.cc
// version: $Id: isip_network_converter.cc,v 1.9 2003/05/12 22:28:49 huang Exp $
//

// isip include files
//
#include <LanguageModel.h>
#include "isip_network_converter.h"

// isip_net_converter: language model converter utility
//
// this is the language model converter utility. it can be used for
// convert the output of network builder to the model file that can
// be used for the isip_recognizer.
//
int main(int argc, const char** argv) {
 
 // set the sdb
 //
 Sdb sdb;
 CommandLine cmdl(sdb);
 
 cmdl.setUsage(
#include "usage_message.text"
                );
 cmdl.setHelp(
#include "help_message.text"
               );
 
 cmdl.setIdent("$Revision: 1.9 $",
	       "$Name: isip_r00_n11 $",
	       "$Date: 2003/05/12 22:28:49 $");

 // add a command line option for the output mode
 //
 String input_format(L"NATIVE");
 cmdl.addOptionParam(input_format, OPTION_INPUT_FORMAT, L"NATIVE");

 // add a command line option for the input mode
 //
 String output_format(L"JSGF");
 cmdl.addOptionParam(output_format, OPTION_OUTPUT_FORMAT, L"JSGF");

 // add a command line option for output type
 //
 String output_type(L"TEXT");
 cmdl.addOptionParam(output_type, OPTION_OUTPUT_TYPE, L"TEXT");

 // add a command line option for the debug level
 //
 DebugLevel debug_level;
 cmdl.addOptionParam(debug_level, OPTION_DEBUG_LEVEL, DEBUG_LEVEL_DEFAULT);

 // parse the command line
 //
 if (!cmdl.parse(argc, argv)) {
   cmdl.printUsage();
 };

 // input file and output file
 //
 Filename input_lm_file;
 Filename output_lm_file;
 Filename output_sm_file;

 // get the input file
 //
 if ( !sdb.gotoFirst() ){
   cmdl.printUsage();
 }
 sdb.getName(input_lm_file);

 // get the output language model file
 //
 if ( !sdb.gotoNext() ){
   cmdl.printUsage();
 }
 sdb.getName(output_lm_file);
 
 // get the output statistical model file
 //
 if ( !sdb.gotoNext() ){
   cmdl.printUsage();
 }
 sdb.getName(output_sm_file);
 
 // print the command line
 //
 cmdl.printCommandLine();
 cmdl.printVersion();

 // check the input file arguments
 //
 if(input_lm_file.eq(NULL)) {
   String msg(L"Error: no input file specified ");
   Console::put(msg);
   Error::handle(PROG_NAME, L"main", Error::NO_PARAM_FILE,
		 __FILE__, __LINE__);
 };

 if(output_lm_file.eq(NULL)) {
   String msg(L"Error: no output file specified ");
   Console::put(msg);
   Error::handle(PROG_NAME, L"main", Error::NO_PARAM_FILE,
		 __FILE__, __LINE__);
 };

 // set the options to upper case
 //
 input_format.toUpper();
 output_format.toUpper();
 output_type.toUpper();

 //Console::put(input_format);
 //Console::put(output_format);
 //Console::put(output_type);
 
 if ( input_format.eq(output_format)){
   String msg(L"input and output formats are the same ");
   Console::put(msg);
   Error::handle(PROG_NAME, L"main", Error::ARG,
		 __FILE__, __LINE__);
 }

 // set the output type
 //
 LanguageModel::OUTPUT_TYPE out_type = LanguageModel::TEXT;
 if(output_type.eq(L"TEXT") || output_type.eq(L"text")) {
   out_type = LanguageModel::TEXT;
 }
 else if(output_type.eq(L"BINARY") || output_type.eq(L"binary")) {
   out_type = LanguageModel::BINARY;
 }
 else {
   cmdl.printUsage();
 }

 // input is the native java network builder format
 //
 if ( input_format.eq(L"NATIVE") && output_format.eq(L"DIGRAPH")){
   nativeToModel(input_lm_file, output_lm_file, output_sm_file,
		 LanguageModel::NATIVE, out_type);
 }
 else if ( input_format.eq(L"NATIVE") && output_format.eq(L"JSGF")){
   nativeToModel(input_lm_file, output_lm_file, output_sm_file,
		 LanguageModel::JSGF, out_type);
 }
 else if ( input_format.eq(L"DIGRAPH") || input_format.eq(L"JSGF") ){
   if ( output_format.eq(L"NATIVE") ){
     modelToNative(input_lm_file, output_lm_file);
   }
   else if (output_format.eq(L"JSGF")) {
     modelToModel(input_lm_file, output_lm_file,
		  LanguageModel::JSGF, out_type);
   }
   else if (output_format.eq(L"DIGRAPH")) {
     modelToModel(input_lm_file, output_lm_file,
		  LanguageModel::NATIVE, out_type);
   }
   else {
     cmdl.printUsage();
   }
 }
 else{
   cmdl.printUsage();
 }
 
 // exit gracefully
 //
 return Integral::exit();

}

// method: modelToModel
//
// arguments:
//  FileName&  input_file_name_a: (input) file name
//  FileName& output_file_name_a: (output) file name
//
// return: boolean value indicating status
//
// this method reads the data into a hash table
//
boolean modelToModel(Filename& input_file_name_a,
		     Filename& output_file_name_a,
		     LanguageModel::OUTPUT_FORMAT out_format_a,
		     LanguageModel::OUTPUT_TYPE out_type_a){

  // the search levels, sub-graphs are maintained at each SearchLevel 
  //
  Vector<SearchLevel> search_levels(1);
  
  // use the language model class to output different types
  //
  LanguageModel lm_model_01;
  lm_model_01.load(input_file_name_a, search_levels);
 
  // use the language model class to output different types
  //
  LanguageModel lm_model_02;
  lm_model_02.store(output_file_name_a, search_levels,
		    out_format_a, out_type_a);

  // exit gracefully
  //
  return true;
  
}

// method: modelToNative
//
// arguments:
//  FileName&  input_file_name_a: (input) file name
//  FileName& output_file_name_a: (output) file name
//
// return: boolean value indicating status
//
// this method reads the data into a hash table
//
boolean modelToNative(Filename& input_file_name_a,
		      Filename& output_file_name_a){

  // the search levels, sub-graphs are maintained at each SearchLevel 
  //
  Vector<SearchLevel> search_levels(1);
  
  // use the language model class to output different types
  //
  LanguageModel lm_model;
  lm_model.load(input_file_name_a, search_levels);
 
  // store the number of levels
  //
  HashTable<String, String>  hash_data;
  String hash_key;
  long num_of_levels = search_levels.length();
  String num_levels;
  num_levels.assign(num_of_levels);
  hash_key.assign(IO_CDATA_LEVEL_SIZE);
  hash_data.insert(hash_key, &num_levels);
 
  // read each level
  //
  for (long i = 0 ; i < num_of_levels; i++){
    
    // set the prefix
    //
    String curr_level;
    curr_level.assign((long)i);
   
    String level_prefix;
    level_prefix.assign(IO_SEARCHLEVEL_PREFIX);
    level_prefix.concat(curr_level);

    // read one level
    //
    Vector<SearchSymbol> graph_name;
    if ( i == 0 ){
      graph_name.concat(LEVEL_TREE_TOP_GRAMMAR);
    }
    else{
      if ( search_levels(i-1).getContextMap().length() > 0 ){
	convertContextMapping(search_levels(i-1).getContextMap(), graph_name);
      }
      else {
	graph_name = search_levels(i-1).getSymbolTable();
      }
    }
    storeSearchLevel(search_levels(i),  hash_data, graph_name, level_prefix);
   
  }

  // write to the file
  //
  File output_file;
  if (!output_file.open(output_file_name_a, File::WRITE_ONLY)) {
    return Error::handle(output_file_name_a,
			 L"open error", Error::TEST, __FILE__, __LINE__);
  }

  // set the string
  //
  String output;
  //writeHashTable(hash_data, output, L"test");
  writeHashTable(hash_data, output_file, L"test");
  //Console::put(output);
  //output_file.put(output);
  output_file.close();
 
  // exit gracefully
  //
  return true;
  
}

// method: nativeToModel
//
// arguments:
//  FileName&  input_file_name_a: (input) file name
//  FileName& output_file_name_a: (output) file name
//
// return: boolean value indicating status
//
// this method reads the data into a hash table
//
boolean nativeToModel(Filename& input_file_name_a,
		      Filename& output_file_name_a,
		      Filename& output_sm_file_a,
		      LanguageModel::OUTPUT_FORMAT out_format_a,
		      LanguageModel::OUTPUT_TYPE out_type_a){

  // read the file line by line and parse the input data
  // the data will be stored in a HashTable
  //
  HashTable<String, String>  hash_data;
  readHashTable(input_file_name_a,  hash_data);
  
  // read the number of levels
  //
  long num_of_levels = 0;
  hash_data.get(IO_CDATA_LEVEL_SIZE)->get(num_of_levels);
  
  Long level_num(num_of_levels);
  //level_num.debug(L"level_num");

  // the search levels, sub-graphs are maintained at each SearchLevel 
  //
  Vector<SearchLevel> search_levels(num_of_levels);
  
  // read each level
  //
  for (long i = 0 ; i < num_of_levels; i++){
    
    // set the prefix
    //
    String curr_level;
    curr_level.assign((long)i);
   
    String level_prefix;
    level_prefix.assign(IO_SEARCHLEVEL_PREFIX);
    level_prefix.concat(curr_level);

    // read one level
    //
    readSearchLevel(search_levels(i),  hash_data, level_prefix);
   
  }
 
  // use the language model class to output different types
  //
  LanguageModel lm_model;
  lm_model.store(output_file_name_a, search_levels, out_format_a, out_type_a);

  // write the statistical model file
  //
  if(!output_sm_file_a.eq(NULL)) {

    // print messages
    //
    String msg(L"store the statistical file");
    Console::put(msg);
    // store the sm symbols
    //
    Sof sof_sm;
    sof_sm.open(output_sm_file_a, File::WRITE_ONLY);
      

    // read each level
    //
    for (long i = 0 ; i < num_of_levels; i++){
    
      // set the prefix
      //
      String curr_level;
      curr_level.assign((long)i);
   
      String level_prefix;
      level_prefix.assign(IO_SEARCHLEVEL_PREFIX);
      level_prefix.concat(curr_level);

      // write the symbols to the sm file
      //
      Vector<SearchSymbol> sm_symbols;
      HashTable<SearchSymbol, Long>  sm_table;
      
      // read one level
      //
      readSMSymbols(sm_symbols,  hash_data, level_prefix);

      for ( int j = 0; j < sm_symbols.length(); j++ ){

	// no multiple copies
	//
	if ( sm_table.get(sm_symbols(j)) == NULL){
	  
	  Long index(j);
	  // put the data into the hash table
	  //
	  sm_table.insert(sm_symbols(j), &index);       
	}
      }

      // debug message
      //
      //sm_table.debug(L"sm_table");

      // write the hashtable from the input sof file
      //
      if (!sm_table.isEmpty()) {
	if(!sm_table.write(sof_sm, i,
			   SearchLevel::PARAM_STAT_HASH)) {
	  return Error::handle(L"isip_network_converter", L"store symbols with statistical models", Error::ARG, __FILE__, __LINE__);
	}
      }
    }

    // close the sm file
    //
    sof_sm.close();
    
  }

  // exit gracefully
  //
  return true;
  
}

// method: readHashTable
//
// arguments:
//  FileName&: (input) file name
//  HashTable&: (output) the data will be put into the hash table
//
// return: boolean value indicating status
//
// this method reads the data into a hash table
//
boolean readHashTable(Filename& input_file_name,
		      HashTable<String, String>& hash_table_a){

  // set local variable
  //
  File input_file;
  String line;

  input_file.open(input_file_name, File::READ_ONLY);
  
  while ( input_file.get(line) ){
    
    // parse the line
    //
    line.trim();
    
    if ( line.firstChr(COMMENT_CHAR) == 0 ){
     continue;
    }
    else{
      
      // find the parameter name and the value
      //
     long count = line.countTokens(DELIMITER_CHAR);
     
     if ( count != 2 ){
       return Error::handle(L"read", L"parsing error in model file",
			    Error::ARG, __FILE__, __LINE__);  
     }
     
     long pos = 0;
     Vector<String> values(count);
     
     // get each word by tokenizing using multiple spaces as a delimiter
     //
     for (long i = 0; i < count; i++) {
       line.tokenize(values(i), pos, DELIMITER_CHAR);
     }
     
     // put the data into the hash table
     //
     hash_table_a.insert(values(0), &values(1));       

     // debug information
     //
     if (debug_level >= Integral::BRIEF) {
       //if ( true ) {
       Console::put(line);
     }
     
    }
  }

  // close the input file
  //
  input_file.close();

  // exit gracefully
  //
  return true;
  
}

// method: readSMSymbols
//
// arguments:
//  SearchLevel&: (output) construct the search level
//  HashTable<String, String>&: (intput) the hash table which stored data
//  String& prefix_a: (output) the pefix
//
// return: boolean value indicating status
//
// this method reads symbols with statistical model
//
boolean readSMSymbols(Vector<SearchSymbol>& sm_symbols_a,
		      HashTable<String, String>& hash_table_a,
		      String& prefix_a) {
  // local variable
  //
  String hash_key;
  String symbol_prefix;

  // find the number of symbols in SM symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_SM_SYMBOL_SIZE);
  symbol_prefix.assign(prefix_a);
  symbol_prefix.concat(IO_LEVEL_SM_SYMBOL);
  readSymbols(sm_symbols_a,
	      hash_table_a, hash_key, symbol_prefix);

  // exit gracefully
  //
  return true;
  
}

// method: readSearchLevel
//
// arguments:
//  SearchLevel&: (output) construct the search level
//  HashTable<String, String>&: (intput) the hash table which stored data
//  String& prefix_a: (output) the pefix
//
// return: boolean value indicating status
//
// this method reads the data into a SearchLevel
//
boolean readSearchLevel(SearchLevel& search_level_a,
			HashTable<String, String>& hash_table_a,
			String& prefix_a) {
  // local variable
  //
  String hash_key;

  // set the name for this level
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_NAME);
  String level_tag;
  level_tag.assign(*hash_table_a.get(hash_key));

  // debug information
  //
  if (debug_level >= Integral::BRIEF) {
    level_tag.debug(L"level_tag");
  }
	
  search_level_a.setLevelTag(level_tag);
  
  // set symbol table and all other symbols
  // including : non_speech, exclude_symbols,
  //

  // find the number of symbols in the symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_SYMBOL_SIZE);
  String symbol_prefix;
  symbol_prefix.assign(prefix_a);
  symbol_prefix.concat(IO_LEVEL_SYMBOL);
  readSymbols(search_level_a.getSymbolTable(),
	      hash_table_a, hash_key, symbol_prefix);

  // find the number of symbols in the context symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_CONTEXT_SIZE);
  symbol_prefix.assign(prefix_a);
  symbol_prefix.concat(IO_LEVEL_CONTEXT);
  Vector<SearchSymbol> context_symbols;
  readSymbols(context_symbols, 
	      hash_table_a, hash_key, symbol_prefix);
  convertContextMapping(context_symbols,
			search_level_a.getContextMap());

  // find the number of symbols in the non-speech symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_NON_SPEECH_SYMBOL_SIZE);
  symbol_prefix.assign(prefix_a);
  symbol_prefix.concat(IO_LEVEL_NON_SPEECH_SYMBOL);
  readSymbols(search_level_a.getNonSpeechSymbolTable(),
	      hash_table_a, hash_key, symbol_prefix);

  // find the number of symbols in the context-less symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_CONTEXTLESS_SYMBOL_SIZE);
  symbol_prefix.assign(prefix_a);
  symbol_prefix.concat(IO_LEVEL_CONTEXTLESS_SYMBOL);
  readSymbols(search_level_a.getContextLessSymbolTable(),
	      hash_table_a, hash_key, symbol_prefix);
  
  // find the number of symbols in the dummy symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_DUMMY_SYMBOL_SIZE);
  symbol_prefix.assign(prefix_a);
  symbol_prefix.concat(IO_LEVEL_DUMMY_SYMBOL);
  readSymbols(search_level_a.getDummySymbolTable(),
	      hash_table_a, hash_key, symbol_prefix);
  
  // find the number of symbols in the skip symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_SKIP_SYMBOL_SIZE);
  symbol_prefix.assign(prefix_a);
  symbol_prefix.concat(IO_LEVEL_SKIP_SYMBOL);
  readSymbols(search_level_a.getSkipSymbolTable(),
	      hash_table_a, hash_key, symbol_prefix);
  
  // find the number of symbols in the exclude symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_EXCLUDE_SYMBOL_SIZE);
  symbol_prefix.assign(prefix_a);
  symbol_prefix.concat(IO_LEVEL_EXCLUDE_SYMBOL);
  readSymbols(search_level_a.getExcludeSymbolTable(),
	      hash_table_a, hash_key, symbol_prefix);
  
  // find the number of graphs
  //
  long num_of_graphs = 0;

  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_GRAPH_SIZE);
  
  hash_table_a.get(hash_key)->get(num_of_graphs);
  Long graph_num(num_of_graphs);
  //graph_num.debug(L"graph_num");
  
  // set the number of graphs
  //
  if ( num_of_graphs > 0 ){
    search_level_a.setNumSubGraphs(num_of_graphs);
  }
  
  // read graphs for a paticular level
  //
  for (long j = 0; j < num_of_graphs; j++){

   // set the prefix
   //
   String curr_graph;
   curr_graph.assign((long)j);
   
   String graph_prefix;
   graph_prefix.assign(prefix_a);
   graph_prefix.concat(IO_DIGRAPH_PREFIX);
   graph_prefix.concat(curr_graph);

   // read the graph
   //
   readSubGraph(search_level_a, search_level_a.getSubGraph(j),
		hash_table_a, graph_prefix);
  }
  
  // exit gracefully
  //
  return true;
  
}

// method: readSubGraph
//
// arguments:
//  DiGraph<SearchNode>&: (output) construct this DiGraph
//  HashTable<String, String>&: (intput) the hash table which holds the data
//  String& prefix_a: (input) the prefix
//
// return: boolean value indicating status
//
// this method reads the data into a DiGraph<SearchNode>
//
boolean readSubGraph(SearchLevel& search_level_a,
		     DiGraph<SearchNode>& sub_graph_a,
		     HashTable<String, String>& hash_table_a,
		     String& prefix_a) {
  // local variable
  //
  String hash_key;
  SingleLinkedList<SingleLinkedList<GraphVertex<SearchNode> > >
    vertices_list(DstrBase::SYSTEM);

  // debug infor
  //
  //prefix_a.debug(L"subgraph: ");

  // get the number of vertices
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_DIGRAPH_VERTEX_SIZE);
  long num_of_vertices = 0;
  hash_table_a.get(hash_key)->get(num_of_vertices);

  // get the number of arcs
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_DIGRAPH_ARC_SIZE);
  long num_of_arcs = 0;
  hash_table_a.get(hash_key)->get(num_of_arcs);

  // debug information
  //
  Long arcs_num(num_of_arcs);
  //arcs_num.debug(L"arcs_num");

  // read graphs for a paticular level
  //
  for (long i = 0; i < num_of_vertices; i++){

   // set the prefix
   //
   String curr_vertex;
   curr_vertex.assign((long)i);
   
   String vertex_prefix;
   vertex_prefix.assign(prefix_a);
   vertex_prefix.concat(IO_VERTEX_PREFIX);
   vertex_prefix.concat(curr_vertex);

   // create a new list to hold a list of vertices
   //
   SingleLinkedList<GraphVertex<SearchNode> >* vertices_on_node =
     new SingleLinkedList<GraphVertex<SearchNode> >(DstrBase::USER);
   
   // read the vertex
   //
   readVertices(search_level_a, sub_graph_a,
		*vertices_on_node, hash_table_a, vertex_prefix);
   vertices_list.insert(vertices_on_node);
   
  }

  // read graphs for a paticular level
  //
  for (long j = 0; j < num_of_arcs; j++){

   // set the prefix
   //
   String curr_arc;
   curr_arc.assign((long)j);
   
   String arc_prefix;
   arc_prefix.assign(prefix_a);
   arc_prefix.concat(IO_ARC_PREFIX);
   arc_prefix.concat(curr_arc);

   // read the arcs
   //
   readArcs(sub_graph_a, vertices_list, hash_table_a, arc_prefix);
  }

  // exit gracefully
  //
  return true;
  
}

// method: readVertices
//
// arguments:
//  DiGraph<SearchNode>&: (output) construct this DiGraph
//  SingleLinkedList<GraphVertex<SearchNode>>& vertices_list_a:
//                  (input) hold the pointers of Vertices
//  HashTable<String, String>&: (intput) the hash table which holds the data
//  String& prefix_a: (input) the prefix
//
// return: boolean value indicating status
//
// this method reads the data into a DiGraph and store pointers of Vertices
//
boolean readVertices(SearchLevel& search_level_a,
		     DiGraph<SearchNode>& sub_graph_a,
		     SingleLinkedList<GraphVertex<SearchNode> > &
		     vertices_list_a,
		     HashTable<String, String>& hash_table_a,
		     String& prefix_a) {
  // local variable
  //
  String hash_key;
  Vector<SearchSymbol> symbol_list;
  GraphVertex<SearchNode>* curr_vert = NULL;

  // debug infor
  //
  //prefix_a.debug(L"vertex: ");

  // read the type of the node
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_VERTEX_TYPE);
  long vertex_type = 0;
  hash_table_a.get(hash_key)->get(vertex_type);

  // branch on the vertex type
  //
  switch ( vertex_type ){

  case START_VERTEX:
    curr_vert = sub_graph_a.getStart();
    vertices_list_a.insert(curr_vert);  
    return true;
    
  case TERM_VERTEX:
    curr_vert = sub_graph_a.getTerm();
    vertices_list_a.insert(curr_vert);  
    return true;

  case NODE_VERTEX:
    break;

  default:
    return Error::handle(L"readVertices", L"vertex type",
			 Error::ARG, __FILE__, __LINE__);
    
  }
  
  // find the number of symbols in the symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_VERTEX_SYMBOL_SIZE);
  String symbol_prefix;
  symbol_prefix.assign(prefix_a);
  symbol_prefix.concat(IO_VERTEX_SYMBOL);
  readSymbols(symbol_list, hash_table_a, hash_key, symbol_prefix);

  // set the search nodes for the graph
  //
  for ( int i = 0; i < symbol_list.length(); i++ ){

    // create a new search node
    //
    SearchNode* curr_node = new SearchNode();
    curr_node->setSearchLevel(&search_level_a);

    long symbol_index = search_level_a.getSymbolIndex(symbol_list(i));

    // check if this index is in the range of symbols
    //
    if (symbol_index < 0 ) {
      Long index(symbol_index);
      index.debug(L"symbol_index");
      symbol_list(i).debug(L"symbol_list(i)");
      symbol_list.debug(L"symbol_list");
      return Error::handle(L"readVertices", L"invalid symbol index",
			   Error::ARG, __FILE__, __LINE__);
    }
    curr_node->setSymbolId(symbol_index);
    curr_vert = sub_graph_a.insertVertex(curr_node);
    vertices_list_a.insert(curr_vert);  
  }

  // debug infor
  //
  //prefix_a.debug(L"vertex: ");

  // exit gracefully
  //
  return true;
  
}

// method: readArcs
//
// arguments:
//  DiGraph<SearchNode>&: (output) construct this DiGraph
//  SingleLinkedList<GraphVertex<SearchNode>>& vertices_list_a:
//                  (output) hold the pointers to vertices
//  HashTable<String, String>&: (intput) the hash table which holds the data
//  String& prefix_a: (input) the prefix
//
// return: boolean value indicating status
//
// this method reads the data into a DiGraph and store pointers of vertices
//
boolean readArcs(DiGraph<SearchNode>& sub_graph_a,
		 SingleLinkedList<SingleLinkedList<GraphVertex<SearchNode> > >&
		 vertices_list_a,
		 HashTable<String, String>& hash_table_a,
		 String& prefix_a) {
    
  // local variable
  //
  String hash_key;

  // debug infor
  //
  //prefix_a.debug(L"arc: ");

  // find the from_id
  //
  long from_id;
  hash_key.assign(prefix_a);
  hash_key.concat(IO_ARC_FROM_ID);
  hash_table_a.get(hash_key)->get(from_id);

  // find the from_id
  //
  long to_id;
  hash_key.assign(prefix_a);
  hash_key.concat(IO_ARC_TO_ID);
  hash_table_a.get(hash_key)->get(to_id);

  // get the epsilon flag
  //
  boolean is_epsilon;
  hash_key.assign(prefix_a);
  hash_key.concat(IO_ARC_EPSILON);
  hash_table_a.get(hash_key)->get(is_epsilon);

  // get the transition
  //
  float weight;
  hash_key.assign(prefix_a);
  hash_key.concat(IO_ARC_WEIGHTS);
  hash_table_a.get(hash_key)->get(weight);  

  // get number of vertex in the from node
  //
  long num_of_from_vertices = vertices_list_a(from_id).length();

  // get number of vertex in the to node
  //
  long num_of_to_vertices = vertices_list_a(to_id).length();

  // set the weights
  //
  if ( ! is_epsilon ){
    //weight = weight / (num_of_from_vertices * num_of_to_vertices ) ;
    weight = weight / num_of_to_vertices ;
  }

  // insert the arc into graphs
  //
  for ( int i= 0; i < num_of_from_vertices; i++ ){
    for ( int j = 0 ; j < num_of_to_vertices; j++ ){

      // get the from vertex and dest vertex
      //
      GraphVertex<SearchNode>* from_vertex =
	&vertices_list_a(from_id)(i);

      GraphVertex<SearchNode>* to_vertex =
	&vertices_list_a(to_id)(j);
      
      // connect the vertices in the searchnode graph
      //
      sub_graph_a.insertArc(from_vertex, to_vertex,
			    is_epsilon,
			    weight);
    }
  }
  
  // debug infor
  //
  //prefix_a.debug(L"arc: ");

  // exit gracefully
  //
  return true;
  
}


// method: readSymbols
//
// arguments:
//  Vector<SearchSymbol>& Symbols_list_a:
//                  (input) hold the pointers of symbols
//  long : (input) total number of symbols to read
//  HashTable<String, String>&: (intput) the hash table which holds the data
//  String& prefix_a: (input) the prefix
//
// return: boolean value indicating status
//
// this method reads the data into a DiGraph and store pointers of Symbols
//
boolean readSymbols(Vector<SearchSymbol>& symbol_list_a,
		    HashTable<String, String>& hash_table_a,
		    String& size_prefix_a,
		    String& prefix_a) {
  // local variable
  //
  long num_of_symbols = 0;
  hash_table_a.get(size_prefix_a)->get(num_of_symbols);

  // read each symbol
  //
  for (long j = 0; j < num_of_symbols; j++){

   // set the hash_key
   //
   String curr_symbol;
   curr_symbol.assign((long)j);
   
   String hash_key;
   hash_key.assign(prefix_a);
   hash_key.concat(curr_symbol);
   curr_symbol.assign(*hash_table_a.get(hash_key));
   String new_symbol;

   for ( long k = 0; k < curr_symbol.length(); k ++ ){
     if ( curr_symbol(k) != '\\' ){
       new_symbol.concat(curr_symbol(k));
     }
     else {
       k++;
       if ( k < curr_symbol.length() ){
	 new_symbol.concat(curr_symbol(k));
       }
     }
   }

   //curr_symbol.trimLeft('\\');
   
   // debug information
   //
   if (debug_level >= Integral::BRIEF) {
     Console::put(new_symbol);
   }
  
   // put the symbol into list
   //
   symbol_list_a.concat(new_symbol);
   //symbol_list_a.concat(curr_symbol);

  }

  // exit gracefully
  //
  return true;
  
}

// method: storeSearchLevel
//
// arguments:
//  SearchLevel&: (output) construct the search level
//  HashTable<String, String>&: (intput) the hash table which stored data
//  String& prefix_a: (output) the pefix
//
// return: boolean value indicating status
//
// this method reads the data into a SearchLevel
//
boolean storeSearchLevel(SearchLevel& search_level_a,
			HashTable<String, String>& hash_table_a,
			 Vector<SearchSymbol>& graph_name_a,
			String& prefix_a) {
  // local variable
  //
  String hash_key;

  // set the name for this level
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_NAME);
  String level_tag;
  level_tag.assign(search_level_a.getLevelTag());
  hash_table_a.insert(hash_key, &level_tag);

  // debug information
  //
  if (debug_level >= Integral::BRIEF) {
    level_tag.debug(L"level_tag");
  }
  
  // set symbol table and all other symbols
  // including : non_speech, exclude_symbols,
  //

  // find the number of symbols in the symbol table
  // store the symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_SYMBOL_SIZE);
  String symbol_prefix = prefix_a;
  symbol_prefix.concat(IO_LEVEL_SYMBOL);

  // remove the NO_LEFT_CONTEXT and NO_RIGHT_CONTEXT symbols in case
  // these symbols exist in the symbol-table
  //
  long j = 0;
  Vector<SearchSymbol>& symbol_table_t = search_level_a.getSymbolTable();
  long len = symbol_table_t.length();
  Vector<SearchSymbol> symbol_table(len);  

  for( long i = 0; i < len; i++) {
    if (!((symbol_table_t(i).eq(SearchSymbol::NO_LEFT_CONTEXT)) ||
	(symbol_table_t(i).eq(SearchSymbol::NO_RIGHT_CONTEXT)))) {
      symbol_table.setLength(j + 1);
      symbol_table(j) = symbol_table_t(i);
      j++;
    }
  }
  
  symbol_table_t.assign(symbol_table);
  
  storeSymbols(search_level_a.getSymbolTable(), hash_table_a,
	       hash_key, symbol_prefix);

  // find the number of contexts in the contexts table
  // and store the context mapping table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_CONTEXT_SIZE);
  Vector<SearchSymbol> context_table;
  convertContextMapping(search_level_a.getContextMap(), context_table);
  String context_prefix = prefix_a;
  context_prefix.concat(IO_LEVEL_CONTEXT);
  storeSymbols(context_table, hash_table_a, hash_key, context_prefix);  
  
  // find the number of symbols in the non-speech symbol table
  // store the symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_NON_SPEECH_SYMBOL_SIZE);
  symbol_prefix.assign(prefix_a);
  symbol_prefix.concat(IO_LEVEL_NON_SPEECH_SYMBOL);
  storeSymbols(search_level_a.getNonSpeechSymbolTable(), hash_table_a,
	       hash_key, symbol_prefix);

  // find the number of symbols in the dummy symbol table
  // store the symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_DUMMY_SYMBOL_SIZE);
  symbol_prefix.assign(prefix_a);
  symbol_prefix.concat(IO_LEVEL_DUMMY_SYMBOL);
  storeSymbols(search_level_a.getDummySymbolTable(), hash_table_a,
	       hash_key, symbol_prefix);

  // find the number of symbols in the contextLess symbol table
  // store the symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_CONTEXTLESS_SYMBOL_SIZE);
  symbol_prefix.assign(prefix_a);
  symbol_prefix.concat(IO_LEVEL_CONTEXTLESS_SYMBOL);
  storeSymbols(search_level_a.getContextLessSymbolTable(), hash_table_a,
	       hash_key, symbol_prefix);

  // find the number of symbols in the skip symbol table
  // store the symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_SKIP_SYMBOL_SIZE);
  symbol_prefix.assign(prefix_a);
  symbol_prefix.concat(IO_LEVEL_SKIP_SYMBOL);
  storeSymbols(search_level_a.getSkipSymbolTable(), hash_table_a,
	       hash_key, symbol_prefix);

  // find the number of symbols in the exclude symbol table
  // store the symbol table
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_EXCLUDE_SYMBOL_SIZE);
  symbol_prefix.assign(prefix_a);
  symbol_prefix.concat(IO_LEVEL_EXCLUDE_SYMBOL);
  storeSymbols(search_level_a.getExcludeSymbolTable(), hash_table_a,
	       hash_key, symbol_prefix);

  // find the number of graphs
  //
  long num_of_graphs = search_level_a.getNumSubGraphs();
  String num_graphs;
  num_graphs.assign(num_of_graphs);
  hash_key.assign(prefix_a);
  hash_key.concat(IO_LEVEL_GRAPH_SIZE);
  hash_table_a.insert(hash_key, &num_graphs);
  
  // store graphs for a paticular level
  //
  for (long j = 0; j < num_of_graphs; j++){

   // set the prefix
   //
   String curr_graph_index;
   curr_graph_index.assign((long)j);
   
   String graph_prefix;
   graph_prefix.assign(prefix_a);
   graph_prefix.concat(IO_DIGRAPH_PREFIX);
   graph_prefix.concat(curr_graph_index);

   // store the graph name
   //
   hash_key.assign(prefix_a);
   hash_key.concat(IO_LEVEL_GRAPH_NAME);
   hash_key.concat(curr_graph_index);
   hash_table_a.insert(hash_key, &graph_name_a(j));

   // debug information
   //
   if (debug_level >= Integral::BRIEF) {
     String msg;
     msg.assign(hash_key);
     msg.concat(L"=");
     msg.concat(graph_name_a(j));
     Console::put(msg);
   }
  
   // read the graph
   //
   storeSubGraph(search_level_a, search_level_a.getSubGraph(j),
		 hash_table_a, graph_prefix);
  }
  
  // exit gracefully
  //
  return true;
  
}

// method: storeSubGraph
//
// arguments:
//  DiGraph<SearchNode>&: (output) construct this DiGraph
//  HashTable<String, String>&: (intput) the hash table which holds the data
//  String& prefix_a: (input) the prefix
//
// return: boolean value indicating status
//
// this method reads the data into a DiGraph<SearchNode>
//
boolean storeSubGraph(SearchLevel& search_level_a,
		     DiGraph<SearchNode>& sub_graph_a,
		     HashTable<String, String>& hash_table_a,
		     String& prefix_a) {
  // local variable
  //
  String hash_key;

  // find the length of the digraph , number of vertices
  //
  String num_of_vertices;
  num_of_vertices.assign(sub_graph_a.length()+2);

  // get the number of vertices
  //
  hash_key.assign(prefix_a);
  hash_key.concat(IO_DIGRAPH_VERTEX_SIZE);
  hash_table_a.insert(hash_key, &num_of_vertices);

  // create local variables
  //
  Vector<Long> y_index;
  long x_index = 0;
  long num_of_arcs = 0;

  // call store vertex recursively
  //
  sub_graph_a.makeColorHash();
  sub_graph_a.setColor(Integral::WHITE);
  storeVertex(*sub_graph_a.getStart(),
	      sub_graph_a,
	      hash_table_a,
	      num_of_arcs,
	      y_index,
	      x_index,
	      prefix_a);

  // set the position for the term
  //
  x_index = y_index.length();
  storeVertex(*sub_graph_a.getTerm(),
	      sub_graph_a,
	      hash_table_a,
	      num_of_arcs,
	      y_index,
	      x_index,
	      prefix_a);
  
  // store the the num_of_arcs
  //
  String num_arcs;
  num_arcs.assign(num_of_arcs);
  hash_key.assign(prefix_a);
  hash_key.concat(IO_DIGRAPH_ARC_SIZE);
  hash_table_a.insert(hash_key, &num_arcs);  
  
  // exit gracefully
  //
  return true;
  
}

// method: storeVertices
//
// arguments:
//  DiGraph<SearchNode>&: (output) construct this DiGraph
//  SingleLinkedList<GraphVertex<SearchNode>>& vertices_list_a:
//                  (input) hold the pointers of Vertices
//  HashTable<String, String>&: (intput) the hash table which holds the data
//  String& prefix_a: (input) the prefix
//
// return: boolean value indicating status
//
// this method reads store vertices of a DiGraph into a hash_table
//
boolean storeVertex(GraphVertex<SearchNode>& vertex_a,
		    DiGraph<SearchNode>& sub_graph_a,
		    HashTable<String, String>& hash_table_a,
		    long& num_of_arcs_a,
		    Vector<Long>& y_index_a,
		    long x_index_a,
		    String& prefix_a) {
  
  // local variable
  //
  long vertex_index = indexOf(sub_graph_a, &vertex_a);
  long from_index = vertex_index;
  long to_index = 0;
  String hash_key;
  SearchSymbol symbol_name;
  long vertex_type = 0;
  String num_of_symbols(L"0");
  
  // set the vertex prefix
  //
  String vertex_prefix;
  vertex_prefix.assign(prefix_a);
  vertex_prefix.concat(IO_VERTEX_PREFIX);
  vertex_prefix.concat(vertex_index);
      
  // store the vertex type
  //
  if ( vertex_a.isStart() ){
    vertex_type = START_VERTEX;
    symbol_name.assign(START_VERTEX_NAME);
  }
  else if ( vertex_a.isTerm() ){
    vertex_type = TERM_VERTEX;
    symbol_name.assign(TERM_VERTEX_NAME);
  }
  else {
    vertex_type = NODE_VERTEX;    
    vertex_a.getItem()->getSymbol(symbol_name);
    
    // store the symbol, only one symbol for each vertex
    //
    hash_key.assign(vertex_prefix);
    hash_key.concat(IO_VERTEX_SYMBOL);
    hash_key.concat(L"0");
    hash_table_a.insert(hash_key, &symbol_name);

    // set the number of symbols, only one symbol on each node
    //
    num_of_symbols.assign(L"1");
  }

  // set the symbol size ( number of symbols on ths vertex )
  //
  hash_key.assign(vertex_prefix);
  hash_key.concat(IO_VERTEX_SYMBOL_SIZE);
  hash_table_a.insert(hash_key, &num_of_symbols);

  // set the vertex name
  //
  hash_key.assign(vertex_prefix);
  hash_key.concat(IO_VERTEX_NAME);
  hash_table_a.insert(hash_key, &symbol_name);

  // store the vertex type
  //
  String type;
  type.assign(vertex_type);
  hash_key.assign(vertex_prefix);
  hash_key.concat(IO_VERTEX_TYPE);
  hash_table_a.insert(hash_key, &type);
  
  // set the width
  //
  hash_key.assign(vertex_prefix);
  hash_key.concat(IO_VERTEX_WIDTH);
  String width;
  width.assign((long)VERTEX_WIDTH);
  hash_table_a.insert(hash_key, &width);

  // set the height
  //
  hash_key.assign(vertex_prefix);
  hash_key.concat(IO_VERTEX_HEIGHT);
  String height;
  height.assign((long)VERTEX_HEIGHT);
  hash_table_a.insert(hash_key, &height);

  // set the location and width and height
  //
  long x_pos = x_index_a;
  long y_pos = 0;

  // set the positions
  //
  if ( x_pos == y_index_a.length() ){
    Long y(0);
    y_index_a.concat(y);
  }else if ( x_pos < y_index_a.length() ){
    y_pos = y_index_a(x_pos);
  }
  else {
    return Error::handle(L"storeVertices", L"invalide x_pos",
			 Error::ARG, __FILE__, __LINE__);
  }

  // compute the x, y position
  //
  String x_loc;
  String y_loc;
  x_loc.assign(VERTEX_INITIAL_X + VERTEX_OFFSET_X * x_pos);
  y_loc.assign(VERTEX_INITIAL_Y + VERTEX_OFFSET_Y * y_pos);
  
  // set the x position
  //
  hash_key.assign(vertex_prefix);
  hash_key.concat(IO_VERTEX_LOC_X);
  hash_table_a.insert(hash_key, &x_loc);

  // set the y position
  //
  hash_key.assign(vertex_prefix);
  hash_key.concat(IO_VERTEX_LOC_Y);
  hash_table_a.insert(hash_key, &y_loc);

  // increase the y index
  //
  y_index_a(x_pos) += 1;

  // set the color of this vertex to black
  //
  // store the vertex type
  //
  if ( (!vertex_a.isStart()) && (!vertex_a.isTerm())){
    sub_graph_a.setColor(&vertex_a, Integral::BLACK);
  }

  // loop over all arcs starting from this node
  //
  for (boolean more = vertex_a.gotoFirst(); more; more = vertex_a.gotoNext()) {

    // if this vertex is white, expand it
    //
    GraphArc<SearchNode>* tmp_arc = vertex_a.getCurr();
    GraphVertex<SearchNode>* vertex = tmp_arc->getVertex();
    to_index = indexOf(sub_graph_a, vertex);

    // store the arc
    //
    storeArc(num_of_arcs_a, from_index, to_index, tmp_arc,
	     hash_table_a, prefix_a);

    // error detecting
    //
    if ( vertex->isStart() ){
      return Error::handle(L"storeVertices", L"loop back to the Start vertex",
			   Error::ARG, __FILE__, __LINE__);
    }
    
    // store vertex
    //
    if ( (!vertex->isStart()) && (!vertex->isTerm())){
      if (sub_graph_a.getColor(vertex) != Integral::BLACK) {
	storeVertex(*vertex, sub_graph_a,hash_table_a, num_of_arcs_a,
		    y_index_a, x_index_a+1, prefix_a);
      }
    }
  }

  // exit gracefully
  //
  return true;
  
}

// method: storeArc
//
// arguments:
//  DiGraph<SearchNode>&: (output) construct this DiGraph
//  SingleLinkedList<GraphVertex<SearchNode>>& vertices_list_a:
//                  (output) hold the pointers to vertices
//  HashTable<String, String>&: (intput) the hash table which holds the data
//  String& prefix_a: (input) the prefix
//
// return: boolean value indicating status
//
// this method reads the data into a DiGraph and store pointers of vertices
//
boolean storeArc(long& arc_index_a,
		 long from_index_a,
		 long to_index_a,
		 GraphArc<SearchNode>*& arc_a,
		 HashTable<String, String>& hash_table_a,
		 String& prefix_a) {
    
  // local variable
  //
  String hash_key;
  String arc_prefix;

  // set the arc prefix
  //
  arc_prefix.assign(prefix_a);
  arc_prefix.concat(IO_ARC_PREFIX);
  arc_prefix.concat(arc_index_a);

  // store the from_id
  //
  String from_id;
  from_id.assign(from_index_a);
  hash_key.assign(arc_prefix);
  hash_key.concat(IO_ARC_FROM_ID);
  hash_table_a.insert(hash_key, &from_id);

  // store the to_id
  //
  String to_id;
  to_id.assign(to_index_a);
  hash_key.assign(arc_prefix);
  hash_key.concat(IO_ARC_TO_ID);
  hash_table_a.insert(hash_key, &to_id);

  // store width
  //
  String width(L"0");
  to_id.assign(to_index_a);
  hash_key.assign(arc_prefix);
  hash_key.concat(IO_ARC_WIDTH);
  hash_table_a.insert(hash_key, &width);

  // store height
  //
  String height(L"0");
  to_id.assign(to_index_a);
  hash_key.assign(arc_prefix);
  hash_key.concat(IO_ARC_HEIGHT);
  hash_table_a.insert(hash_key, &height);

  // store loc_x
  //
  String loc_x(L"0");
  to_id.assign(to_index_a);
  hash_key.assign(arc_prefix);
  hash_key.concat(IO_ARC_LOC_X);
  hash_table_a.insert(hash_key, &loc_x);

  // store loc_y
  //
  String loc_y(L"0");
  to_id.assign(to_index_a);
  hash_key.assign(arc_prefix);
  hash_key.concat(IO_ARC_LOC_Y);
  hash_table_a.insert(hash_key, &loc_y);
  
  // get the epsilon flag
  //
  String is_epsilon;
  if ( arc_a->getEpsilon() ){
    is_epsilon.assign(L"true");
  }
  else {
    is_epsilon.assign(L"false");
  }
  hash_key.assign(arc_prefix);
  hash_key.concat(IO_ARC_EPSILON);
  hash_table_a.insert(hash_key, &is_epsilon);

  // get the transition
  //
  String weight;
  weight.assign(arc_a->getWeight());
  hash_key.assign(arc_prefix);
  hash_key.concat(IO_ARC_WEIGHTS);
  hash_table_a.insert(hash_key, &weight);  

  // increase the arc index
  //
  arc_index_a++;
  
  // exit gracefully
  //
  return true;
  
}

// method: storeSymbols
//
// arguments:
//  Vector<SearchSymbol>& Symbols_list_a:
//                  (input) hold the pointers of symbols
//  long : (input) total number of symbols to read
//  HashTable<String, String>&: (intput) the hash table which holds the data
//  String& prefix_a: (input) the prefix
//
// return: boolean value indicating status
//
// this method store symbols into a hash table
//
boolean storeSymbols(Vector<SearchSymbol>& symbol_list_a,
		    HashTable<String, String>& hash_table_a,
		     String size_prefix_a,
		    String& prefix_a) {
  // local variable
  //
  long num_of_symbols = symbol_list_a.length();
  String num_symbols;
  num_symbols.assign(num_of_symbols);
  hash_table_a.insert(size_prefix_a, &num_symbols);

  // read each symbol
  //
  for (long j = 0; j < symbol_list_a.length(); j++){

   // set the hash_key
   //
   String curr_symbol;
   curr_symbol.assign((long)j);
   
   String hash_key;
   hash_key.assign(prefix_a);
   hash_key.concat(curr_symbol);

   // put the symbol into the symbol table
   //
   hash_table_a.insert(hash_key, &symbol_list_a(j));

  }

  // exit gracefully
  //
  return true;
  
}

// method: indexOf
//
// arguments:
//  Vector<SearchSymbol>& Symbols_list_a:
//                  (input) hold the pointers of symbols
//  long : (input) total number of symbols to read
//  HashTable<String, String>&: (intput) the hash table which holds the data
//  String& prefix_a: (input) the prefix
//
// return: boolean value indicating status
//
// this method find the index of a vertex in DiGraph
//
long indexOf(DiGraph<SearchNode>& sub_graph_a,
		GraphVertex<SearchNode>* vertex_a) {

  // special case
  //
  long size = sub_graph_a.length();
  
  if ( vertex_a->isStart() ){
    return size;
  }
  if ( vertex_a->isTerm() ){
    return size+1;
  }
  
  // search from the beginning for the item
  //
  long count = 0;
  for (boolean more = sub_graph_a.gotoFirst();
       more && ( sub_graph_a.getCurr() != vertex_a);
       more = sub_graph_a.gotoNext()) {
    count++;
  }

  // exit gracefully
  //
  return count;
  
}

// method: writeHashTable
//
// arguments:
//  Vector<SearchSymbol>& Symbols_list_a:
//                  (input) hold the pointers of symbols
//  long : (input) total number of symbols to read
//  HashTable<String, String>&: (intput) the hash table which holds the data
//  String& prefix_a: (input) the prefix
//
// return: boolean value indicating status
//
// this method write the hash table in a format of property=value
//
boolean writeHashTable(HashTable<String, String>& hash_table_a,
		       String & output_a,
		       String  description_a ) {

  // local variable
  //
  output_a.assign(L"# ");
  output_a.concat(description_a);

  // get the keys
  //
  Vector<String> keys;
  Vector<String> values;
  hash_table_a.keys(keys);
  hash_table_a.values(values);

  // put all values
  //
  for ( int i = 0 ; i < keys.length(); i ++ ){
    output_a.concat(L"\n");
    output_a.concat(keys(i));
    output_a.concat(L"=");
    //output_a.concat(*hash_table_a.get(keys(i)));
    output_a.concat(values(i));
  }

  // exit gracefully
  //
  return true;
  
}

// method: writeHashTable
//
// arguments:
//  Vector<SearchSymbol>& Symbols_list_a:
//                  (input) hold the pointers of symbols
//  long : (input) total number of symbols to read
//  HashTable<String, String>&: (intput) the hash table which holds the data
//  String& prefix_a: (input) the prefix
//
// return: boolean value indicating status
//
// this method write the hash table in a format of property=value
//
boolean writeHashTable(HashTable<String, String>& hash_table_a,
		       File & output_a,
		       String  description_a ) {

  // local variable
  //
  String output;
  output.assign(L"# ");
  output.concat(description_a);
  output.concat(L"\n");
  output_a.put(output);

  // get the keys
  //
  Vector<String> keys;
  Vector<String> values;
  hash_table_a.keys(keys);
  hash_table_a.values(values);

  for ( int i = 0 ; i < keys.length(); i ++ ){
    output.assign(keys(i));
    output.concat(L"=");
    output.concat(values(i));
    output.concat(L"\n");
    output_a.put(output);
  }  
  // exit gracefully
  //
  return true;
  
}

// method: storeContextMapping
//
// arguments:
//
// return: a boolean value indicating status
//
// convert the context map to a vector of search symbols
// 
//
boolean convertContextMapping(Vector<ContextMap>& context_map_a,
			      Vector<SearchSymbol>& context_symbol_a){
  
  // local variable
  //
  long len = context_map_a.length();
  context_symbol_a.setCapacity(len);
  context_symbol_a.setLength(len);
  
  // loop over all the context_maps and parse string in each loop
  //
  for (long k = 0; k < len; k++) {
    
    // local variable
    //
    String rule_name;
    SearchSymbol context_symbol;
    Vector<SearchSymbol> context;
    Ulong index;
    
    // get the context and the context-index from the current
    // context-map
    //
    context = context_map_a(k).getContext();
    index = context_map_a(k).getContextIndex();
    
    // set the rule name
    //
    for (long i = 0; i < context.length(); i++){
      
      // read the rule name
      //
      context_symbol.concat(context(i));
      if ( i != (context.length() - 1)){
	context_symbol.concat(L"-");
      }
    }

    // set the context list
    //
    context_symbol_a(index).assign(context_symbol);
  }
  
  // exit gracefully
  //
  return true;
}

// method: convertContextMapping
//
// arguments:
//
// return: a boolean value indicating status
//
// convert the context map to a vector of search symbols
// 
//
boolean convertContextMapping(Vector<SearchSymbol>& context_symbol_a,
			      Vector<ContextMap>& context_map_a){
  
  // local variable
  //
  long len = context_symbol_a.length();
  context_map_a.setCapacity(len);
  context_map_a.setLength(len);
  
  // loop over all the context_maps and parse string in each loop
  //
  for (long k = 0; k < len; k++) {
    
    // local variable
    //
    SearchSymbol context_symbol = context_symbol_a(k);
    SearchSymbol symbol;
    Vector<SearchSymbol> context;

    // get the data string
    //
    long context_length = context_symbol.countTokens(L"-");
    
    // get the context
    //
    long j=0;
    for (long i = 0; i < context_length; i++) {
      context_symbol.tokenize(symbol, j, L"-");
      context.concat(symbol);
    }
    
    // set this context
    //
    if (!context_map_a(k).setContext(context)) {
      return Error::handle(L"convertContextMapping", L"can set context", Error::READ, __FILE__, __LINE__, Error::WARNING);    
    }

    context_map_a(k).setContextIndex(k);
  }
  
  // exit gracefully
  //
  return true;
}
