// file: $isip/util/speech/isip_lm_converter/isip_lm_converter.h
//

// make sure the file is included only once
//
#ifndef ISIP_LM_CONVERTER
#define ISIP_LM_CONVERTER

// isip include files
//
#include <CommandLine.h>
#include <Vector.h>
#include <String.h>
#include <File.h>

// local methods
//
boolean modelToModel(Filename& input_file_name_a,
		     Filename& output_file_name_a,
		     LanguageModel::OUTPUT_FORMAT out_format_a,
		     LanguageModel::OUTPUT_TYPE out_type_a);

boolean nativeToModel(Filename& input_file_name_a,
		      Filename& output_file_name_a,
		      Filename& output_sm_file_a,
		      LanguageModel::OUTPUT_FORMAT out_format_a,
		      LanguageModel::OUTPUT_TYPE out_type_a);

boolean modelToNative(Filename& input_file_name_a,
		      Filename& output_file_name_a);

boolean readHashTable(Filename& input_file_name,
		      HashTable<String, String>& hash_table_a);

boolean readSearchLevel(SearchLevel& search_level_a,
			HashTable<String, String>& hash_table_a,
			String& prefix_a);

boolean readSMSymbols(Vector<SearchSymbol>& sm_symbols_a,
		      HashTable<String, String>& hash_table_a,
		      String& prefix_a);

  boolean readSubGraph(SearchLevel& search_level_a,
		     DiGraph<SearchNode>& sub_graph_a,
		     HashTable<String, String>& hash_table_a,
		     String& prefix_a);

boolean readVertices(SearchLevel& search_level_a,
		     DiGraph<SearchNode>& sub_graph_a,
		     SingleLinkedList<GraphVertex<SearchNode> >&
		     vertices_list_a,
		     HashTable<String, String>& hash_table_a,
		     String& prefix_a);

boolean readArcs(DiGraph<SearchNode>& sub_graph_a,
		 SingleLinkedList<SingleLinkedList<GraphVertex<SearchNode> > >&
		 vertices_list_a,
		 HashTable<String, String>& hash_table_a,
		 String& prefix_a);

boolean readSymbols(Vector<SearchSymbol>& symbol_list_a,
		    HashTable<String, String>& hash_table_a,
		    String& size_prefix_a,
		    String& prefix_a);

boolean storeSearchLevel(SearchLevel& search_level_a,
			HashTable<String, String>& hash_table_a,
			 Vector<SearchSymbol>& graph_name_a,
			 String& prefix_a);

boolean storeSubGraph(SearchLevel& search_level_a,
		     DiGraph<SearchNode>& sub_graph_a,
		     HashTable<String, String>& hash_table_a,
		     String& prefix_a);

boolean storeVertex(GraphVertex<SearchNode>& vertex_a,
		    DiGraph<SearchNode>& sub_graph_a,
		    HashTable<String, String>& hash_table_a,
		    long& num_arcs_a,
		    Vector<Long>& y_index_a,
		    long x_index_a,
		    String& prefix_a);

boolean storeArc(long& arc_index_a,
		 long from_index_a,
		 long to_index_a,
		 GraphArc<SearchNode>*& arc_a,
		 HashTable<String, String>& hash_table_a,
		 String& prefix_a);

boolean storeSymbols(Vector<SearchSymbol>& symbol_list_a,
		    HashTable<String, String>& hash_table_a,
		     String size_prefix_a,
		     String& prefix_a);

long indexOf(DiGraph<SearchNode>& sub_graph_a,
	     GraphVertex<SearchNode>* vertex_a);

boolean writeHashTable(HashTable<String, String>& hash_table_a,
		       String & output_a,
		       String  description_a );

boolean writeHashTable(HashTable<String, String>& hash_table_a,
		       File & output_a,
		       String  description_a );

boolean convertContextMapping(Vector<ContextMap>& context_map_a,
			      Vector<SearchSymbol>& context_symbol_a);
boolean convertContextMapping(Vector<SearchSymbol>& context_symbol_a,
			      Vector<ContextMap>& context_map_a);

// constants
//
static const String PROG_NAME(L"isip_network_converter");
static const String LEVEL_TREE_TOP_GRAMMAR(L"sentence");

static const String START_VERTEX_NAME(L"Start");
static const String TERM_VERTEX_NAME(L"Stop");
static const String NODE_VERTEX_NAME(L"Node");

static const String OPTION_INPUT_FORMAT(L"input_format");
static const String OPTION_OUTPUT_FORMAT(L"output_format");
static const String OPTION_OUTPUT_TYPE(L"output_type");
static const String OPTION_DEBUG_LEVEL(L"debug_level");
static const String EMPTY(String::NULL_STRING);
static const Integral::DEBUG DEBUG_LEVEL_DEFAULT=Integral::NONE;
static DebugLevel debug_level = Integral::NONE;

static const String COMMENT_CHAR(L"#");
static const String DELIMITER_CHAR(L"=");

// all io constants
//
static const long START_VERTEX = 0;
static const long TERM_VERTEX = 1;
static const long NODE_VERTEX = 2;

static const int VERTEX_WIDTH = 70;    
static const int VERTEX_HEIGHT = 48;

static const int VERTEX_INITIAL_X = 100;    
static const int VERTEX_INITIAL_Y = 100;

static const int VERTEX_OFFSET_X = 140;    
static const int VERTEX_OFFSET_Y = 140;


// search level tags
//
static const String IO_CDATA_LEVEL_SIZE(L"level_size");

static const String IO_LEVEL_NAME(L"_name");
static const String IO_LEVEL_GRAPH_SIZE(L"_graph_size");
static const String IO_LEVEL_GRAPH_NAME(L"_graph_name");
static const String IO_LEVEL_SYMBOL_SIZE(L"_symbol_size");
static const String IO_LEVEL_SYMBOL(L"_symbol_");
static const String IO_LEVEL_CONTEXT_SIZE(L"_context_size");
static const String IO_LEVEL_CONTEXT(L"_context_");
static const String IO_LEVEL_NON_SPEECH_SYMBOL_SIZE(L"_non_speech_symbol_size");
static const String IO_LEVEL_NON_SPEECH_SYMBOL(L"_non_speech_symbol_");
static const String IO_LEVEL_CONTEXTLESS_SYMBOL_SIZE(L"_contextless_symbol_size");
static const String IO_LEVEL_CONTEXTLESS_SYMBOL(L"_contextless_symbol_");
static const String IO_LEVEL_DUMMY_SYMBOL(L"_dummy_symbol_");
static const String IO_LEVEL_DUMMY_SYMBOL_SIZE(L"_dummy_symbol_size");
static const String IO_LEVEL_SKIP_SYMBOL(L"_skip_symbol_");
static const String IO_LEVEL_SKIP_SYMBOL_SIZE(L"_skip_symbol_size");
static const String IO_LEVEL_EXCLUDE_SYMBOL(L"_exclude_symbol_");
static const String IO_LEVEL_EXCLUDE_SYMBOL_SIZE(L"_exclude_symbol_size");
static const String IO_LEVEL_SM_SYMBOL(L"_ST_symbol_");
static const String IO_LEVEL_SM_SYMBOL_SIZE(L"_ST_symbol_size");

// graph tags
//
static const String IO_DIGRAPH_VERTEX_SIZE(L"_vertex_size");
static const String IO_DIGRAPH_ARC_SIZE(L"_arc_size");

// vertex tags
//
static const String IO_VERTEX_SYMBOL_SIZE(L"_symbol_size");
static const String IO_VERTEX_NAME(L"_name");
static const String IO_VERTEX_TYPE(L"_type");
static const String IO_VERTEX_WIDTH(L"_width");
static const String IO_VERTEX_HEIGHT(L"_height");
static const String IO_VERTEX_LOC_X(L"_loc_x");
static const String IO_VERTEX_LOC_Y(L"_loc_y");
static const String IO_VERTEX_SYMBOL(L"_S");
    
// arc tags
//
static const String IO_ARC_FROM_ID(L"_from");
static const String IO_ARC_TO_ID(L"_to");
static const String IO_ARC_WEIGHTS(L"_weight");
static const String IO_ARC_EPSILON(L"_epsilon");
static const String IO_ARC_WIDTH(L"_width");
static const String IO_ARC_HEIGHT(L"_height");
static const String IO_ARC_LOC_X(L"_loc_x");
static const String IO_ARC_LOC_Y(L"_loc_y");

// prefix tags
//
static const String IO_SEARCHLEVEL_PREFIX(L"L");
static const String IO_DIGRAPH_PREFIX(L"G");
static const String IO_VERTEX_PREFIX(L"V");
static const String IO_ARC_PREFIX(L"A");

// end of file
//
#endif
